# generated by datamodel-codegen:
#   filename:  openapi.json
#   timestamp: 2022-01-27T19:39:18+00:00

from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import Any, Dict, Iterator, List, Mapping, Optional, Sequence, Union
from uuid import UUID

from pydantic import EmailStr, Extra, Field
from typing_extensions import Literal

from kelvin.sdk.client.data_model import DataModelBase, PaginatorDataModel
from kelvin.sdk.client.utils import instance_classmethod

from . import errors, pagination, requests


class ACPAllocation(DataModelBase):
    """
    ACPAllocation object.

    Parameters
    ----------
    cpu_capacity: Optional[float]
    cpu_requests: Optional[float]
    memory_capacity: Optional[float]
    memory_requests: Optional[float]

    """

    cpu_capacity: Optional[float] = None
    cpu_requests: Optional[float] = None
    memory_capacity: Optional[float] = None
    memory_requests: Optional[float] = None


class ACPCPUUtilization(DataModelBase):
    """
    ACPCPUUtilization object.

    Parameters
    ----------
    timestamp: Optional[int]
    value: Optional[float]

    """

    timestamp: Optional[int] = None
    value: Optional[float] = None


class ACPDisk(DataModelBase):
    """
    ACPDisk object.

    Parameters
    ----------
    total_bytes: Optional[float]
    used_bytes: Optional[float]

    """

    total_bytes: Optional[float] = None
    used_bytes: Optional[float] = None


class ACPEdgeAppVersion(DataModelBase):
    """
    ACPEdgeAppVersion object.

    Parameters
    ----------
    edge_app_version: Optional[str]
    has_updates: Optional[bool]
    platform_version: Optional[str]

    """

    edge_app_version: Optional[str] = None
    has_updates: Optional[bool] = None
    platform_version: Optional[str] = None

    @classmethod
    def force_update_acp_edge_app_version(
        cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Force Update ACP Edge App Version

        **Permission Required:** `kelvin.permission.acp.read`.

        ``forceUpdateACPEdgeAppVersion``: ``POST`` ``/api/v4/acps/{acp_name}/edge-apps/version/force-update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/edge-apps/version/force-update",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_acp_edge_app_version(
        cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> ACPEdgeAppVersion:
        """
        Get ACP Edge App Version

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPEdgeAppVersion``: ``GET`` ``/api/v4/acps/{acp_name}/edge-apps/version/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/edge-apps/version/get",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ACPEdgeAppVersion,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result


class ACPLocationItem(DataModelBase):
    """
    ACPLocationItem object.

    Parameters
    ----------
    lat: Optional[float]
    lon: Optional[float]

    """

    lat: Optional[float] = None
    lon: Optional[float] = None


class ACPMemoryUsage(DataModelBase):
    """
    ACPMemoryUsage object.

    Parameters
    ----------
    timestamp: Optional[int]
    value: Optional[float]

    """

    timestamp: Optional[int] = None
    value: Optional[float] = None


class ACPMetaDataItem(DataModelBase):
    """
    ACPMetaDataItem object.

    Parameters
    ----------
    acp_name: Optional[str]
    created: Optional[datetime]
    name: Optional[str]
    payload: Optional[Dict[str, Any]]
    updated: Optional[datetime]

    """

    acp_name: Optional[str] = None
    created: Optional[datetime] = None
    name: Optional[str] = Field(None, max_length=64)
    payload: Optional[Dict[str, Any]] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_acp_metadata(
        self_or_cls,
        acp_name: str,
        data: Optional[Union[requests.ACPMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPMetadata:
        """
        Create ACP Metadata

        **Permission Required:** `kelvin.permission.acp.create`.

        ``createACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/create``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        data: requests.ACPMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/create",
            {"acp_name": acp_name},
            {},
            {},
            {},
            data,
            requests.ACPMetadataCreate,
            False,
            {"201": ACPMetadata, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_acp_metadata(
        self_or_cls,
        acp_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[ACPMetaDataItem], ACPMetadataListPaginated]:
        """
        List ACP Metadata

        **Pagination Sortable Columns:** `acp_metadata.name`, `acp_metadata.acp_name`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listACPMetadata``: ``GET`` ``/api/v4/acps/{acp_name}/metadata/list``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        search : :obj:`Sequence[str]`
            Search ACP Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/metadata/list",
            {"acp_name": acp_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": ACPMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete ACP Metadata

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/delete``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/delete",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> ACPMetadata:
        """
        Get ACP Metadata

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPMetadata``: ``GET`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/get",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": ACPMetadata, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        data: Optional[Union[requests.ACPMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPMetadata:
        """
        Update ACP Metadata

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name
        data: requests.ACPMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/update",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            data,
            requests.ACPMetadataUpdate,
            False,
            {
                "200": ACPMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class ACPMetadata(DataModelBase):
    """
    ACPMetadata object.

    Parameters
    ----------
    acp_name: Optional[str]
    created: Optional[datetime]
    name: Optional[str]
    payload: Optional[Dict[str, Any]]
    updated: Optional[datetime]

    """

    acp_name: Optional[str] = None
    created: Optional[datetime] = None
    name: Optional[str] = Field(None, max_length=64)
    payload: Optional[Dict[str, Any]] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_acp_metadata(
        self_or_cls,
        acp_name: str,
        data: Optional[Union[requests.ACPMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPMetadata:
        """
        Create ACP Metadata

        **Permission Required:** `kelvin.permission.acp.create`.

        ``createACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/create``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        data: requests.ACPMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/create",
            {"acp_name": acp_name},
            {},
            {},
            {},
            data,
            requests.ACPMetadataCreate,
            False,
            {"201": ACPMetadata, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_acp_metadata(
        self_or_cls,
        acp_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[ACPMetaDataItem], ACPMetadataListPaginated]:
        """
        List ACP Metadata

        **Pagination Sortable Columns:** `acp_metadata.name`, `acp_metadata.acp_name`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listACPMetadata``: ``GET`` ``/api/v4/acps/{acp_name}/metadata/list``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        search : :obj:`Sequence[str]`
            Search ACP Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/metadata/list",
            {"acp_name": acp_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": ACPMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"acp_metadata_name": "name"})
    def delete_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete ACP Metadata

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/delete``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/delete",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_metadata_name": "name"})
    def get_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> ACPMetadata:
        """
        Get ACP Metadata

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPMetadata``: ``GET`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/get",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": ACPMetadata, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_metadata_name": "name"})
    def update_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        data: Optional[Union[requests.ACPMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPMetadata:
        """
        Update ACP Metadata

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name
        data: requests.ACPMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/update",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            data,
            requests.ACPMetadataUpdate,
            False,
            {
                "200": ACPMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class ACPMetadataListPaginated(PaginatorDataModel[ACPMetaDataItem]):
    """
    ACPMetadataListPaginated object.

    Parameters
    ----------
    data: Optional[List[ACPMetaDataItem]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[ACPMetaDataItem]] = None
    pagination: Optional[pagination.Pagination] = None


class ACPNetwork(DataModelBase):
    """
    ACPNetwork object.

    Parameters
    ----------
    total_rx: Optional[float]
    total_tx: Optional[float]

    """

    total_rx: Optional[float] = None
    total_tx: Optional[float] = None


class ACPSettingsAutoUpdate(DataModelBase):
    """
    ACPSettingsAutoUpdate object.

    Parameters
    ----------
    enabled: Optional[bool]
    interval: Optional[int]

    """

    enabled: Optional[bool] = None
    interval: Optional[int] = None


class ACPSettingsMetrics(DataModelBase):
    """
    ACPSettingsMetrics object.

    Parameters
    ----------
    enabled: Optional[bool]
    interval: Optional[int]

    """

    enabled: Optional[bool] = None
    interval: Optional[int] = None


class ACPSettingsStatus(DataModelBase):
    """
    ACPSettingsStatus object.

    Parameters
    ----------
    enabled: Optional[bool]
    interval: Optional[int]

    """

    enabled: Optional[bool] = None
    interval: Optional[int] = None


class ACPStatusCount(DataModelBase):
    """
    ACPStatusCount object.

    Parameters
    ----------
    offline: Optional[int]
    online: Optional[int]
    total: Optional[int]

    """

    offline: Optional[int] = None
    online: Optional[int] = None
    total: Optional[int] = None

    @classmethod
    def get_acp_status_count(cls, _dry_run: bool = False, _client: Any = None) -> ACPStatusCount:
        """
        Get ACP Status Count

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPStatusCount``: ``GET`` ``/api/v4/acps/status/count/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/acps/status/count/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ACPStatusCount,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result


class ACPStatusValues(DataModelBase):
    """
    ACPStatusValues object.

    Parameters
    ----------
    last_seen: Optional[datetime]
    message: Optional[str]
    state: Optional[str]
    warnings: Optional[List[str]]

    """

    last_seen: Optional[datetime] = None
    message: Optional[str] = None
    state: Optional[str] = None
    warnings: Optional[List[str]] = None


class ACPTelemetry(DataModelBase):
    """
    ACPTelemetry object.

    Parameters
    ----------
    allocation: Optional[ACPAllocation]
    cpu_utilization: Optional[List[ACPCPUUtilization]]
    disk: Optional[ACPDisk]
    memory_usage: Optional[List[ACPMemoryUsage]]
    network: Optional[ACPNetwork]

    """

    allocation: Optional[ACPAllocation] = None
    cpu_utilization: Optional[List[ACPCPUUtilization]] = None
    disk: Optional[ACPDisk] = None
    memory_usage: Optional[List[ACPMemoryUsage]] = None
    network: Optional[ACPNetwork] = None

    @classmethod
    def get_acp_telemetry(
        cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> ACPTelemetry:
        """
        Get ACP Telemetry

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPTelemetry``: ``GET`` ``/api/v4/acps/{acp_name}/telemetry/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/telemetry/get",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": ACPTelemetry, "400": ErrorMessages, "401": ErrorMessages, "500": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class ACPWorkloadManifestItem(DataModelBase):
    """
    ACPWorkloadManifestItem object.

    Parameters
    ----------
    action: Optional[str]
    name: Optional[str]
    payload: Optional[Dict[str, Any]]

    """

    action: Optional[str] = None
    name: Optional[str] = None
    payload: Optional[Dict[str, Any]] = None


class ACPWorkloadManifests(DataModelBase):
    """
    ACPWorkloadManifests object.

    Parameters
    ----------
    data: Optional[List[ACPWorkloadManifestItem]]
    names: Optional[List[str]]

    """

    data: Optional[List[ACPWorkloadManifestItem]] = None
    names: Optional[List[str]] = None

    @classmethod
    def get_workloads_manifests(
        cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> ACPWorkloadManifests:
        """
        Get Workloads Manifests assigned to an ACP

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getWorkloadsManifests``: ``GET`` ``/api/v4/acps/{acp_name}/workloads/manifests/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/workloads/manifests/get",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ACPWorkloadManifests,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result


class AlarmAssetStatistics(DataModelBase):
    """
    AlarmAssetStatistics object.

    Parameters
    ----------
    acknowledged: Optional[int]
    active: Optional[int]
    name: Optional[str]
    resolved: Optional[int]

    """

    acknowledged: Optional[int] = None
    active: Optional[int] = None
    name: Optional[str] = None
    resolved: Optional[int] = None


class AlarmAssetsStatistics(DataModelBase):
    """
    AlarmAssetsStatistics object.

    Parameters
    ----------
    data: Optional[List[AlarmAssetStatistics]]

    """

    data: Optional[List[AlarmAssetStatistics]] = None

    @classmethod
    def get_asset_statistics(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> AlarmAssetsStatistics:
        """
        Get Asset Statistics

        **Permission Required:** `kelvin.permission.alarm.read`.

        ``getAssetStatistics``: ``GET`` ``/api/v4/alarms/statistics/assets/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/statistics/assets/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmAssetsStatistics, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class AlarmClass(DataModelBase):
    """
    AlarmClass object.

    Parameters
    ----------
    created: Optional[datetime]
    name: Optional[str]
    owner: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    name: Optional[str] = Field(None, max_length=64)
    owner: Optional[str] = Field(None, max_length=64)
    title: Optional[str] = Field(None, max_length=64)
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_alarm_class(
        self_or_cls,
        data: Optional[Union[requests.AlarmClassCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmClass:
        """
        Create Alarm Class

        **Permission Required:** `kelvin.permission.alarm_class.create`.

        ``createAlarmClass``: ``POST`` ``/api/v4/alarms/class/create``

        Parameters
        ----------
        data: requests.AlarmClassCreate, optional
        **kwargs:
            Extra parameters for requests.AlarmClassCreate
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/class/create",
            {},
            {},
            {},
            {},
            data,
            requests.AlarmClassCreate,
            False,
            {"201": AlarmClass, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_alarm_class(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        owner: Optional[str] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AlarmClass], AlarmClassListPaginated]:
        """
        List Alarm Class

        **Pagination Sortable Columns:** `alarm_class.name`, `alarm_class.title`, `alarm_class.created`, `alarm_class.updated`, `alarm_class.owner`

        **Permission Required:** `kelvin.permission.alarm_class.read`.

        ``listAlarmClass``: ``GET`` ``/api/v4/alarms/class/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search Alarm Class by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        owner : :obj:`str`
            Filter Alarm Class by owner. Possible Values: kelvin/custom

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/class/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "owner": owner,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmClassListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"alarm_class_name": "name"})
    def delete_alarm_class(
        self_or_cls, alarm_class_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Alarm Class

        **Permission Required:** `kelvin.permission.alarm_class.delete`.

        ``deleteAlarmClass``: ``POST`` ``/api/v4/alarms/class/{alarm_class_name}/delete``

        Parameters
        ----------
        alarm_class_name : :obj:`str`, optional
            Alarm Class Name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/class/{alarm_class_name}/delete",
            {"alarm_class_name": alarm_class_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_class_name": "name"})
    def get_alarm_class(
        self_or_cls, alarm_class_name: str, _dry_run: bool = False, _client: Any = None
    ) -> AlarmClass:
        """
        Get Alarm Class

        **Permission Required:** `kelvin.permission.alarm_class.read`.

        ``getAlarmClass``: ``GET`` ``/api/v4/alarms/class/{alarm_class_name}/get``

        Parameters
        ----------
        alarm_class_name : :obj:`str`, optional
            Alarm Class Name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/class/{alarm_class_name}/get",
            {"alarm_class_name": alarm_class_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmClass, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_class_name": "name"})
    def update_alarm_class(
        self_or_cls,
        alarm_class_name: str,
        data: Optional[Union[requests.AlarmClassUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmClass:
        """
        Update Alarm Class

        **Permission Required:** `kelvin.permission.alarm_class.update`.

        ``updateAlarmClass``: ``POST`` ``/api/v4/alarms/class/{alarm_class_name}/update``

        Parameters
        ----------
        alarm_class_name : :obj:`str`, optional
            Alarm Class Name
        data: requests.AlarmClassUpdate, optional
        **kwargs:
            Extra parameters for requests.AlarmClassUpdate
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/class/{alarm_class_name}/update",
            {"alarm_class_name": alarm_class_name},
            {},
            {},
            {},
            data,
            requests.AlarmClassUpdate,
            False,
            {
                "200": AlarmClass,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class AlarmClassListPaginated(PaginatorDataModel[AlarmClass]):
    """
    AlarmClassListPaginated object.

    Parameters
    ----------
    data: Optional[List[AlarmClass]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AlarmClass]] = None
    pagination: Optional[pagination.Pagination] = None


class AlarmFilter(DataModelBase):
    """
    AlarmFilter object.

    Parameters
    ----------
    created: Optional[datetime]
    id: Optional[str]
    payload: Optional[Dict[str, Any]]
    title: Optional[str]
    updated: Optional[datetime]
    username: Optional[str]
    version: Optional[str]

    """

    created: Optional[datetime] = None
    id: Optional[str] = None
    payload: Optional[Dict[str, Any]] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None
    username: Optional[str] = None
    version: Optional[str] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_alarm_filter(
        self_or_cls,
        data: Optional[Union[requests.AlarmFilterCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmFilter:
        """
        Create Alarm Filter

        **Permission Required:** `kelvin.permission.alarm_filter.create`.

        ``createAlarmFilter``: ``POST`` ``/api/v4/alarms/filters/create``

        Parameters
        ----------
        data: requests.AlarmFilterCreate, optional
        **kwargs:
            Extra parameters for requests.AlarmFilterCreate
              - payload: dict
              - title: str
              - username: str
              - version: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/filters/create",
            {},
            {},
            {},
            {},
            data,
            requests.AlarmFilterCreate,
            False,
            {"201": AlarmFilter, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_alarm_filter(
        cls,
        search: Optional[Sequence[str]] = None,
        alarm_class_name: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AlarmFilter], AlarmFilterListPaginated]:
        """
        List Alarm Filter

        **Pagination Sortable Columns:** `alarm_filter.title`, `alarm_filter.username`, `alarm_filter.version`, `alarm_filter.created`, `alarm_filter.updated`

        **Permission Required:** `kelvin.permission.alarm_filter.read`.

        ``listAlarmFilter``: ``GET`` ``/api/v4/alarms/filters/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search alarm filter by title or version
        alarm_class_name : :obj:`Sequence[str]`
            Filter alarm filter by alarm class
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/filters/list",
            {},
            {
                "search": search,
                "alarm_class_name": alarm_class_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmFilterListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"alarm_filter_id": "id"})
    def delete_alarm_filter(
        self_or_cls, alarm_filter_id: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Alarm Filter

        **Permission Required:** `kelvin.permission.alarm_filter.delete`.

        ``deleteAlarmFilter``: ``POST`` ``/api/v4/alarms/filters/{alarm_filter_id}/delete``

        Parameters
        ----------
        alarm_filter_id : :obj:`str`, optional
            Alarm Filter id

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/filters/{alarm_filter_id}/delete",
            {"alarm_filter_id": alarm_filter_id},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_filter_id": "id"})
    def get_alarm_filter(
        self_or_cls,
        alarm_filter_id: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> AlarmFilter:
        """
        Get Alarm Filter

        **Permission Required:** `kelvin.permission.alarm_filter.read`.

        ``getAlarmFilter``: ``GET`` ``/api/v4/alarms/filters/{alarm_filter_id}/get``

        Parameters
        ----------
        alarm_filter_id : :obj:`str`, optional
            Alarm Filter ID
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/filters/{alarm_filter_id}/get",
            {"alarm_filter_id": alarm_filter_id},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmFilter, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_filter_id": "id"})
    def update_alarm_filter(
        self_or_cls,
        alarm_filter_id: str,
        data: Optional[Union[requests.AlarmFilterUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmFilter:
        """
        Update Alarm Filter

        **Permission Required:** `kelvin.permission.alarm_filter.update`.

        ``updateAlarmFilter``: ``POST`` ``/api/v4/alarms/filters/{alarm_filter_id}/update``

        Parameters
        ----------
        alarm_filter_id : :obj:`str`, optional
            Alarm Filter ID
        data: requests.AlarmFilterUpdate, optional
        **kwargs:
            Extra parameters for requests.AlarmFilterUpdate
              - payload: dict
              - title: str
              - version: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/filters/{alarm_filter_id}/update",
            {"alarm_filter_id": alarm_filter_id},
            {},
            {},
            {},
            data,
            requests.AlarmFilterUpdate,
            False,
            {
                "200": AlarmFilter,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class AlarmFilterListPaginated(PaginatorDataModel[AlarmFilter]):
    """
    AlarmFilterListPaginated object.

    Parameters
    ----------
    data: Optional[List[AlarmFilter]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AlarmFilter]] = None
    pagination: Optional[pagination.Pagination] = None


class AlarmType(DataModelBase):
    """
    AlarmType object.

    Parameters
    ----------
    alarm_class_name: Optional[str]
    created: Optional[datetime]
    name: Optional[str]
    owner: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]

    """

    alarm_class_name: Optional[str] = Field(None, max_length=64)
    created: Optional[datetime] = None
    name: Optional[str] = Field(None, max_length=64)
    owner: Optional[str] = Field(None, max_length=64)
    title: Optional[str] = Field(None, max_length=64)
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_alarm_type(
        self_or_cls,
        data: Optional[Union[requests.AlarmTypeCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmType:
        """
        Create Alarm Type

        **Permission Required:** `kelvin.permission.alarm_type.create`.

        ``createAlarmType``: ``POST`` ``/api/v4/alarms/types/create``

        Parameters
        ----------
        data: requests.AlarmTypeCreate, optional
        **kwargs:
            Extra parameters for requests.AlarmTypeCreate
              - alarm_class_name: str
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/types/create",
            {},
            {},
            {},
            {},
            data,
            requests.AlarmTypeCreate,
            False,
            {"201": AlarmType, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_alarm_type(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        alarm_class_name: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        owner: Optional[str] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AlarmType], AlarmTypeListPaginated]:
        """
        List Alarm Type

        **Pagination Sortable Columns:** `alarm_type.name`, `alarm_type.title`, `alarm_type.created`, `alarm_type.updated`, `alarm_class.name`, `alarm_class.title`, `alarm_type.owner`

        **Permission Required:** `kelvin.permission.alarm_type.read`.

        ``listAlarmType``: ``GET`` ``/api/v4/alarms/types/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search alarm type by name or title
        alarm_class_name : :obj:`Sequence[str]`
            Filter alarm type by alarm class
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        owner : :obj:`str`
            Filter Alarm Type by owner. Possible Values: kelvin/custom

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/types/list",
            {},
            {
                "search": search,
                "alarm_class_name": alarm_class_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "owner": owner,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmTypeListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"alarm_type_name": "name"})
    def delete_alarm_type(
        self_or_cls, alarm_type_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Alarm Type

        **Permission Required:** `kelvin.permission.alarm_type.delete`.

        ``deleteAlarmType``: ``POST`` ``/api/v4/alarms/types/{alarm_type_name}/delete``

        Parameters
        ----------
        alarm_type_name : :obj:`str`, optional
            Alarm Type name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/types/{alarm_type_name}/delete",
            {"alarm_type_name": alarm_type_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_type_name": "name"})
    def get_alarm_type(
        self_or_cls, alarm_type_name: str, _dry_run: bool = False, _client: Any = None
    ) -> AlarmType:
        """
        Get Alarm Type

        **Permission Required:** `kelvin.permission.alarm_type.read`.

        ``getAlarmType``: ``GET`` ``/api/v4/alarms/types/{alarm_type_name}/get``

        Parameters
        ----------
        alarm_type_name : :obj:`str`, optional
            Alarm Type name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/types/{alarm_type_name}/get",
            {"alarm_type_name": alarm_type_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmType, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_type_name": "name"})
    def update_alarm_type(
        self_or_cls,
        alarm_type_name: str,
        data: Optional[Union[requests.AlarmTypeUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmType:
        """
        Update Alarm Type

        **Permission Required:** `kelvin.permission.alarm_type.update`.

        ``updateAlarmType``: ``POST`` ``/api/v4/alarms/types/{alarm_type_name}/update``

        Parameters
        ----------
        alarm_type_name : :obj:`str`, optional
            Alarm Type name
        data: requests.AlarmTypeUpdate, optional
        **kwargs:
            Extra parameters for requests.AlarmTypeUpdate
              - alarm_class_name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/types/{alarm_type_name}/update",
            {"alarm_type_name": alarm_type_name},
            {},
            {},
            {},
            data,
            requests.AlarmTypeUpdate,
            False,
            {
                "200": AlarmType,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class AlarmTypeListPaginated(PaginatorDataModel[AlarmType]):
    """
    AlarmTypeListPaginated object.

    Parameters
    ----------
    data: Optional[List[AlarmType]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AlarmType]] = None
    pagination: Optional[pagination.Pagination] = None


class AppVersion(DataModelBase):
    """
    AppVersion object.

    Parameters
    ----------
    created: Optional[datetime]
    id: Optional[str]
    payload: Optional[Dict[str, Any]]
    updated: Optional[datetime]
    version: Optional[str]

    """

    created: Optional[datetime] = None
    id: Optional[str] = None
    payload: Optional[Dict[str, Any]] = None
    updated: Optional[datetime] = None
    version: Optional[str] = None

    @classmethod
    def delete_app_version(
        cls, app_name: str, app_version: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete App Version

        **Note:** This operation also deletes the associated Docker Image

        **Permission Required:** `kelvin.permission.appregistry.delete`.

        ``deleteAppVersion``: ``POST`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/delete``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/delete",
            {"app_name": app_name, "app_version": app_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def download_app_version(
        cls, app_name: str, app_version: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Download App Version


        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``downloadAppVersion``: ``GET`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/download``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/download",
            {"app_name": app_name, "app_version": app_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_app_version(
        cls, app_name: str, app_version: str, _dry_run: bool = False, _client: Any = None
    ) -> AppVersion:
        """
        Get App Version

        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``getAppVersion``: ``GET`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/get``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/get",
            {"app_name": app_name, "app_version": app_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AppVersion, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_app_version_status(
        cls, app_name: str, app_version: str, _dry_run: bool = False, _client: Any = None
    ) -> AppVersionStatus:
        """
        Get App Version Status

        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``getAppVersionStatus``: ``GET`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/status/get``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/status/get",
            {"app_name": app_name, "app_version": app_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AppVersionStatus, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def upload_app_version(
        cls, app_name: str, app_version: str, file: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Upload App Version


        **Permission Required:** `kelvin.permission.appregistry.create`.

        ``uploadAppVersion``: ``POST`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/upload``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version
        file : :obj:`str`, optional
            File to upload

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/upload",
            {"app_name": app_name, "app_version": app_version},
            {},
            {"file": file},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class AppVersionStatus(DataModelBase):
    """
    AppVersionStatus object.

    Parameters
    ----------
    build_log: Optional[str]
    id: Optional[str]
    name: Optional[str]
    status: Optional[str]
    version: Optional[str]

    """

    build_log: Optional[str] = None
    id: Optional[str] = None
    name: Optional[str] = None
    status: Optional[str] = None
    version: Optional[str] = None

    @classmethod
    def list_app_version_status(
        cls,
        app_versions: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AppVersionStatus], AppVersionStatusListPaginated]:
        """
        List App Version Status

        **Pagination Sortable Columns:** `app_version.id`

        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``listAppVersionStatus``: ``GET`` ``/api/v4/appregistry/versions/status/list``

        Parameters
        ----------
        app_versions : :obj:`Sequence[str]`
            Filter by App Versions. Pattern: `<app_name>:<version>`
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/versions/status/list",
            {},
            {
                "app_versions": app_versions,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AppVersionStatusListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def get_app_version_status(
        cls, app_name: str, app_version: str, _dry_run: bool = False, _client: Any = None
    ) -> AppVersionStatus:
        """
        Get App Version Status

        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``getAppVersionStatus``: ``GET`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/status/get``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/status/get",
            {"app_name": app_name, "app_version": app_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AppVersionStatus, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class AppVersionStatusListPaginated(PaginatorDataModel[AppVersionStatus]):
    """
    AppVersionStatusListPaginated object.

    Parameters
    ----------
    data: Optional[List[AppVersionStatus]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AppVersionStatus]] = None
    pagination: Optional[pagination.Pagination] = None


class AssetClass(DataModelBase):
    """
    AssetClass object.

    Parameters
    ----------
    created: Optional[datetime]
    device_type_name: Optional[str]
    device_type_title: Optional[str]
    name: Optional[str]
    owner: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    device_type_name: Optional[str] = Field(None, max_length=64)
    device_type_title: Optional[str] = Field(None, max_length=64)
    name: Optional[str] = Field(None, max_length=64)
    owner: Optional[str] = Field(None, max_length=64)
    title: Optional[str] = Field(None, max_length=64)
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_class(
        self_or_cls,
        data: Optional[Union[requests.AssetClassCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetClass:
        """
        Create Asset Class

        **Permission Required:** `kelvin.permission.asset_class.create`.

        ``createAssetClass``: ``POST`` ``/api/v4/assets/class/create``

        Parameters
        ----------
        data: requests.AssetClassCreate, optional
        **kwargs:
            Extra parameters for requests.AssetClassCreate
              - device_type_name: str
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/class/create",
            {},
            {},
            {},
            {},
            data,
            requests.AssetClassCreate,
            False,
            {"201": AssetClass, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset_class(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        owner: Optional[str] = None,
        device_type_name: Optional[Sequence[str]] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetClass], AssetClassListPaginated]:
        """
        List Asset Class

        **Pagination Sortable Columns:** `asset_class.name`, `asset_class.title`, `asset_class.created`, `asset_class.updated`, `asset_class.owner`

        **Permission Required:** `kelvin.permission.asset_class.read`.

        ``listAssetClass``: ``GET`` ``/api/v4/assets/class/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search Asset Class by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        owner : :obj:`str`
            Filter Asset Class by owner. Possible Values: kelvin/custom
        device_type_name : :obj:`Sequence[str]`
            Filter by Asset Class' Device Type Name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/class/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "owner": owner,
                "device_type_name": device_type_name,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetClassListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"asset_class_name": "name"})
    def delete_asset_class(
        self_or_cls, asset_class_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset Class

        **Permission Required:** `kelvin.permission.asset_class.delete`.

        ``deleteAssetClass``: ``POST`` ``/api/v4/assets/class/{asset_class_name}/delete``

        Parameters
        ----------
        asset_class_name : :obj:`str`, optional
            Asset Class Name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/class/{asset_class_name}/delete",
            {"asset_class_name": asset_class_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_class_name": "name"})
    def get_asset_class(
        self_or_cls, asset_class_name: str, _dry_run: bool = False, _client: Any = None
    ) -> AssetClass:
        """
        Get Asset Class

        **Permission Required:** `kelvin.permission.asset_class.read`.

        ``getAssetClass``: ``GET`` ``/api/v4/assets/class/{asset_class_name}/get``

        Parameters
        ----------
        asset_class_name : :obj:`str`, optional
            Asset Class Name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/class/{asset_class_name}/get",
            {"asset_class_name": asset_class_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetClass,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_class_name": "name"})
    def update_asset_class(
        self_or_cls,
        asset_class_name: str,
        data: Optional[Union[requests.AssetClassUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetClass:
        """
        Update Asset Class

        **Permission Required:** `kelvin.permission.asset_class.update`.

        ``updateAssetClass``: ``POST`` ``/api/v4/assets/class/{asset_class_name}/update``

        Parameters
        ----------
        asset_class_name : :obj:`str`, optional
            Asset Class Name
        data: requests.AssetClassUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetClassUpdate
              - device_type_name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/class/{asset_class_name}/update",
            {"asset_class_name": asset_class_name},
            {},
            {},
            {},
            data,
            requests.AssetClassUpdate,
            False,
            {
                "200": AssetClass,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class AssetClassListPaginated(PaginatorDataModel[AssetClass]):
    """
    AssetClassListPaginated object.

    Parameters
    ----------
    data: Optional[List[AssetClass]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AssetClass]] = None
    pagination: Optional[pagination.Pagination] = None


class AssetHierarchyItem(DataModelBase):
    """
    AssetHierarchyItem object.

    Parameters
    ----------
    asset_type_name: Optional[str]
    device_type_name: Optional[str]
    hierarchy: Optional[List[AssetHierarchyItem]]
    name: Optional[str]
    new: Optional[bool]
    status: Optional[str]
    title: Optional[str]

    """

    asset_type_name: Optional[str] = None
    device_type_name: Optional[str] = None
    hierarchy: Optional[List[AssetHierarchyItem]] = None
    name: Optional[str] = None
    new: Optional[bool] = None
    status: Optional[str] = None
    title: Optional[str] = None


class AssetHierarchyList(DataModelBase):
    """
    AssetHierarchyList object.

    Parameters
    ----------
    data: Optional[List[AssetHierarchyItem]]

    """

    data: Optional[List[AssetHierarchyItem]] = None

    @classmethod
    def upload_assets(
        cls, file: str, dry_run: Optional[bool] = None, _dry_run: bool = False, _client: Any = None
    ) -> AssetHierarchyList:
        """
        Upload Assets

        **Permission Required:** `kelvin.permission.asset.create`.

        ``uploadAssets``: ``POST`` ``/api/v4/assets/create/upload``

        Parameters
        ----------
        file : :obj:`str`, optional
            CSV File
        dry_run : :obj:`bool`
            If set to true performs only validation

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/assets/create/upload",
            {},
            {"dry_run": dry_run},
            {"file": file},
            {},
            None,
            None,
            False,
            {
                "200": AssetHierarchyList,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "412": CSVErrorMessage,
            },
            False,
            _dry_run,
        )
        return result


class AssetLocationItem(DataModelBase):
    """
    AssetLocationItem object.

    Parameters
    ----------
    lat: Optional[float]
    lon: Optional[float]

    """

    lat: Optional[float] = None
    lon: Optional[float] = None


class AssetProperty(DataModelBase):
    """
    AssetProperty object.

    Parameters
    ----------
    name: Optional[str]
    title: Optional[str]
    value: Optional[str]

    """

    name: Optional[str] = Field(None, max_length=64)
    title: Optional[str] = None
    value: Optional[str] = None


class AssetStatusCount(DataModelBase):
    """
    AssetStatusCount object.

    Parameters
    ----------
    offline: Optional[int]
    online: Optional[int]
    partially_online: Optional[int]
    total: Optional[int]
    unknown: Optional[int]

    """

    offline: Optional[int] = None
    online: Optional[int] = None
    partially_online: Optional[int] = None
    total: Optional[int] = None
    unknown: Optional[int] = None

    @classmethod
    def get_asset_status_count(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> AssetStatusCount:
        """
        Get Asset Status Count

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetStatusCount``: ``GET`` ``/api/v4/assets/status/count/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/status/count/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetStatusCount,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result


class AssetStatusItem(DataModelBase):
    """
    AssetStatusItem object.

    Parameters
    ----------
    last_seen: Optional[datetime]
    state: Optional[str]

    """

    last_seen: Optional[datetime] = None
    state: Optional[str] = None


class AssetTemplateProperties(DataModelBase):
    """
    AssetTemplateProperties object.

    Parameters
    ----------
    name: Optional[str]
    title: Optional[str]
    value: Optional[str]

    """

    name: Optional[str] = None
    title: Optional[str] = None
    value: Optional[str] = None


class AssetType(DataModelBase):
    """
    AssetType object.

    Parameters
    ----------
    asset_class_device_type_name: Optional[str]
    asset_class_device_type_title: Optional[str]
    asset_class_name: Optional[str]
    asset_class_title: Optional[str]
    created: Optional[datetime]
    metadata: Optional[Dict[str, Any]]
    name: Optional[str]
    owner: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]

    """

    asset_class_device_type_name: Optional[str] = Field(None, max_length=64)
    asset_class_device_type_title: Optional[str] = Field(None, max_length=64)
    asset_class_name: Optional[str] = Field(None, max_length=64)
    asset_class_title: Optional[str] = Field(None, max_length=64)
    created: Optional[datetime] = None
    metadata: Optional[Dict[str, Any]] = None
    name: Optional[str] = Field(None, max_length=64)
    owner: Optional[str] = Field(None, max_length=64)
    title: Optional[str] = Field(None, max_length=64)
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_type(
        self_or_cls,
        data: Optional[Union[requests.AssetTypeCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetType:
        """
        Create Asset Type

        **Permission Required:** `kelvin.permission.asset_type.create`.

        ``createAssetType``: ``POST`` ``/api/v4/assets/types/create``

        Parameters
        ----------
        data: requests.AssetTypeCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeCreate
              - asset_class_name: str
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/create",
            {},
            {},
            {},
            {},
            data,
            requests.AssetTypeCreate,
            False,
            {"201": AssetType, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset_type(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        asset_class_name: Optional[Sequence[str]] = None,
        metadata: Optional[Sequence[str]] = None,
        metadata_filter: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        owner: Optional[str] = None,
        device_type_name: Optional[Sequence[str]] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetType], AssetTypeListPaginated]:
        """
        List Asset Type

        **Pagination Sortable Columns:** `asset_type.name`, `asset_type.title`, `asset_type.created`, `asset_type.updated`, `asset_class.name`, `asset_class.title`, `asset_type.owner`

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``listAssetType``: ``GET`` ``/api/v4/assets/types/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search asset type by name or title
        asset_class_name : :obj:`Sequence[str]`
            Filter asset type by asset class
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response
        metadata_filter : :obj:`Sequence[str]`
            Dot separated string to filter by metadata values Same <metadata_name>
            will result in OR queries. Different <metadata_name> will result in
            AND queries. Example:
            <metadata_name>.<payload_key_to_search>=<payload_value_to_search>
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        owner : :obj:`str`
            Filter Asset Class by owner. Possible Values: kelvin/custom
        device_type_name : :obj:`Sequence[str]`
            Filter asset type by asset class device type name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/list",
            {},
            {
                "search": search,
                "asset_class_name": asset_class_name,
                "metadata": metadata,
                "metadata_filter": metadata_filter,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "owner": owner,
                "device_type_name": device_type_name,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTypeListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_name": "name"})
    def delete_asset_type(
        self_or_cls, asset_type_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset Type

        **Permission Required:** `kelvin.permission.asset_type.delete`.

        ``deleteAssetType``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/delete``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/delete",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_name": "name"})
    def get_asset_type(
        self_or_cls,
        asset_type_name: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> AssetType:
        """
        Get Asset Type

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``getAssetType``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/get``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/get",
            {"asset_type_name": asset_type_name},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {"200": AssetType, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_name": "name"})
    def create_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        data: Optional[Union[requests.AssetTypeMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTypeMetadata:
        """
        Create Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.create`.

        ``createAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/create``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        data: requests.AssetTypeMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/create",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            data,
            requests.AssetTypeMetadataCreate,
            False,
            {
                "201": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_name": "name"})
    def list_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetTypeMetadata], AssetTypeMetadataListPaginated]:
        """
        List Asset Type Metadata

        **Pagination Sortable Columns:** `asset_type_metadata.name`

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``listAssetTypeMetadata``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/metadata/list``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        search : :obj:`Sequence[str]`
            Search Asset Type Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/metadata/list",
            {"asset_type_name": asset_type_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTypeMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_name": "name"})
    def delete_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.delete`.

        ``deleteAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/delete``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/delete",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_name": "name"})
    def get_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> AssetTypeMetadata:
        """
        Get Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``getAssetTypeMetadata``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/get``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/get",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_name": "name"})
    def update_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        data: Optional[Union[requests.AssetTypeMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTypeMetadata:
        """
        Update Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.update`.

        ``updateAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/update``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name
        data: requests.AssetTypeMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/update",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            data,
            requests.AssetTypeMetadataUpdate,
            False,
            {
                "200": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_name": "name"})
    def update_asset_type(
        self_or_cls,
        asset_type_name: str,
        data: Optional[Union[requests.AssetTypeUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetType:
        """
        Update Asset Type

        **Permission Required:** `kelvin.permission.asset_type.update`.

        ``updateAssetType``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/update``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        data: requests.AssetTypeUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeUpdate
              - asset_class_name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/update",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            data,
            requests.AssetTypeUpdate,
            False,
            {
                "200": AssetType,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class AssetTypeListPaginated(PaginatorDataModel[AssetType]):
    """
    AssetTypeListPaginated object.

    Parameters
    ----------
    data: Optional[List[AssetType]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AssetType]] = None
    pagination: Optional[pagination.Pagination] = None


class AssetTypeMetadata(DataModelBase):
    """
    AssetTypeMetadata object.

    Parameters
    ----------
    asset_type_name: Optional[str]
    created: Optional[datetime]
    name: Optional[str]
    payload: Optional[Dict[str, Any]]
    updated: Optional[datetime]

    """

    asset_type_name: Optional[str] = None
    created: Optional[datetime] = None
    name: Optional[str] = Field(None, max_length=64)
    payload: Optional[Dict[str, Any]] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        data: Optional[Union[requests.AssetTypeMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTypeMetadata:
        """
        Create Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.create`.

        ``createAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/create``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        data: requests.AssetTypeMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/create",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            data,
            requests.AssetTypeMetadataCreate,
            False,
            {
                "201": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetTypeMetadata], AssetTypeMetadataListPaginated]:
        """
        List Asset Type Metadata

        **Pagination Sortable Columns:** `asset_type_metadata.name`

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``listAssetTypeMetadata``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/metadata/list``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        search : :obj:`Sequence[str]`
            Search Asset Type Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/metadata/list",
            {"asset_type_name": asset_type_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTypeMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_metadata_name": "name"})
    def delete_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.delete`.

        ``deleteAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/delete``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/delete",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_metadata_name": "name"})
    def get_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> AssetTypeMetadata:
        """
        Get Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``getAssetTypeMetadata``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/get``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/get",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_type_metadata_name": "name"})
    def update_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        data: Optional[Union[requests.AssetTypeMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTypeMetadata:
        """
        Update Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.update`.

        ``updateAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/update``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name
        data: requests.AssetTypeMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/update",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            data,
            requests.AssetTypeMetadataUpdate,
            False,
            {
                "200": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class AssetTypeMetadataListPaginated(PaginatorDataModel[AssetTypeMetadata]):
    """
    AssetTypeMetadataListPaginated object.

    Parameters
    ----------
    data: Optional[List[AssetTypeMetadata]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AssetTypeMetadata]] = None
    pagination: Optional[pagination.Pagination] = None


class AuditLogger(DataModelBase):
    """
    AuditLogger object.

    Parameters
    ----------
    action: Optional[str]
    created: Optional[datetime]
    id: Optional[int]
    identifier: Optional[str]
    meta: Optional[Dict[str, Any]]
    namespace: Optional[str]
    request_id: Optional[str]
    user_id: Optional[str]
    username: Optional[str]

    """

    action: Optional[str] = None
    created: Optional[datetime] = None
    id: Optional[int] = None
    identifier: Optional[str] = None
    meta: Optional[Dict[str, Any]] = None
    namespace: Optional[str] = None
    request_id: Optional[str] = None
    user_id: Optional[str] = None
    username: Optional[str] = None

    @instance_classmethod
    @DataModelBase.translate()
    def list_audit_log(
        self_or_cls,
        action: Optional[Sequence[str]] = None,
        user_id: Optional[Sequence[str]] = None,
        namespace: Optional[Sequence[str]] = None,
        identifier: Optional[Sequence[str]] = None,
        username: Optional[Sequence[str]] = None,
        request_id: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        from_date_time: Optional[datetime] = None,
        to_date_time: Optional[datetime] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AuditLogger], AuditLoggerListPaginated]:
        """
        List Audit Log

        **Pagination Sortable Columns:** `audit_log.id`, `audit_log.action`, `audit_log.username`, `audit_log.created`, `request_uri`

        **Note:** Both `from_date_time` and `to_date_time` support RFC3339 format with microseconds

        **Permission Required:** `kelvin.permission.audit_log.read`.

        ``listAuditLog``: ``GET`` ``/api/v4/instance/auditlog/list``

        Parameters
        ----------
        action : :obj:`Sequence[str]`
            Filter by action
        user_id : :obj:`Sequence[str]`
            Filter by user_id
        namespace : :obj:`Sequence[str]`
            Filter by namespace
        identifier : :obj:`Sequence[str]`
            Filter by identifier
        username : :obj:`Sequence[str]`
            Filter by username
        request_id : :obj:`Sequence[str]`
            Filter by request_id
        search : :obj:`Sequence[str]`
            Search Audit Log by action, username or namespace
        from_date_time : :obj:`datetime`
            Filter actions occurred at or after this date-time
        to_date_time : :obj:`datetime`
            Filter actions occurred until or at this date-time
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            false

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/instance/auditlog/list",
            {},
            {
                "action": action,
                "user_id": user_id,
                "namespace": namespace,
                "identifier": identifier,
                "username": username,
                "request_id": request_id,
                "search": search,
                "from_date_time": from_date_time,
                "to_date_time": to_date_time,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AuditLoggerListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"audit_logger_id": "id"})
    def get_audit_log(
        self_or_cls, audit_logger_id: str, _dry_run: bool = False, _client: Any = None
    ) -> AuditLogger:
        """
        Get Audit Log

        **Permission Required:** `kelvin.permission.audit_log.read`.

        ``getAuditLog``: ``GET`` ``/api/v4/instance/auditlog/{audit_logger_id}/get``

        Parameters
        ----------
        audit_logger_id : :obj:`str`, optional
            Audit Logger ID

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/instance/auditlog/{audit_logger_id}/get",
            {"audit_logger_id": audit_logger_id},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AuditLogger, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class AuditLoggerListPaginated(PaginatorDataModel[AuditLogger]):
    """
    AuditLoggerListPaginated object.

    Parameters
    ----------
    data: Optional[List[AuditLogger]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AuditLogger]] = None
    pagination: Optional[pagination.Pagination] = None


class Bridge(DataModelBase):
    """
    Bridge object.

    Parameters
    ----------
    acp_name: Optional[str]
    created: Optional[datetime]
    name: Optional[str]
    payload: Optional[Dict[str, Any]]
    protocol: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]
    workload_name: Optional[str]

    """

    acp_name: Optional[str] = None
    created: Optional[datetime] = None
    name: Optional[str] = None
    payload: Optional[Dict[str, Any]] = None
    protocol: Optional[str] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None
    workload_name: Optional[str] = None

    @instance_classmethod
    @DataModelBase.translate()
    def deploy_bridge(
        self_or_cls,
        data: Optional[Union[requests.BridgeDeploy, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Bridge:
        """
        Deploy Bridge App

        **Permission Required:** `kelvin.permission.bridge.create`.

        ``deployBridge``: ``POST`` ``/api/v4/bridges/deploy``

        Parameters
        ----------
        data: requests.BridgeDeploy, optional
        **kwargs:
            Extra parameters for requests.BridgeDeploy
              - acp_name: str
              - name: str
              - payload: dict
              - protocol: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/bridges/deploy",
            {},
            {},
            {},
            {},
            data,
            requests.BridgeDeploy,
            False,
            {"201": Bridge, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_bridge(
        self_or_cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        acp_name: Optional[Sequence[str]] = None,
        protocol: Optional[Sequence[str]] = None,
        workload_name: Optional[Sequence[str]] = None,
        status_state: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[Bridge], BridgeListPaginated]:
        """
        List Bridge

        **Pagination Sortable Columns:** `bridge.name`, `bridge.title`, `bridge.protocol`, `bridge.acp_name`, `bridge.workload_name`, `bridge.created`, `bridge.updated`, `status_state`, `status_last_seen`

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``listBridge``: ``GET`` ``/api/v4/bridges/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter bridges by name
        search : :obj:`Sequence[str]`
            Search bridges by name or title
        acp_name : :obj:`Sequence[str]`
            Filter bridges by acp_name
        protocol : :obj:`Sequence[str]`
            Filter bridges by protocol. Supported protocols [`modbus`, `opc-ua`,
            `mqtt`, `roc`]
        workload_name : :obj:`Sequence[str]`
            Filter bridges by workload_name
        status_state : :obj:`Sequence[str]`
            Filter bridges by status state
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/list",
            {},
            {
                "names": names,
                "search": search,
                "acp_name": acp_name,
                "protocol": protocol,
                "workload_name": workload_name,
                "status_state": status_state,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": BridgeListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def list_mqtt_topics(
        cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> MQTTTopicList:
        """
        List MQTT Topics

        **Sortable Columns:** `bridge_mqtt_topic.name`, `bridge_mqtt_topic.title`,
        `bridge_mqtt_topic.topic`, `bridge_mqtt_topic.created`, `bridge_mqtt_topic.updated`

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``listMQTTTopics``: ``GET`` ``/api/v4/bridges/protocols/mqtt/configuration/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter MQTT topics by name
        search : :obj:`Sequence[str]`
            Search MQTT topics by name, title or topic
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/protocols/mqtt/configuration/list",
            {},
            {"names": names, "search": search, "sort_by": sort_by, "sort_asc": sort_asc},
            {},
            {},
            None,
            None,
            False,
            {"200": MQTTTopicList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"bridge_name": "name"})
    def list_bridge_assets(
        self_or_cls,
        bridge_name: str,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetItem], AssetListPaginated]:
        """
        List Bridge Assets

        **Pagination Sortable Columns:** `asset.name`, `asset.title`, `asset.created`, `asset.updated`, `asset_type.title`

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``listBridgeAssets``: ``GET`` ``/api/v4/bridges/{bridge_name}/assets/list``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/{bridge_name}/assets/list",
            {"bridge_name": bridge_name},
            {
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"bridge_name": "name"})
    def list_bridge_data_streams(
        self_or_cls,
        bridge_name: str,
        asset_name: Optional[Sequence[str]] = None,
        metric_name: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[BridgeDataStreamItem], BridgeDataStreamListPaginated]:
        """
        List Bridge Data Streams

        **Pagination Sortable Columns:** `bridge_metric_asset.asset_name`, `bridge_metric_asset.metric_name`

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``listBridgeDataStreams``: ``GET`` ``/api/v4/bridges/{bridge_name}/datastreams/list``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name
        asset_name : :obj:`Sequence[str]`
            Filter data stream by asset name
        metric_name : :obj:`Sequence[str]`
            Filter data stream by metric name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/{bridge_name}/datastreams/list",
            {"bridge_name": bridge_name},
            {
                "asset_name": asset_name,
                "metric_name": metric_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": BridgeDataStreamListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"bridge_name": "name"})
    def delete_bridge(
        self_or_cls, bridge_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Bridge application

        **Permission Required:** `kelvin.permission.bridge.delete`.

        ``deleteBridge``: ``POST`` ``/api/v4/bridges/{bridge_name}/delete``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/bridges/{bridge_name}/delete",
            {"bridge_name": bridge_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"bridge_name": "name"})
    def get_bridge(
        self_or_cls, bridge_name: str, _dry_run: bool = False, _client: Any = None
    ) -> Bridge:
        """
        Get Bridge

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``getBridge``: ``GET`` ``/api/v4/bridges/{bridge_name}/get``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/{bridge_name}/get",
            {"bridge_name": bridge_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": Bridge, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"bridge_name": "name"})
    def list_bridge_metrics(
        self_or_cls,
        bridge_name: str,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[Metric], MetricListPaginated]:
        """
        List Bridge Metrics

        **Pagination Sortable Columns:** `metric.name`, `metric.title`, `metric.created`, `metric.updated`, `metric.data_type_name`

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``listBridgeMetrics``: ``GET`` ``/api/v4/bridges/{bridge_name}/metrics/list``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/{bridge_name}/metrics/list",
            {"bridge_name": bridge_name},
            {
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": MetricListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"bridge_name": "name"})
    def start_bridge(
        self_or_cls, bridge_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Start Bridge application

        **Permission Required:** `kelvin.permission.bridge.update`.

        ``startBridge``: ``GET`` ``/api/v4/bridges/{bridge_name}/start``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/{bridge_name}/start",
            {"bridge_name": bridge_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"bridge_name": "name"})
    def stop_bridge(
        self_or_cls, bridge_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Stop Bridge application

        **Permission Required:** `kelvin.permission.bridge.update`.

        ``stopBridge``: ``GET`` ``/api/v4/bridges/{bridge_name}/stop``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/{bridge_name}/stop",
            {"bridge_name": bridge_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class BridgeDataStreamItem(DataModelBase):
    """
    BridgeDataStreamItem object.

    Parameters
    ----------
    asset_name: Optional[str]
    metric_name: Optional[str]
    payload: Optional[Dict[str, Any]]

    """

    asset_name: Optional[str] = None
    metric_name: Optional[str] = None
    payload: Optional[Dict[str, Any]] = None

    @instance_classmethod
    @DataModelBase.translate()
    def list_bridge_data_streams(
        self_or_cls,
        bridge_name: str,
        asset_name: Optional[Sequence[str]] = None,
        metric_name: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[BridgeDataStreamItem], BridgeDataStreamListPaginated]:
        """
        List Bridge Data Streams

        **Pagination Sortable Columns:** `bridge_metric_asset.asset_name`, `bridge_metric_asset.metric_name`

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``listBridgeDataStreams``: ``GET`` ``/api/v4/bridges/{bridge_name}/datastreams/list``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name
        asset_name : :obj:`Sequence[str]`
            Filter data stream by asset name
        metric_name : :obj:`Sequence[str]`
            Filter data stream by metric name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/{bridge_name}/datastreams/list",
            {"bridge_name": bridge_name},
            {
                "asset_name": asset_name,
                "metric_name": metric_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": BridgeDataStreamListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result


class BridgeDataStreamListPaginated(PaginatorDataModel[BridgeDataStreamItem]):
    """
    BridgeDataStreamListPaginated object.

    Parameters
    ----------
    data: Optional[List[BridgeDataStreamItem]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[BridgeDataStreamItem]] = None
    pagination: Optional[pagination.Pagination] = None


class BridgeListPaginated(PaginatorDataModel[Bridge]):
    """
    BridgeListPaginated object.

    Parameters
    ----------
    data: Optional[List[Bridge]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[Bridge]] = None
    pagination: Optional[pagination.Pagination] = None


class CSVPosition(DataModelBase):
    """
    CSVPosition object.

    Parameters
    ----------
    column: Optional[int]
    field_description: Optional[str]
    field_solution: Optional[str]
    row: Optional[int]

    """

    column: Optional[int] = None
    field_description: Optional[str] = None
    field_solution: Optional[str] = None
    row: Optional[int] = None


class ClusterCIDR(DataModelBase):
    """
    ClusterCIDR object.

    Parameters
    ----------
    pod_cidr: Optional[str]
    service_cidr: Optional[str]

    """

    pod_cidr: Optional[str] = None
    service_cidr: Optional[str] = None


class ClusterCIDRItem(DataModelBase):
    """
    ClusterCIDRItem object.

    Parameters
    ----------
    cluster_name: Optional[str]
    created: Optional[datetime]
    kube_pod_cidr: Optional[str]
    kube_service_cidr: Optional[str]
    updated: Optional[datetime]

    """

    cluster_name: Optional[str] = None
    created: Optional[datetime] = None
    kube_pod_cidr: Optional[str] = None
    kube_service_cidr: Optional[str] = None
    updated: Optional[datetime] = None

    @classmethod
    def list_cluster_cidr(
        cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[ClusterCIDRItem], ClusterCIDRListPaginated]:
        """
        List Cluster CIDR

        **Pagination Sortable Columns:** `cidr.kube_pod_cidr`, `cidr.kube_service_cidr`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listClusterCIDR``: ``GET`` ``/api/v4/orchestration/cidr/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search CIDR by Kubernetes Pod IP, Kubernetes Service IP or Cluster
            Name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/orchestration/cidr/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": ClusterCIDRListPaginated, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result


class ClusterCIDRListPaginated(PaginatorDataModel[ClusterCIDRItem]):
    """
    ClusterCIDRListPaginated object.

    Parameters
    ----------
    data: Optional[List[ClusterCIDRItem]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[ClusterCIDRItem]] = None
    pagination: Optional[pagination.Pagination] = None


class ClusterManifestItem(DataModelBase):
    """
    ClusterManifestItem object.

    Parameters
    ----------
    content: Optional[str]
    file_name: Optional[str]

    """

    content: Optional[str] = None
    file_name: Optional[str] = None


class ClusterManifestList(DataModelBase):
    """
    ClusterManifestList object.

    Parameters
    ----------
    manifests: Optional[List[ClusterManifestItem]]
    revision: Optional[str]

    """

    manifests: Optional[List[ClusterManifestItem]] = None
    revision: Optional[str] = None

    @classmethod
    def get_cluster_manifests(
        cls,
        cluster_name: str,
        version: Optional[str] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> ClusterManifestList:
        """
        Get Cluster Manifests

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getClusterManifests``: ``GET`` ``/api/v4/orchestration/clusters/{cluster_name}/manifests/get``

        Parameters
        ----------
        cluster_name : :obj:`str`, optional
            Cluster name
        version : :obj:`str`
            Current Version

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/orchestration/clusters/{cluster_name}/manifests/get",
            {"cluster_name": cluster_name},
            {"version": version},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ClusterManifestList,
                "204": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result


class ComponentStatus(DataModelBase):
    """
    ComponentStatus object.

    Parameters
    ----------
    name: Optional[str]
    status: Optional[bool]

    """

    name: Optional[str] = None
    status: Optional[bool] = None


class CreatedBy(DataModelBase):
    """
    CreatedBy object.

    Parameters
    ----------
    name: Optional[str]
    title: Optional[str]
    type: Optional[str]

    """

    name: Optional[str] = None
    title: Optional[str] = None
    type: Optional[str] = None


class DataLabelCluster(DataModelBase):
    """
    DataLabelCluster object.

    Parameters
    ----------
    count: Optional[int]
    end_date: Optional[datetime]
    label_names: Optional[List[str]]
    start_date: Optional[datetime]

    """

    count: Optional[int] = None
    end_date: Optional[datetime] = None
    label_names: Optional[List[str]] = None
    start_date: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def get_data_labels_clustering(
        self_or_cls,
        data: Optional[Union[requests.DataLabelCluster, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> DataLabelCluster:
        """
        Get Data Labels Clustering

        **Note**:
        * `time_bucket`: needed to aggregate a cluster. Use Go parsing for duration:
        https://golang.org/pkg/time/#ParseDuration
        * `label_names`: lists data labels using **OR** operator;
        * `metrics`: lists data labels using **OR** operator;
        * `start_date`: lists data labels with start_date >= query date (RFC3339 format with microseconds);
        * `end_date`: lists data labels with end_date <= query date (RFC3339 format with microseconds)

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``getDataLabelsClustering``: ``POST`` ``/api/v4/datalabels/clustering/get``

        Parameters
        ----------
        data: requests.DataLabelCluster, optional
        **kwargs:
            Extra parameters for requests.DataLabelCluster
              - end_date: datetime
              - label_names: list
              - metrics: list
              - start_date: datetime
              - time_bucket: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/clustering/get",
            {},
            {},
            {},
            {},
            data,
            requests.DataLabelCluster,
            False,
            {"200": DataLabelCluster, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result


class DataLabelMetricItem(DataModelBase):
    """
    DataLabelMetricItem object.

    Parameters
    ----------
    asset_name: Optional[str]
    name: Optional[str]

    """

    asset_name: Optional[str] = None
    name: Optional[str] = None


class Type(Enum):
    user = "user"
    workload = "workload"


class DataLabelSource(DataModelBase):
    """
    DataLabelSource object.

    Parameters
    ----------
    info: Optional[Dict[str, Any]]
    type: Optional[Type]

    """

    info: Optional[Dict[str, Any]] = None
    type: Optional[Type] = None


class DataTypeSchema(DataModelBase):
    """
    DataTypeSchema object.

    Parameters
    ----------

    """

    class Config(DataModelBase.Config):
        extra = Extra.allow

    @classmethod
    def get_data_type_schema(
        cls,
        data_type_name: str,
        data_type_version: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> DataTypeSchema:
        """
        Get Data Type Schema

        **Permission Required:** `kelvin.permission.data_type.read`.

        ``getDataTypeSchema``: ``GET`` ``/api/v4/datatypes/{data_type_name}/versions/{data_type_version}/schema/get``

        Parameters
        ----------
        data_type_name : :obj:`str`, optional
            Data Type name
        data_type_version : :obj:`str`, optional
            Data Type version

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/datatypes/{data_type_name}/versions/{data_type_version}/schema/get",
            {"data_type_name": data_type_name, "data_type_version": data_type_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": DataTypeSchema,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result


class Dependency(DataModelBase):
    """
    Dependency object.

    Parameters
    ----------
    collection: Optional[str]
    name: Optional[str]
    title: Optional[str]

    """

    collection: Optional[str] = None
    name: Optional[str] = None
    title: Optional[str] = None


class ErrorMessage(DataModelBase):
    """
    ErrorMessage object.

    Parameters
    ----------
    error_code: Optional[int]
    http_status_code: Optional[int]
    message: Optional[List[str]]

    """

    error_code: Optional[int] = None
    http_status_code: Optional[int] = Field(None, example=400)
    message: Optional[List[str]] = None


class ErrorMessages(DataModelBase):
    """
    ErrorMessages object.

    Parameters
    ----------
    errors: Optional[List[ErrorMessage]]

    """

    errors: Optional[List[ErrorMessage]] = None


class FailureDetectionProperty(DataModelBase):
    """
    FailureDetectionProperty object.

    Parameters
    ----------
    name: Optional[str]
    unit: Optional[str]
    value: Optional[str]

    """

    name: Optional[str] = None
    unit: Optional[str] = None
    value: Optional[str] = None


class FailureItem(DataModelBase):
    """
    FailureItem object.

    Parameters
    ----------
    created: Optional[datetime]
    name: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    name: Optional[str] = None
    updated: Optional[datetime] = None


class FailureList(DataModelBase):
    """
    FailureList object.

    Parameters
    ----------
    data: Optional[List[FailureItem]]

    """

    data: Optional[List[FailureItem]] = None

    @classmethod
    def list_failure_modes(
        cls,
        component_failure_class_name: Optional[str] = None,
        part_failure_class_name: Optional[str] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> FailureList:
        """
        List Failure Modes

        **Permission Required:** `kelvin.permission.failure_detection.read`.

        ``listFailureModes``: ``GET`` ``/api/v4/failure-detection/modes/list``

        Parameters
        ----------
        component_failure_class_name : :obj:`str`
            Filter failures by component failure class name
        part_failure_class_name : :obj:`str`
            Filter failures by part failure class name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/failure-detection/modes/list",
            {},
            {
                "component_failure_class_name": component_failure_class_name,
                "part_failure_class_name": part_failure_class_name,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": FailureList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def list_failure_reasons(
        cls,
        component_failure_class_name: Optional[str] = None,
        part_failure_class_name: Optional[str] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> FailureList:
        """
        List Failure Reasons

        **Permission Required:** `kelvin.permission.failure_detection.read`.

        ``listFailureReasons``: ``GET`` ``/api/v4/failure-detection/reasons/list``

        Parameters
        ----------
        component_failure_class_name : :obj:`str`
            Filter failures by component failure class name
        part_failure_class_name : :obj:`str`
            Filter failures by part failure class name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/failure-detection/reasons/list",
            {},
            {
                "component_failure_class_name": component_failure_class_name,
                "part_failure_class_name": part_failure_class_name,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": FailureList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class ICDField(DataModelBase):
    """
    ICDField object.

    Parameters
    ----------
    array: Optional[bool]
    description: Optional[str]
    name: Optional[str]
    type: Optional[str]

    """

    array: Optional[bool] = None
    description: Optional[str] = None
    name: Optional[str] = None
    type: Optional[str] = None


class ICDPayload(DataModelBase):
    """
    ICDPayload object.

    Parameters
    ----------
    class_name: Optional[str]
    description: Optional[str]
    fields: Optional[List[ICDField]]
    name: Optional[str]
    version: Optional[str]

    """

    class_name: Optional[str] = None
    description: Optional[str] = None
    fields: Optional[List[ICDField]] = None
    name: Optional[str] = None
    version: Optional[str] = None


class InstanceHealthStatus(DataModelBase):
    """
    InstanceHealthStatus object.

    Parameters
    ----------
    components: Optional[List[ComponentStatus]]
    status: Optional[bool]

    """

    components: Optional[List[ComponentStatus]] = None
    status: Optional[bool] = None

    @classmethod
    def get_instance_health_status(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> InstanceHealthStatus:
        """
        Get Instance Health Status

        **Permission Required:** `n/a`.

        ``getInstanceHealthStatus``: ``GET`` ``/api/v4/instance/status/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/instance/status/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"201": InstanceHealthStatus, "400": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class IntegrationAwsRegion(DataModelBase):
    """
    IntegrationAwsRegion object.

    Parameters
    ----------
    created: Optional[datetime]
    name: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    name: Optional[str] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_aws_region(
        self_or_cls,
        data: Optional[Union[requests.IntegrationAwsRegionCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> IntegrationAwsRegion:
        """
        Create AWS Region

        **Permission Required:** `kelvin.permission.integration.create`.

        ``createAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/create``

        Parameters
        ----------
        data: requests.IntegrationAwsRegionCreate, optional
        **kwargs:
            Extra parameters for requests.IntegrationAwsRegionCreate
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/create",
            {},
            {},
            {},
            {},
            data,
            requests.IntegrationAwsRegionCreate,
            False,
            {"201": IntegrationAwsRegion, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_aws_regions(
        cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> IntegrationAwsRegionList:
        """
        List AWS Regions

        **Pagination Sortable Columns:** `aws_regions.name`, `aws_regions.title`, `aws_regions.created`, `aws_regions.updated`

        **Permission Required:** `kelvin.permission.integration.read`.

        ``listAWSRegions``: ``GET`` ``/api/v4/integrations/aws/regions/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter Regions by name
        search : :obj:`Sequence[str]`
            Search Regions by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/integrations/aws/regions/list",
            {},
            {
                "names": names,
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": IntegrationAwsRegionList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_aws_region(
        self_or_cls, name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete AWS Region

        **Permission Required:** `kelvin.permission.integration.delete`.

        ``deleteAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/{name}/delete``

        Parameters
        ----------
        name : :obj:`str`, optional
            Region name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/{name}/delete",
            {"name": name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_aws_region(
        self_or_cls,
        name: str,
        data: Optional[Union[requests.IntegrationAwsRegionUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> IntegrationAwsRegion:
        """
        Update AWS Region

        **Permission Required:** `kelvin.permission.integration.update`.

        ``updateAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/{name}/update``

        Parameters
        ----------
        name : :obj:`str`, optional
            Region name
        data: requests.IntegrationAwsRegionUpdate, optional
        **kwargs:
            Extra parameters for requests.IntegrationAwsRegionUpdate
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/{name}/update",
            {"name": name},
            {},
            {},
            {},
            data,
            requests.IntegrationAwsRegionUpdate,
            False,
            {
                "200": IntegrationAwsRegion,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class IntegrationAwsRegionList(PaginatorDataModel[IntegrationAwsRegion]):
    """
    IntegrationAwsRegionList object.

    Parameters
    ----------
    data: Optional[List[IntegrationAwsRegion]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[IntegrationAwsRegion]] = None
    pagination: Optional[pagination.Pagination] = None

    @classmethod
    def create_aws_region(
        cls,
        data: Optional[Union[requests.IntegrationAwsRegionCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> IntegrationAwsRegion:
        """
        Create AWS Region

        **Permission Required:** `kelvin.permission.integration.create`.

        ``createAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/create``

        Parameters
        ----------
        data: requests.IntegrationAwsRegionCreate, optional
        **kwargs:
            Extra parameters for requests.IntegrationAwsRegionCreate
              - name: str
              - title: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/create",
            {},
            {},
            {},
            {},
            data,
            requests.IntegrationAwsRegionCreate,
            False,
            {"201": IntegrationAwsRegion, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_aws_regions(
        cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> IntegrationAwsRegionList:
        """
        List AWS Regions

        **Pagination Sortable Columns:** `aws_regions.name`, `aws_regions.title`, `aws_regions.created`, `aws_regions.updated`

        **Permission Required:** `kelvin.permission.integration.read`.

        ``listAWSRegions``: ``GET`` ``/api/v4/integrations/aws/regions/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter Regions by name
        search : :obj:`Sequence[str]`
            Search Regions by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/integrations/aws/regions/list",
            {},
            {
                "names": names,
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": IntegrationAwsRegionList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def delete_aws_region(cls, name: str, _dry_run: bool = False, _client: Any = None) -> None:
        """
        Delete AWS Region

        **Permission Required:** `kelvin.permission.integration.delete`.

        ``deleteAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/{name}/delete``

        Parameters
        ----------
        name : :obj:`str`, optional
            Region name

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/{name}/delete",
            {"name": name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def update_aws_region(
        cls,
        name: str,
        data: Optional[Union[requests.IntegrationAwsRegionUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> IntegrationAwsRegion:
        """
        Update AWS Region

        **Permission Required:** `kelvin.permission.integration.update`.

        ``updateAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/{name}/update``

        Parameters
        ----------
        name : :obj:`str`, optional
            Region name
        data: requests.IntegrationAwsRegionUpdate, optional
        **kwargs:
            Extra parameters for requests.IntegrationAwsRegionUpdate
              - name: str
              - title: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/{name}/update",
            {"name": name},
            {},
            {},
            {},
            data,
            requests.IntegrationAwsRegionUpdate,
            False,
            {
                "200": IntegrationAwsRegion,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class IntegrationConfiguration(DataModelBase):
    """
    IntegrationConfiguration object.

    Parameters
    ----------
    aws_access_key_id: Optional[str]
    aws_region: Optional[str]
    polling_rate: Optional[int]

    """

    aws_access_key_id: Optional[str] = None
    aws_region: Optional[str] = None
    polling_rate: Optional[int] = None


class IntegrationStatus(DataModelBase):
    """
    IntegrationStatus object.

    Parameters
    ----------
    last_seen: Optional[datetime]
    message: Optional[str]
    state: Optional[str]

    """

    last_seen: Optional[datetime] = None
    message: Optional[str] = None
    state: Optional[str] = None


class Label(DataModelBase):
    """
    Label object.

    Parameters
    ----------
    created: Optional[datetime]
    metadata: Optional[Dict[str, Any]]
    name: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    metadata: Optional[Dict[str, Any]] = None
    name: Optional[str] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_label(
        self_or_cls,
        data: Optional[Union[requests.LabelCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Label:
        """
        Create Label

        **Permission Required:** `kelvin.permission.data_label.create`.

        ``createLabel``: ``POST`` ``/api/v4/datalabels/labels/create``

        Parameters
        ----------
        data: requests.LabelCreate, optional
        **kwargs:
            Extra parameters for requests.LabelCreate
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/create",
            {},
            {},
            {},
            {},
            data,
            requests.LabelCreate,
            False,
            {"201": Label, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_label(
        self_or_cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        metadata: Optional[Sequence[str]] = None,
        metadata_filter: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[Label], LabelListPaginated]:
        """
        List Label

        **Pagination Sortable Columns:** `label.name`

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``listLabel``: ``GET`` ``/api/v4/datalabels/labels/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter labels by name
        search : :obj:`Sequence[str]`
            Search Labels by name or title
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response
        metadata_filter : :obj:`Sequence[str]`
            Dot separated string to filter by metadata values Same <metadata_name>
            will result in OR queries. Different <metadata_name> will result in
            AND queries. Example:
            <metadata_name>.<payload_key_to_search>=<payload_value_to_search>
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/list",
            {},
            {
                "names": names,
                "search": search,
                "metadata": metadata,
                "metadata_filter": metadata_filter,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": LabelListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def create_label_metadata_bulk(
        cls,
        data: Optional[Union[requests.LabelMetadataBulkCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Create Label Metadata Bulk

        **Permission Required:** `kelvin.permission.data_label.create`.

        ``createLabelMetadataBulk``: ``POST`` ``/api/v4/datalabels/labels/metadata/create``

        Parameters
        ----------
        data: requests.LabelMetadataBulkCreate, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataBulkCreate

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/metadata/create",
            {},
            {},
            {},
            {},
            data,
            requests.LabelMetadataBulkCreate,
            False,
            {
                "201": None,
                "207": ErrorMessages,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def delete_label_metadata_bulk(
        cls,
        data: Optional[Union[requests.LabelMetadataBulkDelete, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Delete Label Metadata Bulk

        **Permission Required:** `kelvin.permission.data_label.delete`.

        ``deleteLabelMetadataBulk``: ``POST`` ``/api/v4/datalabels/labels/metadata/delete``

        Parameters
        ----------
        data: requests.LabelMetadataBulkDelete, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataBulkDelete

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/metadata/delete",
            {},
            {},
            {},
            {},
            data,
            requests.LabelMetadataBulkDelete,
            False,
            {
                "200": None,
                "207": ErrorMessages,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"label_name": "name"})
    def delete_label(
        self_or_cls, label_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Label

        **Permission Required:** `kelvin.permission.data_label.delete`.

        ``deleteLabel``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/delete``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/delete",
            {"label_name": label_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"label_name": "name"})
    def get_label(
        self_or_cls,
        label_name: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Label:
        """
        Get Label

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``getLabel``: ``GET`` ``/api/v4/datalabels/labels/{label_name}/get``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/{label_name}/get",
            {"label_name": label_name},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {"200": Label, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"label_name": "name"})
    def create_label_metadata(
        self_or_cls,
        label_name: str,
        data: Optional[Union[requests.LabelMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> LabelMetadata:
        """
        Create Label Metadata

        **Permission Required:** `kelvin.permission.data_label.create`.

        ``createLabelMetadata``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/metadata/create``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        data: requests.LabelMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/metadata/create",
            {"label_name": label_name},
            {},
            {},
            {},
            data,
            requests.LabelMetadataCreate,
            False,
            {
                "201": LabelMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"label_name": "name"})
    def list_label_metadata(
        self_or_cls,
        label_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[LabelMetadata], LabelMetadataListPaginated]:
        """
        List Label Metadata

        **Pagination Sortable Columns:** `label_metadata.name`, `label_metadata.label_name`

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``listLabelMetadata``: ``GET`` ``/api/v4/datalabels/labels/{label_name}/metadata/list``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        search : :obj:`Sequence[str]`
            Search Label Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/{label_name}/metadata/list",
            {"label_name": label_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": LabelMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"label_name": "name"})
    def delete_label_metadata(
        self_or_cls,
        label_name: str,
        label_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete Label Metadata

        **Permission Required:** `kelvin.permission.data_label.delete`.

        ``deleteLabelMetadata``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/delete``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        label_metadata_name : :obj:`str`, optional
            Label Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/delete",
            {"label_name": label_name, "label_metadata_name": label_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"label_name": "name"})
    def get_label_metadata(
        self_or_cls,
        label_name: str,
        label_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> LabelMetadata:
        """
        Get Label Metadata

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``getLabelMetadata``: ``GET`` ``/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/get``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        label_metadata_name : :obj:`str`, optional
            Label Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/get",
            {"label_name": label_name, "label_metadata_name": label_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": LabelMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"label_name": "name"})
    def update_label_metadata(
        self_or_cls,
        label_name: str,
        label_metadata_name: str,
        data: Optional[Union[requests.LabelMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> LabelMetadata:
        """
        Update Label Metadata

        **Permission Required:** `kelvin.permission.data_label.update`.

        ``updateLabelMetadata``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/update``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        label_metadata_name : :obj:`str`, optional
            Label Metadata name
        data: requests.LabelMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/update",
            {"label_name": label_name, "label_metadata_name": label_metadata_name},
            {},
            {},
            {},
            data,
            requests.LabelMetadataUpdate,
            False,
            {
                "200": LabelMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"label_name": "name"})
    def update_label(
        self_or_cls,
        label_name: str,
        data: Optional[Union[requests.LabelUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Label:
        """
        Update Label

        **Permission Required:** `kelvin.permission.data_label.update`.

        ``updateLabel``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/update``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        data: requests.LabelUpdate, optional
        **kwargs:
            Extra parameters for requests.LabelUpdate
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/update",
            {"label_name": label_name},
            {},
            {},
            {},
            data,
            requests.LabelUpdate,
            False,
            {
                "200": Label,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class LabelListPaginated(PaginatorDataModel[Label]):
    """
    LabelListPaginated object.

    Parameters
    ----------
    data: Optional[List[Label]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[Label]] = None
    pagination: Optional[pagination.Pagination] = None


class LabelMetadata(DataModelBase):
    """
    LabelMetadata object.

    Parameters
    ----------
    created: Optional[datetime]
    label_name: Optional[str]
    name: Optional[str]
    payload: Optional[Dict[str, Any]]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    label_name: Optional[str] = None
    name: Optional[str] = Field(None, max_length=64)
    payload: Optional[Dict[str, Any]] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_label_metadata(
        self_or_cls,
        label_name: str,
        data: Optional[Union[requests.LabelMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> LabelMetadata:
        """
        Create Label Metadata

        **Permission Required:** `kelvin.permission.data_label.create`.

        ``createLabelMetadata``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/metadata/create``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        data: requests.LabelMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/metadata/create",
            {"label_name": label_name},
            {},
            {},
            {},
            data,
            requests.LabelMetadataCreate,
            False,
            {
                "201": LabelMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_label_metadata(
        self_or_cls,
        label_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[LabelMetadata], LabelMetadataListPaginated]:
        """
        List Label Metadata

        **Pagination Sortable Columns:** `label_metadata.name`, `label_metadata.label_name`

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``listLabelMetadata``: ``GET`` ``/api/v4/datalabels/labels/{label_name}/metadata/list``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        search : :obj:`Sequence[str]`
            Search Label Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/{label_name}/metadata/list",
            {"label_name": label_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": LabelMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"label_metadata_name": "name"})
    def delete_label_metadata(
        self_or_cls,
        label_name: str,
        label_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete Label Metadata

        **Permission Required:** `kelvin.permission.data_label.delete`.

        ``deleteLabelMetadata``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/delete``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        label_metadata_name : :obj:`str`, optional
            Label Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/delete",
            {"label_name": label_name, "label_metadata_name": label_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"label_metadata_name": "name"})
    def get_label_metadata(
        self_or_cls,
        label_name: str,
        label_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> LabelMetadata:
        """
        Get Label Metadata

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``getLabelMetadata``: ``GET`` ``/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/get``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        label_metadata_name : :obj:`str`, optional
            Label Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/get",
            {"label_name": label_name, "label_metadata_name": label_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": LabelMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"label_metadata_name": "name"})
    def update_label_metadata(
        self_or_cls,
        label_name: str,
        label_metadata_name: str,
        data: Optional[Union[requests.LabelMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> LabelMetadata:
        """
        Update Label Metadata

        **Permission Required:** `kelvin.permission.data_label.update`.

        ``updateLabelMetadata``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/update``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        label_metadata_name : :obj:`str`, optional
            Label Metadata name
        data: requests.LabelMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/update",
            {"label_name": label_name, "label_metadata_name": label_metadata_name},
            {},
            {},
            {},
            data,
            requests.LabelMetadataUpdate,
            False,
            {
                "200": LabelMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class LabelMetadataListPaginated(PaginatorDataModel[LabelMetadata]):
    """
    LabelMetadataListPaginated object.

    Parameters
    ----------
    data: Optional[List[LabelMetadata]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[LabelMetadata]] = None
    pagination: Optional[pagination.Pagination] = None


class MQTTTopicItem(DataModelBase):
    """
    MQTTTopicItem object.

    Parameters
    ----------
    created: Optional[datetime]
    name: Optional[str]
    payload: Optional[str]
    title: Optional[str]
    topic: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    name: Optional[str] = None
    payload: Optional[str] = None
    title: Optional[str] = None
    topic: Optional[str] = None
    updated: Optional[datetime] = None


class MQTTTopicList(DataModelBase):
    """
    MQTTTopicList object.

    Parameters
    ----------
    data: Optional[List[MQTTTopicItem]]

    """

    data: Optional[List[MQTTTopicItem]] = None

    @classmethod
    def list_mqtt_topics(
        cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> MQTTTopicList:
        """
        List MQTT Topics

        **Sortable Columns:** `bridge_mqtt_topic.name`, `bridge_mqtt_topic.title`,
        `bridge_mqtt_topic.topic`, `bridge_mqtt_topic.created`, `bridge_mqtt_topic.updated`

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``listMQTTTopics``: ``GET`` ``/api/v4/bridges/protocols/mqtt/configuration/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter MQTT topics by name
        search : :obj:`Sequence[str]`
            Search MQTT topics by name, title or topic
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/protocols/mqtt/configuration/list",
            {},
            {"names": names, "search": search, "sort_by": sort_by, "sort_asc": sort_asc},
            {},
            {},
            None,
            None,
            False,
            {"200": MQTTTopicList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class Metric(DataModelBase):
    """
    Metric object.

    Parameters
    ----------
    created: Optional[datetime]
    data_type_name: Optional[str]
    data_type_version: Optional[str]
    display_unit: Optional[str]
    name: Optional[str]
    semantic_type_name: Optional[str]
    semantic_type_properties: Optional[Dict[str, Any]]
    title: Optional[str]
    updated: Optional[datetime]
    write: Optional[bool]

    """

    created: Optional[datetime] = None
    data_type_name: Optional[str] = None
    data_type_version: Optional[str] = None
    display_unit: Optional[str] = None
    name: Optional[str] = None
    semantic_type_name: Optional[str] = None
    semantic_type_properties: Optional[Dict[str, Any]] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None
    write: Optional[bool] = None

    @classmethod
    def list_bridge_metrics(
        cls,
        bridge_name: str,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[Metric], MetricListPaginated]:
        """
        List Bridge Metrics

        **Pagination Sortable Columns:** `metric.name`, `metric.title`, `metric.created`, `metric.updated`, `metric.data_type_name`

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``listBridgeMetrics``: ``GET`` ``/api/v4/bridges/{bridge_name}/metrics/list``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/{bridge_name}/metrics/list",
            {"bridge_name": bridge_name},
            {
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": MetricListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def create_metric(
        self_or_cls,
        data: Optional[Union[requests.MetricCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Metric:
        """
        Create Metric

        **Permission Required:** `kelvin.permission.metric.create`.

        ``createMetric``: ``POST`` ``/api/v4/metrics/create``

        Parameters
        ----------
        data: requests.MetricCreate, optional
        **kwargs:
            Extra parameters for requests.MetricCreate
              - data_type_name: str
              - data_type_version: str
              - display_unit: str
              - name: str
              - semantic_type_name: str
              - semantic_type_properties: dict
              - title: str
              - write: bool

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/metrics/create",
            {},
            {},
            {},
            {},
            data,
            requests.MetricCreate,
            False,
            {"201": Metric, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def upload_metrics(
        cls, file: str, dry_run: Optional[bool] = None, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Upload Metrics

        **Permission Required:** `kelvin.permission.metric.create`.

        ``uploadMetrics``: ``POST`` ``/api/v4/metrics/create/upload``

        Parameters
        ----------
        file : :obj:`str`, optional
            CSV File
        dry_run : :obj:`bool`
            If set to true performs only validation

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/metrics/create/upload",
            {},
            {"dry_run": dry_run},
            {"file": file},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "412": CSVErrorMessage},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_metric(
        self_or_cls,
        data_type_name: Optional[Sequence[str]] = None,
        data_type_version: Optional[Sequence[str]] = None,
        semantic_type_name: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        search: Optional[Sequence[str]] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[Metric], MetricListPaginated]:
        """
        List Metric

        **Pagination Sortable Columns:** `metric.name`, `metric.title`, `metric.data_type_name`,
        `metric.data_type_version`, `metric.display_unit`, `metric.semantic_type_name`, `metric.write`,
        `metric.created`, `metric.updated`, `semantic_type.title`

        **Permission Required:** `kelvin.permission.metric.read`.

        ``listMetric``: ``GET`` ``/api/v4/metrics/list``

        Parameters
        ----------
        data_type_name : :obj:`Sequence[str]`
            Filter metrics by Data Type Name
        data_type_version : :obj:`Sequence[str]`
            Filter metrics by Data Type Version
        semantic_type_name : :obj:`Sequence[str]`
            Filter metrics by Semantic Type Name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        search : :obj:`Sequence[str]`
            Search metric by name, title

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/metrics/list",
            {},
            {
                "data_type_name": data_type_name,
                "data_type_version": data_type_version,
                "semantic_type_name": semantic_type_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "search": search,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": MetricListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def list_semantic_type(
        cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> SemanticTypeList:
        """
        List Semantic Type

        **Sortable Columns:** `semantic_type.name`, `semantic_type.title`,
        `semantic_type.created`, `semantic_type.updated`

        **Permission Required:** `kelvin.permission.metric.read`.

        ``listSemanticType``: ``GET`` ``/api/v4/metrics/semantictypes/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter semantic types by name
        search : :obj:`Sequence[str]`
            Search semantic types by name or title
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/metrics/semantictypes/list",
            {},
            {"names": names, "search": search, "sort_by": sort_by, "sort_asc": sort_asc},
            {},
            {},
            None,
            None,
            False,
            {"200": SemanticTypeList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"metric_name": "name"})
    def delete_metric(
        self_or_cls, metric_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Metric

        **Permission Required:** `kelvin.permission.metric.delete`.

        ``deleteMetric``: ``POST`` ``/api/v4/metrics/{metric_name}/delete``

        Parameters
        ----------
        metric_name : :obj:`str`, optional
            Metric name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/metrics/{metric_name}/delete",
            {"metric_name": metric_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"metric_name": "name"})
    def get_metric(
        self_or_cls, metric_name: str, _dry_run: bool = False, _client: Any = None
    ) -> Metric:
        """
        Get Metric

        **Permission Required:** `kelvin.permission.metric.read`.

        ``getMetric``: ``GET`` ``/api/v4/metrics/{metric_name}/get``

        Parameters
        ----------
        metric_name : :obj:`str`, optional
            Metric name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/metrics/{metric_name}/get",
            {"metric_name": metric_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": Metric, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"metric_name": "name"})
    def update_metric(
        self_or_cls,
        metric_name: str,
        data: Optional[Union[requests.MetricUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Metric:
        """
        Update Metric

        **Permission Required:** `kelvin.permission.metric.update`.

        ``updateMetric``: ``POST`` ``/api/v4/metrics/{metric_name}/update``

        Parameters
        ----------
        metric_name : :obj:`str`, optional
            Metric name
        data: requests.MetricUpdate, optional
        **kwargs:
            Extra parameters for requests.MetricUpdate
              - data_type_name: str
              - data_type_version: str
              - display_unit: str
              - semantic_type_name: str
              - semantic_type_properties: dict
              - title: str
              - write: bool

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/metrics/{metric_name}/update",
            {"metric_name": metric_name},
            {},
            {},
            {},
            data,
            requests.MetricUpdate,
            False,
            {
                "200": Metric,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class MetricListPaginated(PaginatorDataModel[Metric]):
    """
    MetricListPaginated object.

    Parameters
    ----------
    data: Optional[List[Metric]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[Metric]] = None
    pagination: Optional[pagination.Pagination] = None


class NetworkingAddressItem(DataModelBase):
    """
    NetworkingAddressItem object.

    Parameters
    ----------
    address: Optional[str]
    interface: Optional[str]
    port: Optional[int]

    """

    address: Optional[str] = None
    interface: Optional[str] = None
    port: Optional[int] = None


class NetworkingItem(DataModelBase):
    """
    NetworkingItem object.

    Parameters
    ----------
    addresses: Optional[List[NetworkingAddressItem]]
    name: Optional[str]
    protocol: Optional[str]

    """

    addresses: Optional[List[NetworkingAddressItem]] = None
    name: Optional[str] = None
    protocol: Optional[str] = None


class OrchestrationProvision(DataModelBase):
    """
    OrchestrationProvision object.

    Parameters
    ----------
    provision_script: Optional[str]

    """

    provision_script: Optional[str] = None

    @classmethod
    def download_cluster_provision_script(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> OrchestrationProvision:
        """
        Download Cluster Provision Script

        **Permission Required:** `kelvin.permission.orchestration.read`.

        ``downloadClusterProvisionScript``: ``GET`` ``/api/v4/orchestration/clusters/provision/script/download``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/orchestration/clusters/provision/script/download",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": OrchestrationProvision, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class PropertyOptions(DataModelBase):
    """
    PropertyOptions object.

    Parameters
    ----------
    name: Optional[str]
    title: Optional[str]

    """

    name: Optional[str] = None
    title: Optional[str] = None


class Secret(DataModelBase):
    """
    Secret object.

    Parameters
    ----------
    name: Optional[str]

    """

    name: Optional[str] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_secret(
        self_or_cls,
        data: Optional[Union[requests.SecretCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Secret:
        """
        Create Secret

        **Permission Required:** `kelvin.permission.secret.update`.

        ``createSecret``: ``POST`` ``/api/v4/secrets/create``

        Parameters
        ----------
        data: requests.SecretCreate, optional
        **kwargs:
            Extra parameters for requests.SecretCreate
              - name: str
              - value: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/secrets/create",
            {},
            {},
            {},
            {},
            data,
            requests.SecretCreate,
            False,
            {"201": Secret, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_secret(
        cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[Secret], SecretListPaginated]:
        """
        List Secret

        **Permission Required:** `kelvin.permission.secret.read`.

        ``listSecret``: ``GET`` ``/api/v4/secrets/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search secrets by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/secrets/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": SecretListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"secret_name": "name"})
    def delete_secret(
        self_or_cls, secret_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Secret

        **Permission Required:** `kelvin.permission.secret.delete`.

        ``deleteSecret``: ``POST`` ``/api/v4/secrets/{secret_name}/delete``

        Parameters
        ----------
        secret_name : :obj:`str`, optional
            Secret name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/secrets/{secret_name}/delete",
            {"secret_name": secret_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class SecretListPaginated(PaginatorDataModel[Secret]):
    """
    SecretListPaginated object.

    Parameters
    ----------
    data: Optional[List[Secret]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[Secret]] = None
    pagination: Optional[pagination.Pagination] = None


class SemanticTypeItem(DataModelBase):
    """
    SemanticTypeItem object.

    Parameters
    ----------
    created: Optional[datetime]
    name: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    name: Optional[str] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None


class SemanticTypeList(DataModelBase):
    """
    SemanticTypeList object.

    Parameters
    ----------
    data: Optional[List[SemanticTypeItem]]

    """

    data: Optional[List[SemanticTypeItem]] = None

    @classmethod
    def list_semantic_type(
        cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> SemanticTypeList:
        """
        List Semantic Type

        **Sortable Columns:** `semantic_type.name`, `semantic_type.title`,
        `semantic_type.created`, `semantic_type.updated`

        **Permission Required:** `kelvin.permission.metric.read`.

        ``listSemanticType``: ``GET`` ``/api/v4/metrics/semantictypes/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter semantic types by name
        search : :obj:`Sequence[str]`
            Search semantic types by name or title
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/metrics/semantictypes/list",
            {},
            {"names": names, "search": search, "sort_by": sort_by, "sort_asc": sort_asc},
            {},
            {},
            None,
            None,
            False,
            {"200": SemanticTypeList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class SimpleAsset(DataModelBase):
    """
    SimpleAsset object.

    Parameters
    ----------
    name: Optional[str]
    state: Optional[str]

    """

    name: Optional[str] = None
    state: Optional[str] = None


class SourceAssets(DataModelBase):
    """
    SourceAssets object.

    Parameters
    ----------
    name: Optional[str]
    title: Optional[str]

    """

    name: Optional[str] = None
    title: Optional[str] = None


class StatusStatistics(DataModelBase):
    """
    StatusStatistics object.

    Parameters
    ----------
    acknowledged: Optional[int]
    active: Optional[int]
    resolved: Optional[int]

    """

    acknowledged: Optional[int] = None
    active: Optional[int] = None
    resolved: Optional[int] = None


class StorageData(DataModelBase):
    """
    StorageData object.

    Parameters
    ----------
    _insertion_timestamp: Optional[datetime]
    asset_name: Optional[str]
    name: Optional[str]
    payload: Optional[Dict[str, Any]]
    timestamp: Optional[datetime]

    """

    _insertion_timestamp: Optional[datetime] = None
    asset_name: Optional[str] = Field(None, max_length=64)
    name: Optional[str] = Field(None, max_length=64)
    payload: Optional[Dict[str, Any]] = None
    timestamp: Optional[datetime] = None


class StorageMetric(DataModelBase):
    """
    StorageMetric object.

    Parameters
    ----------
    asset_name: Optional[str]
    created: Optional[datetime]
    fields: Optional[List[str]]
    name: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]

    """

    asset_name: Optional[str] = Field(None, max_length=64)
    created: Optional[datetime] = None
    fields: Optional[List[str]] = None
    name: Optional[str] = Field(None, max_length=64)
    title: Optional[str] = Field(None, max_length=64)
    updated: Optional[datetime] = None


class StorageMetricListPaginated(PaginatorDataModel[StorageMetric]):
    """
    StorageMetricListPaginated object.

    Parameters
    ----------
    data: Optional[List[StorageMetric]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[StorageMetric]] = None
    pagination: Optional[pagination.Pagination] = None


class User(DataModelBase):
    """
    User object.

    Parameters
    ----------
    created: Optional[datetime]
    email: Optional[EmailStr]
    first_name: Optional[str]
    id: Optional[UUID]
    last_name: Optional[str]
    username: Optional[str]

    """

    created: Optional[datetime] = None
    email: Optional[EmailStr] = None
    first_name: Optional[str] = Field(None, max_length=64)
    id: Optional[UUID] = None
    last_name: Optional[str] = Field(None, max_length=64)
    username: Optional[str] = None

    @instance_classmethod
    @DataModelBase.translate()
    def list_user(
        self_or_cls,
        username: Optional[Sequence[str]] = None,
        email: Optional[Sequence[str]] = None,
        name: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[User], UserListPaginated]:
        """
        List User

        **Pagination Sortable Columns:** `user_entity.id`, `user_entity.username`, `user_entity.first_name`, `user_entity.last_name`, `user_entity.email`

        **Permission Required:** `kelvin.permission.user.read`.

        ``listUser``: ``GET`` ``/api/v4/users/list``

        Parameters
        ----------
        username : :obj:`Sequence[str]`
            User's username
        email : :obj:`Sequence[str]`
            User's email
        name : :obj:`Sequence[str]`
            User's name
        search : :obj:`Sequence[str]`
            Search Users by name or email
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/users/list",
            {},
            {
                "username": username,
                "email": email,
                "name": name,
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": UserListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def get_current_user(cls, _dry_run: bool = False, _client: Any = None) -> UserWithPermissions:
        """
        Get Current User

        **Permission Required:** `n/a`.

        ``getCurrentUser``: ``GET`` ``/api/v4/users/me``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/users/me",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": UserWithPermissions, "400": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"user_id": "id"})
    def get_user(self_or_cls, user_id: str, _dry_run: bool = False, _client: Any = None) -> User:
        """
        Get User

        **Permission Required:** `kelvin.permission.user.read`.

        ``getUser``: ``GET`` ``/api/v4/users/{user_id}/get``

        Parameters
        ----------
        user_id : :obj:`str`, optional
            User ID

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/users/{user_id}/get",
            {"user_id": user_id},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": User, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class UserListPaginated(PaginatorDataModel[User]):
    """
    UserListPaginated object.

    Parameters
    ----------
    data: Optional[List[User]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[User]] = None
    pagination: Optional[pagination.Pagination] = None


class UserWithPermissions(DataModelBase):
    """
    UserWithPermissions object.

    Parameters
    ----------
    created: Optional[datetime]
    email: Optional[EmailStr]
    first_name: Optional[str]
    id: Optional[UUID]
    last_name: Optional[str]
    permissions: Optional[List[str]]
    username: Optional[str]

    """

    created: Optional[datetime] = None
    email: Optional[EmailStr] = None
    first_name: Optional[str] = Field(None, max_length=64)
    id: Optional[UUID] = None
    last_name: Optional[str] = Field(None, max_length=64)
    permissions: Optional[List[str]] = None
    username: Optional[str] = None

    @instance_classmethod
    @DataModelBase.translate()
    def list_user(
        self_or_cls,
        username: Optional[Sequence[str]] = None,
        email: Optional[Sequence[str]] = None,
        name: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[User], UserListPaginated]:
        """
        List User

        **Pagination Sortable Columns:** `user_entity.id`, `user_entity.username`, `user_entity.first_name`, `user_entity.last_name`, `user_entity.email`

        **Permission Required:** `kelvin.permission.user.read`.

        ``listUser``: ``GET`` ``/api/v4/users/list``

        Parameters
        ----------
        username : :obj:`Sequence[str]`
            User's username
        email : :obj:`Sequence[str]`
            User's email
        name : :obj:`Sequence[str]`
            User's name
        search : :obj:`Sequence[str]`
            Search Users by name or email
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/users/list",
            {},
            {
                "username": username,
                "email": email,
                "name": name,
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": UserListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def get_current_user(cls, _dry_run: bool = False, _client: Any = None) -> UserWithPermissions:
        """
        Get Current User

        **Permission Required:** `n/a`.

        ``getCurrentUser``: ``GET`` ``/api/v4/users/me``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/users/me",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": UserWithPermissions, "400": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_user(cls, user_id: str, _dry_run: bool = False, _client: Any = None) -> User:
        """
        Get User

        **Permission Required:** `kelvin.permission.user.read`.

        ``getUser``: ``GET`` ``/api/v4/users/{user_id}/get``

        Parameters
        ----------
        user_id : :obj:`str`, optional
            User ID

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/users/{user_id}/get",
            {"user_id": user_id},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": User, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class WireguardPeer(DataModelBase):
    """
    WireguardPeer object.

    Parameters
    ----------
    allowed_ips: Optional[List[str]]
    created: Optional[datetime]
    endpoint: Optional[str]
    name: Optional[str]
    public_key: Optional[str]
    title: Optional[str]
    type: Optional[str]
    updated: Optional[datetime]
    wireguard_ip: Optional[str]

    """

    allowed_ips: Optional[List[str]] = None
    created: Optional[datetime] = None
    endpoint: Optional[str] = None
    name: Optional[str] = None
    public_key: Optional[str] = None
    title: Optional[str] = None
    type: Optional[str] = None
    updated: Optional[datetime] = None
    wireguard_ip: Optional[str] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_wireguard_peer(
        self_or_cls,
        data: Optional[Union[requests.WireguardPeerCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> WireguardPeer:
        """
        Create Wireguard Peer

        **Permission Required:** `kelvin.permission.acp.create`.

        ``createWireguardPeer``: ``POST`` ``/api/v4/wireguard/peers/create``

        Parameters
        ----------
        data: requests.WireguardPeerCreate, optional
        **kwargs:
            Extra parameters for requests.WireguardPeerCreate
              - allowed_ips: list
              - endpoint: str
              - name: str
              - public_key: str
              - title: str
              - type: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/wireguard/peers/create",
            {},
            {},
            {},
            {},
            data,
            requests.WireguardPeerCreate,
            False,
            {
                "201": WireguardPeer,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_wireguard_peer(
        cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[WireguardPeer], WireguardPeerListPaginated]:
        """
        List Wireguard Peer

        **Pagination Sortable Columns:** `wireguard_peer.name`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listWireguardPeer``: ``GET`` ``/api/v4/wireguard/peers/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search Peers by name, title or public key
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/wireguard/peers/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": WireguardPeerListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def get_wireguard_graph(
        cls, format: Optional[str] = None, _dry_run: bool = False, _client: Any = None
    ) -> WireguardTunnel:
        """
        Get Wireguard Graph returns a visual representation of all the tunnels in the system

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getWireguardGraph``: ``POST`` ``/api/v4/wireguard/peers/tunnels/graph/get``

        Parameters
        ----------
        format : :obj:`str`
            Desired return format (svg or dot)

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/wireguard/peers/tunnels/graph/get",
            {},
            {"format": format},
            {},
            {},
            None,
            None,
            False,
            {
                "200": WireguardTunnel,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "500": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @classmethod
    def delete_wireguard_peer(
        cls, peer_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Wireguard Peer

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteWireguardPeer``: ``POST`` ``/api/v4/wireguard/peers/{peer_name}/delete``

        Parameters
        ----------
        peer_name : :obj:`str`, optional
            Peer name

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/wireguard/peers/{peer_name}/delete",
            {"peer_name": peer_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_wireguard_peer(
        cls, peer_name: str, _dry_run: bool = False, _client: Any = None
    ) -> WireguardPeer:
        """
        Get Wireguard Peer

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getWireguardPeer``: ``GET`` ``/api/v4/wireguard/peers/{peer_name}/get``

        Parameters
        ----------
        peer_name : :obj:`str`, optional
            Peer name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/wireguard/peers/{peer_name}/get",
            {"peer_name": peer_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": WireguardPeer, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessage},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def create_wireguard_tunnel(
        cls,
        peer_name: str,
        data: Optional[Union[requests.WireguardTunnelCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> WireguardTunnel:
        """
        Create Wireguard Tunnel

        **Permission Required:** `kelvin.permission.acp.create`.

        ``createWireguardTunnel``: ``POST`` ``/api/v4/wireguard/peers/{peer_name}/tunnels/create``

        Parameters
        ----------
        peer_name : :obj:`str`, optional
            Source peer
        data: requests.WireguardTunnelCreate, optional
        **kwargs:
            Extra parameters for requests.WireguardTunnelCreate
              - keep_alive: int
              - wireguard_peer: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/wireguard/peers/{peer_name}/tunnels/create",
            {"peer_name": peer_name},
            {},
            {},
            {},
            data,
            requests.WireguardTunnelCreate,
            False,
            {
                "201": WireguardTunnel,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def delete_wireguard_tunnel(
        cls, peer_name: str, _dry_run: bool = False, _client: Any = None
    ) -> WireguardTunnel:
        """
        Delete Wireguard Tunnel

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteWireguardTunnel``: ``POST`` ``/api/v4/wireguard/peers/{peer_name}/tunnels/delete``

        Parameters
        ----------
        peer_name : :obj:`str`, optional
            Source peer

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/wireguard/peers/{peer_name}/tunnels/delete",
            {"peer_name": peer_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": WireguardTunnel,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @classmethod
    def list_wireguard_tunnel(
        cls,
        peer_name: str,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[WireguardPeer], WireguardPeerListPaginated]:
        """
        List Wireguard Tunnel

        **Pagination Sortable Columns:** `wireguard_tunnel.wg_src`, `wireguard_tunnel.wg_peer`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listWireguardTunnel``: ``GET`` ``/api/v4/wireguard/peers/{peer_name}/tunnels/list``

        Parameters
        ----------
        peer_name : :obj:`str`, optional
            Source peer
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/wireguard/peers/{peer_name}/tunnels/list",
            {"peer_name": peer_name},
            {
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": WireguardPeerListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def update_wireguard_peer(
        self_or_cls,
        peer_name: str,
        data: Optional[Union[requests.WireguardPeerUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> WireguardPeer:
        """
        Update Wireguard Peer

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateWireguardPeer``: ``POST`` ``/api/v4/wireguard/peers/{peer_name}/update``

        Parameters
        ----------
        peer_name : :obj:`str`, optional
            Peer name
        data: requests.WireguardPeerUpdate, optional
        **kwargs:
            Extra parameters for requests.WireguardPeerUpdate
              - allowed_ips: list
              - endpoint: str
              - public_key: str
              - title: str
              - type: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/wireguard/peers/{peer_name}/update",
            {"peer_name": peer_name},
            {},
            {},
            {},
            data,
            requests.WireguardPeerUpdate,
            False,
            {
                "200": WireguardPeer,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessage,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class WireguardPeerListPaginated(PaginatorDataModel[WireguardPeer]):
    """
    WireguardPeerListPaginated object.

    Parameters
    ----------
    data: Optional[List[WireguardPeer]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[WireguardPeer]] = None
    pagination: Optional[pagination.Pagination] = None


class WireguardTunnel(DataModelBase):
    """
    WireguardTunnel object.

    Parameters
    ----------
    created: Optional[datetime]
    keep_alive: Optional[int]
    updated: Optional[datetime]
    wireguard_peer: Optional[str]

    """

    created: Optional[datetime] = None
    keep_alive: Optional[int] = None
    updated: Optional[datetime] = None
    wireguard_peer: Optional[str] = None

    @classmethod
    def get_wireguard_graph(
        cls, format: Optional[str] = None, _dry_run: bool = False, _client: Any = None
    ) -> WireguardTunnel:
        """
        Get Wireguard Graph returns a visual representation of all the tunnels in the system

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getWireguardGraph``: ``POST`` ``/api/v4/wireguard/peers/tunnels/graph/get``

        Parameters
        ----------
        format : :obj:`str`
            Desired return format (svg or dot)

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/wireguard/peers/tunnels/graph/get",
            {},
            {"format": format},
            {},
            {},
            None,
            None,
            False,
            {
                "200": WireguardTunnel,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "500": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_wireguard_tunnel(
        self_or_cls,
        peer_name: str,
        data: Optional[Union[requests.WireguardTunnelCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> WireguardTunnel:
        """
        Create Wireguard Tunnel

        **Permission Required:** `kelvin.permission.acp.create`.

        ``createWireguardTunnel``: ``POST`` ``/api/v4/wireguard/peers/{peer_name}/tunnels/create``

        Parameters
        ----------
        peer_name : :obj:`str`, optional
            Source peer
        data: requests.WireguardTunnelCreate, optional
        **kwargs:
            Extra parameters for requests.WireguardTunnelCreate
              - keep_alive: int
              - wireguard_peer: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/wireguard/peers/{peer_name}/tunnels/create",
            {"peer_name": peer_name},
            {},
            {},
            {},
            data,
            requests.WireguardTunnelCreate,
            False,
            {
                "201": WireguardTunnel,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def delete_wireguard_tunnel(
        cls, peer_name: str, _dry_run: bool = False, _client: Any = None
    ) -> WireguardTunnel:
        """
        Delete Wireguard Tunnel

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteWireguardTunnel``: ``POST`` ``/api/v4/wireguard/peers/{peer_name}/tunnels/delete``

        Parameters
        ----------
        peer_name : :obj:`str`, optional
            Source peer

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/wireguard/peers/{peer_name}/tunnels/delete",
            {"peer_name": peer_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": WireguardTunnel,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @classmethod
    def list_wireguard_tunnel(
        cls,
        peer_name: str,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[WireguardPeer], WireguardPeerListPaginated]:
        """
        List Wireguard Tunnel

        **Pagination Sortable Columns:** `wireguard_tunnel.wg_src`, `wireguard_tunnel.wg_peer`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listWireguardTunnel``: ``GET`` ``/api/v4/wireguard/peers/{peer_name}/tunnels/list``

        Parameters
        ----------
        peer_name : :obj:`str`, optional
            Source peer
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/wireguard/peers/{peer_name}/tunnels/list",
            {"peer_name": peer_name},
            {
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": WireguardPeerListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result


class WorkloadAllocation(DataModelBase):
    """
    WorkloadAllocation object.

    Parameters
    ----------
    cpu_requests: Optional[float]
    memory_requests: Optional[float]

    """

    cpu_requests: Optional[float] = None
    memory_requests: Optional[float] = None


class WorkloadCPUUtilization(DataModelBase):
    """
    WorkloadCPUUtilization object.

    Parameters
    ----------
    timestamp: Optional[int]
    value: Optional[float]

    """

    timestamp: Optional[int] = None
    value: Optional[float] = None


class WorkloadLogs(DataModelBase):
    """
    WorkloadLogs object.

    Parameters
    ----------
    logs: Optional[Dict[str, List[str]]]

    """

    logs: Optional[Dict[str, List[str]]] = None

    @classmethod
    def get_workload_logs(
        cls,
        workload_name: str,
        tail_lines: Optional[str] = None,
        since_time: Optional[str] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> WorkloadLogs:
        """
        Get Workload Logs

        **Permission Required:** `kelvin.permission.workload.read`.

        ``getWorkloadLogs``: ``GET`` ``/api/v4/workloads/{workload_name}/logs/get``

        Parameters
        ----------
        workload_name : :obj:`str`, optional
            Workload name
        tail_lines : :obj:`str`
            The number of lines from the end of the logs to show
        since_time : :obj:`str`
            A timestamp from which to show logs

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/workloads/{workload_name}/logs/get",
            {"workload_name": workload_name},
            {"tail_lines": tail_lines, "since_time": since_time},
            {},
            {},
            None,
            None,
            False,
            {
                "200": WorkloadLogs,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "500": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result


class WorkloadMemoryUsage(DataModelBase):
    """
    WorkloadMemoryUsage object.

    Parameters
    ----------
    timestamp: Optional[int]
    value: Optional[float]

    """

    timestamp: Optional[int] = None
    value: Optional[float] = None


class WorkloadNetwork(DataModelBase):
    """
    WorkloadNetwork object.

    Parameters
    ----------
    total_rx: Optional[float]
    total_tx: Optional[float]

    """

    total_rx: Optional[float] = None
    total_tx: Optional[float] = None


class WorkloadStatus(DataModelBase):
    """
    WorkloadStatus object.

    Parameters
    ----------
    last_seen: Optional[datetime]
    message: Optional[str]
    state: Optional[str]
    warnings: Optional[List[str]]

    """

    last_seen: Optional[datetime] = None
    message: Optional[str] = None
    state: Optional[str] = None
    warnings: Optional[List[str]] = None


class WorkloadStatusValuesItem(DataModelBase):
    """
    WorkloadStatusValuesItem object.

    Parameters
    ----------
    last_seen: Optional[datetime]
    message: Optional[str]
    name: Optional[str]
    state: Optional[str]
    warnings: Optional[List[str]]

    """

    last_seen: Optional[datetime] = None
    message: Optional[str] = None
    name: Optional[str] = None
    state: Optional[str] = None
    warnings: Optional[List[str]] = None


class WorkloadTelemetry(DataModelBase):
    """
    WorkloadTelemetry object.

    Parameters
    ----------
    allocation: Optional[WorkloadAllocation]
    cpu_utilization: Optional[List[WorkloadCPUUtilization]]
    memory_usage: Optional[List[WorkloadMemoryUsage]]
    network: Optional[WorkloadNetwork]

    """

    allocation: Optional[WorkloadAllocation] = None
    cpu_utilization: Optional[List[WorkloadCPUUtilization]] = None
    memory_usage: Optional[List[WorkloadMemoryUsage]] = None
    network: Optional[WorkloadNetwork] = None

    @classmethod
    def get_workload_telemetry(
        cls, workload_name: str, _dry_run: bool = False, _client: Any = None
    ) -> WorkloadTelemetry:
        """
        Get Workload Telemetry

        **Permission Required:** `kelvin.permission.workload.read`.

        ``getWorkloadTelemetry``: ``GET`` ``/api/v4/workloads/{workload_name}/telemetry/get``

        Parameters
        ----------
        workload_name : :obj:`str`, optional
            Workload name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/workloads/{workload_name}/telemetry/get",
            {"workload_name": workload_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": WorkloadTelemetry, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class Storage(DataModelBase):
    """
    Storage object.

    Parameters
    ----------

    """

    @classmethod
    def create_historian_metric(
        cls,
        data: Optional[Union[requests.StorageBulkCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Create Historian Metric persists time series data.

        **Note:**
        * If a Metric timestamp already exists on the database, this endpoint will
        overwrite the stored payload with the new payload value.

        **Payload Details:**
        * `name`: name of the Metric
        * `source`: application that generated the data. Example: `modbus`
        * `asset_name`: name of the Asset
        * `payload`: data to persist
        * `timestamp`: RFC3339 timestamps with added micro-second precision

        **Permission Required:** `kelvin.permission.storage.create`.

        ``createHistorianMetric``: ``POST`` ``/api/v4/storage/historian/create``

        Parameters
        ----------
        data: requests.StorageBulkCreate, optional
        **kwargs:
            Extra parameters for requests.StorageBulkCreate
              - data: list

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/storage/historian/create",
            {},
            {},
            {},
            {},
            data,
            requests.StorageBulkCreate,
            False,
            {"201": None, "207": ErrorMessages, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def get_historian_metric_last(
        cls,
        name: str,
        asset_name: str,
        fields: Optional[str] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> StorageData:
        """
        Get Historian Metric Last returns the last value inserted on the
        time series database for a single Metric. The calculation is
        based on the most recent timestamp available for the Metric.

        **Permission Required:** `kelvin.permission.storage.read`.

        ``getHistorianMetricLast``: ``GET`` ``/api/v4/storage/historian/last/get``

        Parameters
        ----------
        name : :obj:`str`, optional
            Metric Name
        asset_name : :obj:`str`, optional
            Asset name
        fields : :obj:`str`
            List of field identifiers

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/storage/historian/last/get",
            {},
            {"name": name, "asset_name": asset_name, "fields": fields},
            {},
            {},
            None,
            None,
            False,
            {"200": StorageData, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_historian_metric_last_advanced(
        cls,
        data: Optional[Union[requests.StorageLast, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Iterator[StorageData]:
        """
        Get Historian Metric Last Advanced returns the last values inserted on the
        time series database for a list of Metric selectors. The calculation is
        based on the most recent timestamp available for each Metric.

        **Permission Required:** `kelvin.permission.storage.read`.

        ``getHistorianMetricLastAdvanced``: ``POST`` ``/api/v4/storage/historian/last/get``

        Parameters
        ----------
        data: requests.StorageLast, optional
        **kwargs:
            Extra parameters for requests.StorageLast
              - selectors: list

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/storage/historian/last/get",
            {},
            {},
            {},
            {},
            data,
            requests.StorageLast,
            False,
            {"200": StorageData, "401": ErrorMessages, "404": ErrorMessages},
            True,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_historian_metric(
        cls,
        name: Optional[Sequence[str]] = None,
        asset_name: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[StorageMetric], StorageMetricListPaginated]:
        """
        List Historian Metric

        **Pagination Sortable Columns:** `storage_metric.name`, `storage_metric.asset_name`

        **Permission Required:** `kelvin.permission.storage.read`.

        ``listHistorianMetric``: ``GET`` ``/api/v4/storage/historian/list``

        Parameters
        ----------
        name : :obj:`Sequence[str]`
            Metric name
        asset_name : :obj:`Sequence[str]`
            Asset name
        search : :obj:`Sequence[str]`
            Search metrics by name or asset_name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/storage/historian/list",
            {},
            {
                "name": name,
                "asset_name": asset_name,
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": StorageMetricListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def delete_historian_range(
        cls,
        data: Optional[Union[requests.StorageDelete, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Delete Historian Range will delete all associated time series values between a given time-range.

        **Payload Details:**
        * `start_time`: supports RFC3339 timestamps with added micro-second precision
        * `end_time`: supports RFC3339 timestamps with added micro-second precision

        **Permission Required:** `kelvin.permission.storage.delete`.

        ``deleteHistorianRange``: ``POST`` ``/api/v4/storage/historian/range/delete``

        Parameters
        ----------
        data: requests.StorageDelete, optional
        **kwargs:
            Extra parameters for requests.StorageDelete
              - end_time: datetime
              - selectors: list
              - start_time: datetime

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/storage/historian/range/delete",
            {},
            {},
            {},
            {},
            data,
            requests.StorageDelete,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def download_historian_metrics(
        cls,
        data: Optional[Union[requests.StorageRangeAdvanced, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Download Historian Metrics returns a **CSV file** with time series data for a given time range
        List of Metric selectors.

        **Payload Description:**
        * `start_time` supports RFC3339 timestamps with added micro-second precision
        * `end_time` supports RFC3339 timestamps with added micro-second precision
        * `time_bucket` uses an human readable format which is covered in more detail in
        https://golang.org/pkg/time/#ParseDuration. Example: `5m`, `1h`, `1w`
        * `time_shift` is used to shift the time buckets from the epoch. Every time bucket starts from epoch
        so, if you want to calculate the time bucket starting on a different date and time, you'll need to
        set this. Example: `5m`, `1h`, `1w`
        * `fill` allows you to fill missing points from a time bucket. Check below for supported fill values.
        * `agg` allows you to choose an aggregation. Check below for supported aggregations.

        **Supported Fill:**
        * `none`: Default behaviour.
        * `null`: Fills the missing points with null string
        * `linear`: Performs a linear regression
        * `previous`: Uses the previous non-empty value
        * `int value`: Fills the missing points with the specified number

        **Supported Aggregations:**
        * Numeric types (`int32`, `int64`, etc) are:
        `none`, `count`, `distinct`, `integral`, `mean`, `median`, `mode`, `spread`, `stddev`, `sum`, `max` & `min`
        * Non-numeric types (`text`, etc) are:
        `count`, `distinct` and `mode`

        **Permission Required:** `kelvin.permission.storage.read`.

        ``downloadHistorianMetrics``: ``POST`` ``/api/v4/storage/historian/range/download``

        Parameters
        ----------
        data: requests.StorageRangeAdvanced, optional
        **kwargs:
            Extra parameters for requests.StorageRangeAdvanced
              - agg: str
              - end_time: datetime
              - fill: str
              - group_by_selector: bool
              - order: str
              - selectors: list
              - start_time: datetime
              - time_bucket: str
              - time_shift: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/storage/historian/range/download",
            {},
            {},
            {},
            {},
            data,
            requests.StorageRangeAdvanced,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def get_historian_metric_range(
        cls,
        name: str,
        asset_name: str,
        start_time: datetime,
        end_time: datetime,
        agg: Optional[
            Literal[
                "none",
                "count",
                "distinct",
                "integral",
                "mean",
                "median",
                "mode",
                "spread",
                "stddev",
                "sum",
                "max",
                "min",
            ]
        ] = None,
        time_bucket: Optional[str] = None,
        time_shift: Optional[str] = None,
        fill: Optional[str] = None,
        order: Optional[str] = None,
        fields: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Iterator[StorageData]:
        """
        Get Historian Metric Range returns time series data for a given time range
        and a list of Metric selectors.

        **Payload Description:**
        * `start_time` supports RFC3339 timestamps with added micro-second precision
        * `end_time` supports RFC3339 timestamps with added micro-second precision
        * `time_bucket` uses an human readable format which is covered in more detail in
        https://golang.org/pkg/time/#ParseDuration. Example: `5m`, `1h`, `1w`
        * `time_shift` is used to shift the time buckets from the epoch. Every time bucket starts from epoch
        so, if you want to calculate the time bucket starting on a different date and time, you'll need to
        set this. Example: `5m`, `1h`, `1w`
        * `fill` allows you to fill missing points from a time bucket. Check below for supported fill values.
        * `agg` allows you to choose an aggregation. Check below for supported aggregations.

        **Supported Fill:**
        * `none`: Default behaviour.
        * `null`: Fills the missing points with null string
        * `linear`: Performs a linear regression
        * `previous`: Uses the previous non-empty value
        * `int value`: Fills the missing points with the specified number

        **Supported Aggregations:**
        * Numeric types (`int32`, `int64`, etc) are:
        `none`, `count`, `distinct`, `integral`, `mean`, `median`, `mode`, `spread`, `stddev`, `sum`, `max` & `min`
        * Non-numeric types (`text`, etc) are:
        `count`, `distinct` and `mode`

        **Permission Required:** `kelvin.permission.storage.read`.

        ``getHistorianMetricRange``: ``GET`` ``/api/v4/storage/historian/range/get``

        Parameters
        ----------
        name : :obj:`str`, optional
            Metric Name
        asset_name : :obj:`str`, optional
            Asset name
        start_time : :obj:`datetime`, optional
            Value for the start of the range for which to get data
        end_time : :obj:`datetime`, optional
            Value for the end of the range for which to get data
        agg : :obj:`Literal['none', 'count', 'distinct', 'integral', 'mean', 'median', 'mode', 'spread', 'stddev', 'sum', 'max', 'min']`
            Aggregation (none, count, distinct, integral, mean, median, mode,
            spread, stddev, sum, max, min)
        time_bucket : :obj:`str`
            Interval for which to group by, in human readable format
        time_shift : :obj:`str`
            Interval by which to shift the time buckets, from epoch
        fill : :obj:`str`
            Filling for empty time buckets (check description for possible values)
        order : :obj:`str`
            Order of the data
        fields : :obj:`Sequence[str]`
            Fields

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/storage/historian/range/get",
            {},
            {
                "name": name,
                "asset_name": asset_name,
                "start_time": start_time,
                "end_time": end_time,
                "agg": agg,
                "time_bucket": time_bucket,
                "time_shift": time_shift,
                "fill": fill,
                "order": order,
                "fields": fields,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": StorageData, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            True,
            _dry_run,
        )
        return result

    @classmethod
    def get_historian_metric_advanced_range(
        cls,
        data: Optional[Union[requests.StorageRangeAdvanced, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Iterator[StorageData]:
        """
        Get Historian Metric Advanced Range returns time series data for a given time range
        and a list of Metric selectors.

        **Payload Description:**
        * `start_time` supports RFC3339 timestamps with added micro-second precision
        * `end_time` supports RFC3339 timestamps with added micro-second precision
        * `time_bucket` uses an human readable format which is covered in more detail in
        https://golang.org/pkg/time/#ParseDuration. Example: `5m`, `1h`, `1w`
        * `time_shift` is used to shift the time buckets from the epoch. Every time bucket starts from epoch
        so, if you want to calculate the time bucket starting on a different date and time, you'll need to
        set this. Example: `5m`, `1h`, `1w`
        * `fill` allows you to fill missing points from a time bucket. Check below for supported fill values.
        * `agg` allows you to choose an aggregation. Check below for supported aggregations.

        **Supported Fill:**
        * `none`: Default behaviour.
        * `null`: Fills the missing points with null string
        * `linear`: Performs a linear regression
        * `previous`: Uses the previous non-empty value
        * `int value`: Fills the missing points with the specified number

        **Supported Aggregations:**
        * Numeric types (`int32`, `int64`, etc) are:
        `none`, `count`, `distinct`, `integral`, `mean`, `median`, `mode`, `spread`, `stddev`, `sum`, `max` & `min`
        * Non-numeric types (`text`, etc) are:
        `count`, `distinct` and `mode`

        **Permission Required:** `kelvin.permission.storage.read`.

        ``getHistorianMetricAdvancedRange``: ``POST`` ``/api/v4/storage/historian/range/get``

        Parameters
        ----------
        data: requests.StorageRangeAdvanced, optional
        **kwargs:
            Extra parameters for requests.StorageRangeAdvanced
              - agg: str
              - end_time: datetime
              - fill: str
              - group_by_selector: bool
              - order: str
              - selectors: list
              - start_time: datetime
              - time_bucket: str
              - time_shift: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/storage/historian/range/get",
            {},
            {},
            {},
            {},
            data,
            requests.StorageRangeAdvanced,
            False,
            {"200": StorageData, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            True,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def update_historian_metric(
        cls,
        data: Optional[Union[requests.StorageUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Update Historian Metric allows updating the **title** of a Metric.

        **Permission Required:** `kelvin.permission.storage.update`.

        ``updateHistorianMetric``: ``POST`` ``/api/v4/storage/historian/update``

        Parameters
        ----------
        data: requests.StorageUpdate, optional
        **kwargs:
            Extra parameters for requests.StorageUpdate
              - asset_name: str
              - name: str
              - title: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/storage/historian/update",
            {},
            {},
            {},
            {},
            data,
            requests.StorageUpdate,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def upload_historian_metrics(
        cls, file: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Upload Historian Metrics allows sending a **CSV file** with time series data
        to be persisted.

        **Permission Required:** `kelvin.permission.storage.create`.

        ``uploadHistorianMetrics``: ``POST`` ``/api/v4/storage/historian/upload``

        Parameters
        ----------
        file : :obj:`str`, optional
            CSV File

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/storage/historian/upload",
            {},
            {},
            {"file": file},
            {},
            None,
            None,
            False,
            {"201": None, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class ACPAndWorkloadStatus(DataModelBase):
    """
    ACPAndWorkloadStatus object.

    Parameters
    ----------
    node: Optional[ACPStatusValues]
    workloads: Optional[List[WorkloadStatusValuesItem]]

    """

    node: Optional[ACPStatusValues] = None
    workloads: Optional[List[WorkloadStatusValuesItem]] = None


class ACPItem(DataModelBase):
    """
    ACPItem object.

    Parameters
    ----------
    created: Optional[datetime]
    metadata: Optional[Dict[str, Any]]
    name: Optional[str]
    status: Optional[ACPStatusValues]
    title: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    metadata: Optional[Dict[str, Any]] = None
    name: Optional[str] = Field(None, max_length=64)
    status: Optional[ACPStatusValues] = None
    title: Optional[str] = Field(None, max_length=64)
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def list_acp(
        self_or_cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        connected: Optional[bool] = None,
        metadata: Optional[Sequence[str]] = None,
        metadata_filter: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[ACPItem], ACPListPaginated]:
        """
        List ACP

        **Pagination Sortable Columns:** `acp.name`, `acp.title`, `acp.created`, `acp.updated`, `status_state`, `status_last_seen`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listACP``: ``GET`` ``/api/v4/acps/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter ACPs by name
        search : :obj:`Sequence[str]`
            Search ACP by name or title
        connected : :obj:`bool`
            Filter by connected ACPs. Possible Values: true/false. Default: none
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response
        metadata_filter : :obj:`Sequence[str]`
            Dot separated string to filter by metadata values
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/list",
            {},
            {
                "names": names,
                "search": search,
                "connected": connected,
                "metadata": metadata,
                "metadata_filter": metadata_filter,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": ACPListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def create_acp_metadata_bulk(
        cls,
        data: Optional[Union[requests.ACPMetadataBulkCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Create ACP Metadata Bulk

        **Permission Required:** `kelvin.permission.acp.create`.

        ``createACPMetadataBulk``: ``POST`` ``/api/v4/acps/metadata/create``

        Parameters
        ----------
        data: requests.ACPMetadataBulkCreate, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataBulkCreate

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/acps/metadata/create",
            {},
            {},
            {},
            {},
            data,
            requests.ACPMetadataBulkCreate,
            False,
            {
                "201": None,
                "207": ErrorMessages,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def delete_acp_metadata_bulk(
        cls,
        data: Optional[Union[requests.ACPMetadataBulkDelete, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Delete ACP Metadata Bulk

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteACPMetadataBulk``: ``POST`` ``/api/v4/acps/metadata/delete``

        Parameters
        ----------
        data: requests.ACPMetadataBulkDelete, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataBulkDelete

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/acps/metadata/delete",
            {},
            {},
            {},
            {},
            data,
            requests.ACPMetadataBulkDelete,
            False,
            {
                "200": None,
                "207": ErrorMessages,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def get_acp_status_count(cls, _dry_run: bool = False, _client: Any = None) -> ACPStatusCount:
        """
        Get ACP Status Count

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPStatusCount``: ``GET`` ``/api/v4/acps/status/count/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/acps/status/count/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ACPStatusCount,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def delete_acp(self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None) -> None:
        """
        Delete ACP

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteACP``: ``POST`` ``/api/v4/acps/{acp_name}/delete``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/delete",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def force_update_acp_edge_app_version(
        self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Force Update ACP Edge App Version

        **Permission Required:** `kelvin.permission.acp.read`.

        ``forceUpdateACPEdgeAppVersion``: ``POST`` ``/api/v4/acps/{acp_name}/edge-apps/version/force-update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/edge-apps/version/force-update",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_acp_edge_app_version(
        self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> ACPEdgeAppVersion:
        """
        Get ACP Edge App Version

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPEdgeAppVersion``: ``GET`` ``/api/v4/acps/{acp_name}/edge-apps/version/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/edge-apps/version/get",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ACPEdgeAppVersion,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_acp(
        self_or_cls,
        acp_name: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> ACP:
        """
        Get ACP

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACP``: ``GET`` ``/api/v4/acps/{acp_name}/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/get",
            {"acp_name": acp_name},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {"200": ACP, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def create_acp_metadata(
        self_or_cls,
        acp_name: str,
        data: Optional[Union[requests.ACPMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPMetadata:
        """
        Create ACP Metadata

        **Permission Required:** `kelvin.permission.acp.create`.

        ``createACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/create``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        data: requests.ACPMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/create",
            {"acp_name": acp_name},
            {},
            {},
            {},
            data,
            requests.ACPMetadataCreate,
            False,
            {"201": ACPMetadata, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def list_acp_metadata(
        self_or_cls,
        acp_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[ACPMetaDataItem], ACPMetadataListPaginated]:
        """
        List ACP Metadata

        **Pagination Sortable Columns:** `acp_metadata.name`, `acp_metadata.acp_name`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listACPMetadata``: ``GET`` ``/api/v4/acps/{acp_name}/metadata/list``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        search : :obj:`Sequence[str]`
            Search ACP Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/metadata/list",
            {"acp_name": acp_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": ACPMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def delete_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete ACP Metadata

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/delete``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/delete",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> ACPMetadata:
        """
        Get ACP Metadata

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPMetadata``: ``GET`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/get",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": ACPMetadata, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def update_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        data: Optional[Union[requests.ACPMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPMetadata:
        """
        Update ACP Metadata

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name
        data: requests.ACPMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/update",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            data,
            requests.ACPMetadataUpdate,
            False,
            {
                "200": ACPMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def update_acp_status(
        self_or_cls,
        acp_name: str,
        data: Optional[Union[requests.ACPStatusUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPStatus:
        """
        Update ACP Status

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateACPStatus``: ``POST`` ``/api/v4/acps/{acp_name}/status/update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        data: requests.ACPStatusUpdate, optional
        **kwargs:
            Extra parameters for requests.ACPStatusUpdate
              - node: str
              - workloads: list

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/status/update",
            {"acp_name": acp_name},
            {},
            {},
            {},
            data,
            requests.ACPStatusUpdate,
            False,
            {"200": ACPStatus, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_acp_telemetry(
        self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> ACPTelemetry:
        """
        Get ACP Telemetry

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPTelemetry``: ``GET`` ``/api/v4/acps/{acp_name}/telemetry/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/telemetry/get",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": ACPTelemetry, "400": ErrorMessages, "401": ErrorMessages, "500": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def update_acp(
        self_or_cls,
        acp_name: str,
        data: Optional[Union[requests.ACPUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACP:
        """
        Update ACP

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateACP``: ``POST`` ``/api/v4/acps/{acp_name}/update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        data: requests.ACPUpdate, optional
        **kwargs:
            Extra parameters for requests.ACPUpdate
              - location: str
              - manifests_scrape_enabled: bool
              - manifests_scrape_interval: int
              - metrics_enabled: bool
              - metrics_scrape_interval: int
              - status_scrape_enabled: bool
              - status_scrape_interval: int
              - sync_scrape_interval: int
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/update",
            {"acp_name": acp_name},
            {},
            {},
            {},
            data,
            requests.ACPUpdate,
            False,
            {
                "200": ACP,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_workloads_manifests(
        self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> ACPWorkloadManifests:
        """
        Get Workloads Manifests assigned to an ACP

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getWorkloadsManifests``: ``GET`` ``/api/v4/acps/{acp_name}/workloads/manifests/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/workloads/manifests/get",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ACPWorkloadManifests,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result


class ACPListPaginated(PaginatorDataModel[ACPItem]):
    """
    ACPListPaginated object.

    Parameters
    ----------
    data: Optional[List[ACPItem]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[ACPItem]] = None
    pagination: Optional[pagination.Pagination] = None


class ACPSettings(DataModelBase):
    """
    ACPSettings object.

    Parameters
    ----------
    auto_update: Optional[ACPSettingsAutoUpdate]
    metrics: Optional[ACPSettingsMetrics]
    status: Optional[ACPSettingsStatus]

    """

    auto_update: Optional[ACPSettingsAutoUpdate] = None
    metrics: Optional[ACPSettingsMetrics] = None
    status: Optional[ACPSettingsStatus] = None


class ACPStatus(DataModelBase):
    """
    ACPStatus object.

    Parameters
    ----------
    name: Optional[str]
    status: Optional[ACPStatusValues]

    """

    name: Optional[str] = None
    status: Optional[ACPStatusValues] = None

    @classmethod
    def update_acp_status(
        cls,
        acp_name: str,
        data: Optional[Union[requests.ACPStatusUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPStatus:
        """
        Update ACP Status

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateACPStatus``: ``POST`` ``/api/v4/acps/{acp_name}/status/update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        data: requests.ACPStatusUpdate, optional
        **kwargs:
            Extra parameters for requests.ACPStatusUpdate
              - node: str
              - workloads: list

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/status/update",
            {"acp_name": acp_name},
            {},
            {},
            {},
            data,
            requests.ACPStatusUpdate,
            False,
            {"200": ACPStatus, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result


class Alarm(DataModelBase):
    """
    Alarm object.

    Parameters
    ----------
    acknowledged_at: Optional[datetime]
    acknowledged_by: Optional[str]
    alarm_class_name: Optional[str]
    alarm_class_title: Optional[str]
    alarm_type_name: Optional[str]
    alarm_type_title: Optional[str]
    created_at: Optional[datetime]
    created_by: Optional[CreatedBy]
    custom_identifier: Optional[str]
    description: Optional[str]
    id: Optional[str]
    payload: Optional[Dict[str, Any]]
    resolved_at: Optional[datetime]
    resolved_by: Optional[str]
    severity: Optional[int]
    solution: Optional[str]
    source_assets: Optional[List[SourceAssets]]
    status: Optional[str]
    status_payload: Optional[Dict[str, Any]]
    target_asset_class_name: Optional[str]
    target_asset_class_title: Optional[str]
    target_asset_name: Optional[str]
    target_asset_title: Optional[str]
    target_asset_type_name: Optional[str]
    target_asset_type_title: Optional[str]
    title: Optional[str]
    updated: Optional[datetime]

    """

    acknowledged_at: Optional[datetime] = None
    acknowledged_by: Optional[str] = None
    alarm_class_name: Optional[str] = None
    alarm_class_title: Optional[str] = None
    alarm_type_name: Optional[str] = None
    alarm_type_title: Optional[str] = None
    created_at: Optional[datetime] = None
    created_by: Optional[CreatedBy] = None
    custom_identifier: Optional[str] = None
    description: Optional[str] = None
    id: Optional[str] = None
    payload: Optional[Dict[str, Any]] = None
    resolved_at: Optional[datetime] = None
    resolved_by: Optional[str] = None
    severity: Optional[int] = None
    solution: Optional[str] = None
    source_assets: Optional[List[SourceAssets]] = None
    status: Optional[str] = None
    status_payload: Optional[Dict[str, Any]] = None
    target_asset_class_name: Optional[str] = None
    target_asset_class_title: Optional[str] = None
    target_asset_name: Optional[str] = None
    target_asset_title: Optional[str] = None
    target_asset_type_name: Optional[str] = None
    target_asset_type_title: Optional[str] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_alarm_class(
        self_or_cls,
        data: Optional[Union[requests.AlarmClassCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmClass:
        """
        Create Alarm Class

        **Permission Required:** `kelvin.permission.alarm_class.create`.

        ``createAlarmClass``: ``POST`` ``/api/v4/alarms/class/create``

        Parameters
        ----------
        data: requests.AlarmClassCreate, optional
        **kwargs:
            Extra parameters for requests.AlarmClassCreate
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/class/create",
            {},
            {},
            {},
            {},
            data,
            requests.AlarmClassCreate,
            False,
            {"201": AlarmClass, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_alarm_class(
        cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        owner: Optional[str] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AlarmClass], AlarmClassListPaginated]:
        """
        List Alarm Class

        **Pagination Sortable Columns:** `alarm_class.name`, `alarm_class.title`, `alarm_class.created`, `alarm_class.updated`, `alarm_class.owner`

        **Permission Required:** `kelvin.permission.alarm_class.read`.

        ``listAlarmClass``: ``GET`` ``/api/v4/alarms/class/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search Alarm Class by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        owner : :obj:`str`
            Filter Alarm Class by owner. Possible Values: kelvin/custom

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/class/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "owner": owner,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmClassListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_alarm_class(
        self_or_cls, alarm_class_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Alarm Class

        **Permission Required:** `kelvin.permission.alarm_class.delete`.

        ``deleteAlarmClass``: ``POST`` ``/api/v4/alarms/class/{alarm_class_name}/delete``

        Parameters
        ----------
        alarm_class_name : :obj:`str`, optional
            Alarm Class Name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/class/{alarm_class_name}/delete",
            {"alarm_class_name": alarm_class_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_alarm_class(
        self_or_cls, alarm_class_name: str, _dry_run: bool = False, _client: Any = None
    ) -> AlarmClass:
        """
        Get Alarm Class

        **Permission Required:** `kelvin.permission.alarm_class.read`.

        ``getAlarmClass``: ``GET`` ``/api/v4/alarms/class/{alarm_class_name}/get``

        Parameters
        ----------
        alarm_class_name : :obj:`str`, optional
            Alarm Class Name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/class/{alarm_class_name}/get",
            {"alarm_class_name": alarm_class_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmClass, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_alarm_class(
        self_or_cls,
        alarm_class_name: str,
        data: Optional[Union[requests.AlarmClassUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmClass:
        """
        Update Alarm Class

        **Permission Required:** `kelvin.permission.alarm_class.update`.

        ``updateAlarmClass``: ``POST`` ``/api/v4/alarms/class/{alarm_class_name}/update``

        Parameters
        ----------
        alarm_class_name : :obj:`str`, optional
            Alarm Class Name
        data: requests.AlarmClassUpdate, optional
        **kwargs:
            Extra parameters for requests.AlarmClassUpdate
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/class/{alarm_class_name}/update",
            {"alarm_class_name": alarm_class_name},
            {},
            {},
            {},
            data,
            requests.AlarmClassUpdate,
            False,
            {
                "200": AlarmClass,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_alarm(
        self_or_cls,
        data: Optional[Union[requests.AlarmCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Alarm:
        """
        Create Alarm

        **Permission Required:** `kelvin.permission.alarm.create`.

        ``createAlarm``: ``POST`` ``/api/v4/alarms/create``

        Parameters
        ----------
        data: requests.AlarmCreate, optional
        **kwargs:
            Extra parameters for requests.AlarmCreate
              - alarm_type_name: str
              - created_by: str
              - custom_identifier: str
              - description: str
              - payload: dict
              - severity: int
              - solution: str
              - source_asset_names: list
              - target_asset_name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/create",
            {},
            {},
            {},
            {},
            data,
            requests.AlarmCreate,
            False,
            {"201": Alarm, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_alarm_filter(
        self_or_cls,
        data: Optional[Union[requests.AlarmFilterCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmFilter:
        """
        Create Alarm Filter

        **Permission Required:** `kelvin.permission.alarm_filter.create`.

        ``createAlarmFilter``: ``POST`` ``/api/v4/alarms/filters/create``

        Parameters
        ----------
        data: requests.AlarmFilterCreate, optional
        **kwargs:
            Extra parameters for requests.AlarmFilterCreate
              - payload: dict
              - title: str
              - username: str
              - version: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/filters/create",
            {},
            {},
            {},
            {},
            data,
            requests.AlarmFilterCreate,
            False,
            {"201": AlarmFilter, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_alarm_filter(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        alarm_class_name: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AlarmFilter], AlarmFilterListPaginated]:
        """
        List Alarm Filter

        **Pagination Sortable Columns:** `alarm_filter.title`, `alarm_filter.username`, `alarm_filter.version`, `alarm_filter.created`, `alarm_filter.updated`

        **Permission Required:** `kelvin.permission.alarm_filter.read`.

        ``listAlarmFilter``: ``GET`` ``/api/v4/alarms/filters/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search alarm filter by title or version
        alarm_class_name : :obj:`Sequence[str]`
            Filter alarm filter by alarm class
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/filters/list",
            {},
            {
                "search": search,
                "alarm_class_name": alarm_class_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmFilterListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def delete_alarm_filter(
        cls, alarm_filter_id: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Alarm Filter

        **Permission Required:** `kelvin.permission.alarm_filter.delete`.

        ``deleteAlarmFilter``: ``POST`` ``/api/v4/alarms/filters/{alarm_filter_id}/delete``

        Parameters
        ----------
        alarm_filter_id : :obj:`str`, optional
            Alarm Filter id

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/filters/{alarm_filter_id}/delete",
            {"alarm_filter_id": alarm_filter_id},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_alarm_filter(
        cls,
        alarm_filter_id: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> AlarmFilter:
        """
        Get Alarm Filter

        **Permission Required:** `kelvin.permission.alarm_filter.read`.

        ``getAlarmFilter``: ``GET`` ``/api/v4/alarms/filters/{alarm_filter_id}/get``

        Parameters
        ----------
        alarm_filter_id : :obj:`str`, optional
            Alarm Filter ID
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/filters/{alarm_filter_id}/get",
            {"alarm_filter_id": alarm_filter_id},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmFilter, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_alarm_filter(
        self_or_cls,
        alarm_filter_id: str,
        data: Optional[Union[requests.AlarmFilterUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmFilter:
        """
        Update Alarm Filter

        **Permission Required:** `kelvin.permission.alarm_filter.update`.

        ``updateAlarmFilter``: ``POST`` ``/api/v4/alarms/filters/{alarm_filter_id}/update``

        Parameters
        ----------
        alarm_filter_id : :obj:`str`, optional
            Alarm Filter ID
        data: requests.AlarmFilterUpdate, optional
        **kwargs:
            Extra parameters for requests.AlarmFilterUpdate
              - payload: dict
              - title: str
              - version: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/filters/{alarm_filter_id}/update",
            {"alarm_filter_id": alarm_filter_id},
            {},
            {},
            {},
            data,
            requests.AlarmFilterUpdate,
            False,
            {
                "200": AlarmFilter,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_alarms(
        self_or_cls,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        data: Optional[Union[requests.AlarmList, Mapping[str, Any]]] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Union[Sequence[Alarm], AlarmListPaginated]:
        """
        List Alarms

        **Pagination Sortable Columns:** `alarm.id`, `alarm.title`, `alarm.alarm_type_name`, `alarm.asset_name`, `alarm.status`, `alarm.severity`, `alarm.created`, `alarm.updated`, `alarm.acknowledged_by`, `alarm.acknowledged_at`, `alarm.resolved_at`, `alarm.resolved_by`, `asset_type.name`, `asset_class.name`, `alarm_class.name`, `alarm_class.title`, `alarm_type.title`, `asset_class.title`, `asset_type.title`, `asset.title`

        **Permission Required:** `kelvin.permission.alarm.read`.

        ``listAlarms``: ``POST`` ``/api/v4/alarms/list``

        Parameters
        ----------
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        data: requests.AlarmList, optional
        **kwargs:
            Extra parameters for requests.AlarmList
              - acknowledged_at_end: datetime
              - acknowledged_at_start: datetime
              - acknowledged_by: list
              - alarm_class_names: list
              - alarm_severities: list
              - alarm_statuses: list
              - alarm_type_names: list
              - asset_class_names: list
              - asset_include_children: bool
              - asset_type_names: list
              - created_at_end: datetime
              - created_at_start: datetime
              - created_by: list
              - custom_identifiers: list
              - resolved_at_end: datetime
              - resolved_at_start: datetime
              - resolved_by: list
              - target_asset_names: list

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/list",
            {},
            {
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            data,
            requests.AlarmList,
            False,
            {"200": AlarmListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def get_asset_statistics(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> AlarmAssetsStatistics:
        """
        Get Asset Statistics

        **Permission Required:** `kelvin.permission.alarm.read`.

        ``getAssetStatistics``: ``GET`` ``/api/v4/alarms/statistics/assets/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/statistics/assets/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmAssetsStatistics, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_statistics(
        self_or_cls,
        data: Optional[Union[requests.AlarmList, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmStatistics:
        """
        Get Statistics

        **Permission Required:** `kelvin.permission.alarm.read`.

        ``getStatistics``: ``POST`` ``/api/v4/alarms/statistics/get``

        Parameters
        ----------
        data: requests.AlarmList, optional
        **kwargs:
            Extra parameters for requests.AlarmList
              - acknowledged_at_end: datetime
              - acknowledged_at_start: datetime
              - acknowledged_by: list
              - alarm_class_names: list
              - alarm_severities: list
              - alarm_statuses: list
              - alarm_type_names: list
              - asset_class_names: list
              - asset_include_children: bool
              - asset_type_names: list
              - created_at_end: datetime
              - created_at_start: datetime
              - created_by: list
              - custom_identifiers: list
              - resolved_at_end: datetime
              - resolved_at_start: datetime
              - resolved_by: list
              - target_asset_names: list

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/statistics/get",
            {},
            {},
            {},
            {},
            data,
            requests.AlarmList,
            False,
            {"200": AlarmStatistics, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_alarm_type(
        self_or_cls,
        data: Optional[Union[requests.AlarmTypeCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmType:
        """
        Create Alarm Type

        **Permission Required:** `kelvin.permission.alarm_type.create`.

        ``createAlarmType``: ``POST`` ``/api/v4/alarms/types/create``

        Parameters
        ----------
        data: requests.AlarmTypeCreate, optional
        **kwargs:
            Extra parameters for requests.AlarmTypeCreate
              - alarm_class_name: str
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/types/create",
            {},
            {},
            {},
            {},
            data,
            requests.AlarmTypeCreate,
            False,
            {"201": AlarmType, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_alarm_type(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        alarm_class_name: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        owner: Optional[str] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AlarmType], AlarmTypeListPaginated]:
        """
        List Alarm Type

        **Pagination Sortable Columns:** `alarm_type.name`, `alarm_type.title`, `alarm_type.created`, `alarm_type.updated`, `alarm_class.name`, `alarm_class.title`, `alarm_type.owner`

        **Permission Required:** `kelvin.permission.alarm_type.read`.

        ``listAlarmType``: ``GET`` ``/api/v4/alarms/types/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search alarm type by name or title
        alarm_class_name : :obj:`Sequence[str]`
            Filter alarm type by alarm class
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        owner : :obj:`str`
            Filter Alarm Type by owner. Possible Values: kelvin/custom

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/types/list",
            {},
            {
                "search": search,
                "alarm_class_name": alarm_class_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "owner": owner,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmTypeListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_alarm_type(
        self_or_cls, alarm_type_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Alarm Type

        **Permission Required:** `kelvin.permission.alarm_type.delete`.

        ``deleteAlarmType``: ``POST`` ``/api/v4/alarms/types/{alarm_type_name}/delete``

        Parameters
        ----------
        alarm_type_name : :obj:`str`, optional
            Alarm Type name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/types/{alarm_type_name}/delete",
            {"alarm_type_name": alarm_type_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_alarm_type(
        self_or_cls, alarm_type_name: str, _dry_run: bool = False, _client: Any = None
    ) -> AlarmType:
        """
        Get Alarm Type

        **Permission Required:** `kelvin.permission.alarm_type.read`.

        ``getAlarmType``: ``GET`` ``/api/v4/alarms/types/{alarm_type_name}/get``

        Parameters
        ----------
        alarm_type_name : :obj:`str`, optional
            Alarm Type name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/types/{alarm_type_name}/get",
            {"alarm_type_name": alarm_type_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmType, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_alarm_type(
        self_or_cls,
        alarm_type_name: str,
        data: Optional[Union[requests.AlarmTypeUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmType:
        """
        Update Alarm Type

        **Permission Required:** `kelvin.permission.alarm_type.update`.

        ``updateAlarmType``: ``POST`` ``/api/v4/alarms/types/{alarm_type_name}/update``

        Parameters
        ----------
        alarm_type_name : :obj:`str`, optional
            Alarm Type name
        data: requests.AlarmTypeUpdate, optional
        **kwargs:
            Extra parameters for requests.AlarmTypeUpdate
              - alarm_class_name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/types/{alarm_type_name}/update",
            {"alarm_type_name": alarm_type_name},
            {},
            {},
            {},
            data,
            requests.AlarmTypeUpdate,
            False,
            {
                "200": AlarmType,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_id": "id"})
    def delete_alarm(
        self_or_cls, alarm_id: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Alarm

        **Permission Required:** `kelvin.permission.alarm.delete`.

        ``deleteAlarm``: ``POST`` ``/api/v4/alarms/{alarm_id}/delete``

        Parameters
        ----------
        alarm_id : :obj:`str`, optional
            Alarm UUID

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/{alarm_id}/delete",
            {"alarm_id": alarm_id},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_id": "id"})
    def get_alarm(self_or_cls, alarm_id: str, _dry_run: bool = False, _client: Any = None) -> Alarm:
        """
        Get Alarm

        **Permission Required:** `kelvin.permission.alarm.read`.

        ``getAlarm``: ``GET`` ``/api/v4/alarms/{alarm_id}/get``

        Parameters
        ----------
        alarm_id : :obj:`str`, optional
            Alarm UUID

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/{alarm_id}/get",
            {"alarm_id": alarm_id},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": Alarm, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_id": "id"})
    def update_status(
        self_or_cls,
        alarm_id: str,
        data: Optional[Union[requests.AlarmUpdateStatus, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Update Status

        **Permission Required:** `kelvin.permission.alarm.update`.

        ``updateStatus``: ``POST`` ``/api/v4/alarms/{alarm_id}/status/update``

        Parameters
        ----------
        alarm_id : :obj:`str`, optional
            Alarm UUID
        data: requests.AlarmUpdateStatus, optional
        **kwargs:
            Extra parameters for requests.AlarmUpdateStatus
              - status: str
              - status_payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/{alarm_id}/status/update",
            {"alarm_id": alarm_id},
            {},
            {},
            {},
            data,
            requests.AlarmUpdateStatus,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"alarm_id": "id"})
    def update(
        self_or_cls,
        alarm_id: str,
        data: Optional[Union[requests.AlarmUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Alarm:
        """
        Update Alarm

        **Permission Required:** `kelvin.permission.alarm.update`.

        ``update``: ``POST`` ``/api/v4/alarms/{alarm_id}/update``

        Parameters
        ----------
        alarm_id : :obj:`str`, optional
            Alarm UUID
        data: requests.AlarmUpdate, optional
        **kwargs:
            Extra parameters for requests.AlarmUpdate
              - alarm_type_name: str
              - custom_identifier: str
              - description: str
              - payload: dict
              - severity: int
              - solution: str
              - source_asset_names: list
              - target_asset_name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/{alarm_id}/update",
            {"alarm_id": alarm_id},
            {},
            {},
            {},
            data,
            requests.AlarmUpdate,
            False,
            {"201": Alarm, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result


class AlarmListPaginated(PaginatorDataModel[Alarm]):
    """
    AlarmListPaginated object.

    Parameters
    ----------
    data: Optional[List[Alarm]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[Alarm]] = None
    pagination: Optional[pagination.Pagination] = None


class AlarmStatistics(DataModelBase):
    """
    AlarmStatistics object.

    Parameters
    ----------
    severity: Optional[List[StatusStatistics]]
    status: Optional[StatusStatistics]

    """

    severity: Optional[List[StatusStatistics]] = None
    status: Optional[StatusStatistics] = None

    @classmethod
    def get_asset_statistics(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> AlarmAssetsStatistics:
        """
        Get Asset Statistics

        **Permission Required:** `kelvin.permission.alarm.read`.

        ``getAssetStatistics``: ``GET`` ``/api/v4/alarms/statistics/assets/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/alarms/statistics/assets/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AlarmAssetsStatistics, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_statistics(
        cls,
        data: Optional[Union[requests.AlarmList, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AlarmStatistics:
        """
        Get Statistics

        **Permission Required:** `kelvin.permission.alarm.read`.

        ``getStatistics``: ``POST`` ``/api/v4/alarms/statistics/get``

        Parameters
        ----------
        data: requests.AlarmList, optional
        **kwargs:
            Extra parameters for requests.AlarmList
              - acknowledged_at_end: datetime
              - acknowledged_at_start: datetime
              - acknowledged_by: list
              - alarm_class_names: list
              - alarm_severities: list
              - alarm_statuses: list
              - alarm_type_names: list
              - asset_class_names: list
              - asset_include_children: bool
              - asset_type_names: list
              - created_at_end: datetime
              - created_at_start: datetime
              - created_by: list
              - custom_identifiers: list
              - resolved_at_end: datetime
              - resolved_at_start: datetime
              - resolved_by: list
              - target_asset_names: list

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/alarms/statistics/get",
            {},
            {},
            {},
            {},
            data,
            requests.AlarmList,
            False,
            {"200": AlarmStatistics, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result


class App(DataModelBase):
    """
    App object.

    Parameters
    ----------
    created: Optional[datetime]
    description: Optional[str]
    latest_version: Optional[str]
    name: Optional[str]
    title: Optional[str]
    type: Optional[str]
    updated: Optional[datetime]
    versions: Optional[List[AppVersion]]

    """

    created: Optional[datetime] = None
    description: Optional[str] = None
    latest_version: Optional[str] = None
    name: Optional[str] = Field(None, max_length=64)
    title: Optional[str] = Field(None, max_length=64)
    type: Optional[str] = None
    updated: Optional[datetime] = None
    versions: Optional[List[AppVersion]] = None

    @classmethod
    def create_app(
        cls,
        data: Optional[Union[requests.AppCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> App:
        """
        Create App

        **Note:** If the App already exists, the App Version will be appended to the existing App

        **Permission Required:** `kelvin.permission.appregistry.create`.

        ``createApp``: ``POST`` ``/api/v4/appregistry/create``

        Parameters
        ----------
        data: requests.AppCreate, optional
        **kwargs:
            Extra parameters for requests.AppCreate
              - payload: dict

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/appregistry/create",
            {},
            {},
            {},
            {},
            data,
            requests.AppCreate,
            False,
            {"201": App, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_apps(
        self_or_cls,
        type: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[App], AppListPaginated]:
        """
        List Apps

        **Pagination Sortable Columns:** `app.name`, `app.title`, `app.type`, `app.latest_version`, `app.created`, `app.updated`, `status`

        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``listApps``: ``GET`` ``/api/v4/appregistry/list``

        Parameters
        ----------
        type : :obj:`Sequence[str]`
            Match the type field
        search : :obj:`Sequence[str]`
            Search Apps by name and title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/list",
            {},
            {
                "type": type,
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AppListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"app_versions": "versions"})
    def list_app_version_status(
        self_or_cls,
        app_versions: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AppVersionStatus], AppVersionStatusListPaginated]:
        """
        List App Version Status

        **Pagination Sortable Columns:** `app_version.id`

        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``listAppVersionStatus``: ``GET`` ``/api/v4/appregistry/versions/status/list``

        Parameters
        ----------
        app_versions : :obj:`Sequence[str]`
            Filter by App Versions. Pattern: `<app_name>:<version>`
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/versions/status/list",
            {},
            {
                "app_versions": app_versions,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AppVersionStatusListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"app_name": "name"})
    def delete_app(self_or_cls, app_name: str, _dry_run: bool = False, _client: Any = None) -> None:
        """
        Delete App

        **Note:** Deleting an App will also delete all App Versions

        **Permission Required:** `kelvin.permission.appregistry.delete`.

        ``deleteApp``: ``POST`` ``/api/v4/appregistry/{app_name}/delete``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/appregistry/{app_name}/delete",
            {"app_name": app_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "412": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"app_name": "name"})
    def get_app(self_or_cls, app_name: str, _dry_run: bool = False, _client: Any = None) -> App:
        """
        Get App

        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``getApp``: ``GET`` ``/api/v4/appregistry/{app_name}/get``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/{app_name}/get",
            {"app_name": app_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": App, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"app_name": "name"})
    def update_app(
        self_or_cls,
        app_name: str,
        data: Optional[Union[requests.AppUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> App:
        """
        Update App

        **Permission Required:** `kelvin.permission.appregistry.update`.

        ``updateApp``: ``POST`` ``/api/v4/appregistry/{app_name}/update``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        data: requests.AppUpdate, optional
        **kwargs:
            Extra parameters for requests.AppUpdate
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/appregistry/{app_name}/update",
            {"app_name": app_name},
            {},
            {},
            {},
            data,
            requests.AppUpdate,
            False,
            {"200": App, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"app_name": "name"})
    def delete_app_version(
        self_or_cls, app_name: str, app_version: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete App Version

        **Note:** This operation also deletes the associated Docker Image

        **Permission Required:** `kelvin.permission.appregistry.delete`.

        ``deleteAppVersion``: ``POST`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/delete``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/delete",
            {"app_name": app_name, "app_version": app_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"app_name": "name"})
    def download_app_version(
        self_or_cls, app_name: str, app_version: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Download App Version


        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``downloadAppVersion``: ``GET`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/download``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/download",
            {"app_name": app_name, "app_version": app_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"app_name": "name"})
    def get_app_version(
        self_or_cls, app_name: str, app_version: str, _dry_run: bool = False, _client: Any = None
    ) -> AppVersion:
        """
        Get App Version

        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``getAppVersion``: ``GET`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/get``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/get",
            {"app_name": app_name, "app_version": app_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AppVersion, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"app_name": "name"})
    def get_app_version_status(
        self_or_cls, app_name: str, app_version: str, _dry_run: bool = False, _client: Any = None
    ) -> AppVersionStatus:
        """
        Get App Version Status

        **Permission Required:** `kelvin.permission.appregistry.read`.

        ``getAppVersionStatus``: ``GET`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/status/get``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/status/get",
            {"app_name": app_name, "app_version": app_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AppVersionStatus, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"app_name": "name"})
    def upload_app_version(
        self_or_cls,
        app_name: str,
        app_version: str,
        file: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Upload App Version


        **Permission Required:** `kelvin.permission.appregistry.create`.

        ``uploadAppVersion``: ``POST`` ``/api/v4/appregistry/{app_name}/versions/{app_version}/upload``

        Parameters
        ----------
        app_name : :obj:`str`, optional
            App name
        app_version : :obj:`str`, optional
            App Version
        file : :obj:`str`, optional
            File to upload

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/appregistry/{app_name}/versions/{app_version}/upload",
            {"app_name": app_name, "app_version": app_version},
            {},
            {"file": file},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class AppListPaginated(PaginatorDataModel[App]):
    """
    AppListPaginated object.

    Parameters
    ----------
    data: Optional[List[App]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[App]] = None
    pagination: Optional[pagination.Pagination] = None


class AssetItem(DataModelBase):
    """
    AssetItem object.

    Parameters
    ----------
    asset_class_name: Optional[str]
    asset_class_title: Optional[str]
    asset_type_name: Optional[str]
    asset_type_title: Optional[str]
    created: Optional[datetime]
    device_type_name: Optional[str]
    device_type_title: Optional[str]
    name: Optional[str]
    status: Optional[AssetStatusItem]
    title: Optional[str]
    updated: Optional[datetime]

    """

    asset_class_name: Optional[str] = Field(None, max_length=64)
    asset_class_title: Optional[str] = Field(None, max_length=64)
    asset_type_name: Optional[str] = Field(None, max_length=64)
    asset_type_title: Optional[str] = None
    created: Optional[datetime] = None
    device_type_name: Optional[str] = Field(None, max_length=64)
    device_type_title: Optional[str] = None
    name: Optional[str] = Field(None, max_length=64)
    status: Optional[AssetStatusItem] = None
    title: Optional[str] = Field(None, max_length=64)
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_class(
        self_or_cls,
        data: Optional[Union[requests.AssetClassCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetClass:
        """
        Create Asset Class

        **Permission Required:** `kelvin.permission.asset_class.create`.

        ``createAssetClass``: ``POST`` ``/api/v4/assets/class/create``

        Parameters
        ----------
        data: requests.AssetClassCreate, optional
        **kwargs:
            Extra parameters for requests.AssetClassCreate
              - device_type_name: str
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/class/create",
            {},
            {},
            {},
            {},
            data,
            requests.AssetClassCreate,
            False,
            {"201": AssetClass, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset_class(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        owner: Optional[str] = None,
        device_type_name: Optional[Sequence[str]] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetClass], AssetClassListPaginated]:
        """
        List Asset Class

        **Pagination Sortable Columns:** `asset_class.name`, `asset_class.title`, `asset_class.created`, `asset_class.updated`, `asset_class.owner`

        **Permission Required:** `kelvin.permission.asset_class.read`.

        ``listAssetClass``: ``GET`` ``/api/v4/assets/class/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search Asset Class by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        owner : :obj:`str`
            Filter Asset Class by owner. Possible Values: kelvin/custom
        device_type_name : :obj:`Sequence[str]`
            Filter by Asset Class' Device Type Name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/class/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "owner": owner,
                "device_type_name": device_type_name,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetClassListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_asset_class(
        self_or_cls, asset_class_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset Class

        **Permission Required:** `kelvin.permission.asset_class.delete`.

        ``deleteAssetClass``: ``POST`` ``/api/v4/assets/class/{asset_class_name}/delete``

        Parameters
        ----------
        asset_class_name : :obj:`str`, optional
            Asset Class Name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/class/{asset_class_name}/delete",
            {"asset_class_name": asset_class_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_asset_class(
        self_or_cls, asset_class_name: str, _dry_run: bool = False, _client: Any = None
    ) -> AssetClass:
        """
        Get Asset Class

        **Permission Required:** `kelvin.permission.asset_class.read`.

        ``getAssetClass``: ``GET`` ``/api/v4/assets/class/{asset_class_name}/get``

        Parameters
        ----------
        asset_class_name : :obj:`str`, optional
            Asset Class Name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/class/{asset_class_name}/get",
            {"asset_class_name": asset_class_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetClass,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_asset_class(
        self_or_cls,
        asset_class_name: str,
        data: Optional[Union[requests.AssetClassUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetClass:
        """
        Update Asset Class

        **Permission Required:** `kelvin.permission.asset_class.update`.

        ``updateAssetClass``: ``POST`` ``/api/v4/assets/class/{asset_class_name}/update``

        Parameters
        ----------
        asset_class_name : :obj:`str`, optional
            Asset Class Name
        data: requests.AssetClassUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetClassUpdate
              - device_type_name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/class/{asset_class_name}/update",
            {"asset_class_name": asset_class_name},
            {},
            {},
            {},
            data,
            requests.AssetClassUpdate,
            False,
            {
                "200": AssetClass,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset(
        self_or_cls,
        dry_run: Optional[bool] = None,
        data: Optional[Union[requests.AssetCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Asset:
        """
        Create Asset

        **Permission Required:** `kelvin.permission.asset.create`.

        ``createAsset``: ``POST`` ``/api/v4/assets/create``

        Parameters
        ----------
        dry_run : :obj:`bool`
            Dry Run
        data: requests.AssetCreate, optional
        **kwargs:
            Extra parameters for requests.AssetCreate
              - asset_type_name: str
              - device_type_name: str
              - failure_detection: str
              - hierarchy: list
              - location: str
              - name: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/create",
            {},
            {"dry_run": dry_run},
            {},
            {},
            data,
            requests.AssetCreate,
            False,
            {
                "201": Asset,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "412": errors.ErrorMessage,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def upload_assets(
        cls, file: str, dry_run: Optional[bool] = None, _dry_run: bool = False, _client: Any = None
    ) -> AssetHierarchyList:
        """
        Upload Assets

        **Permission Required:** `kelvin.permission.asset.create`.

        ``uploadAssets``: ``POST`` ``/api/v4/assets/create/upload``

        Parameters
        ----------
        file : :obj:`str`, optional
            CSV File
        dry_run : :obj:`bool`
            If set to true performs only validation

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/assets/create/upload",
            {},
            {"dry_run": dry_run},
            {"file": file},
            {},
            None,
            None,
            False,
            {
                "200": AssetHierarchyList,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "412": CSVErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @classmethod
    def list_asset_hierarchies(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> AssetHierarchyListFull:
        """
        List Asset Hierarchies

        **Permission Required:** `kelvin.permission.asset.read`.

        ``listAssetHierarchies``: ``GET`` ``/api/v4/assets/hierarchy/list``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/hierarchy/list",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AssetHierarchyListFull, "400": errors.ErrorMessage, "401": errors.ErrorMessage},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        asset_type_name: Optional[Sequence[str]] = None,
        status_state: Optional[Sequence[str]] = None,
        device_type_name: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetItem], AssetListPaginated]:
        """
        List Asset

        **Pagination Sortable Columns:** `asset.name`, `asset.title`, `asset.created`, `asset.updated`, `asset_type.name`, `asset_type.title`, `status_state`, `status_last_seen`

        **Permission Required:** `kelvin.permission.asset.read`.

        ``listAsset``: ``GET`` ``/api/v4/assets/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search assets by name or title
        asset_type_name : :obj:`Sequence[str]`
            Filter assets by asset type
        status_state : :obj:`Sequence[str]`
            Filter assets by status state [`online`, `offline`,
            `partially_online`, `unknown`]
        device_type_name : :obj:`Sequence[str]`
            Filter assets by device type
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/list",
            {},
            {
                "search": search,
                "asset_type_name": asset_type_name,
                "status_state": status_state,
                "device_type_name": device_type_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetListPaginated,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def get_asset_status_count(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> AssetStatusCount:
        """
        Get Asset Status Count

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetStatusCount``: ``GET`` ``/api/v4/assets/status/count/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/status/count/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetStatusCount,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_asset_statuses(cls, _dry_run: bool = False, _client: Any = None) -> AssetStatus:
        """
        Get Asset Statuses

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetStatuses``: ``GET`` ``/api/v4/assets/status/current/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/status/current/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetStatus,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_template(
        self_or_cls,
        data: Optional[Union[requests.AssetTemplateCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTemplate:
        """
        Create Asset Template

        **Permission Required:** `kelvin.permission.asset.create`.

        ``createAssetTemplate``: ``POST`` ``/api/v4/assets/templates/create``

        Parameters
        ----------
        data: requests.AssetTemplateCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTemplateCreate
              - asset_type_name: str
              - description: str
              - device_type_name: str
              - failure_detection: str
              - hierarchy: list
              - hints: dict
              - image_url: str
              - name: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/templates/create",
            {},
            {},
            {},
            {},
            data,
            requests.AssetTemplateCreate,
            False,
            {"201": AssetTemplate, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_asset_template(
        cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        asset_type_names: Optional[Sequence[str]] = None,
        device_type_names: Optional[Sequence[str]] = None,
        asset_class_names: Optional[Sequence[str]] = None,
        owner: Optional[str] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetTemplate], AssetTemplateListPaginated]:
        """
        List Asset Template

        **Pagination Sortable Columns:** `asset_template.name`, `asset_template.title`, `asset_template.created`, `asset_template.updated`, `asset_template.asset_type_name`, `asset_template.device_type_name`

        **Permission Required:** `kelvin.permission.asset.read`.

        ``listAssetTemplate``: ``GET`` ``/api/v4/assets/templates/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search Asset Template by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        asset_type_names : :obj:`Sequence[str]`
            Asset Type Names
        device_type_names : :obj:`Sequence[str]`
            Device Type Names
        asset_class_names : :obj:`Sequence[str]`
            Asset Class Names
        owner : :obj:`str`
            Owner

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/templates/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "asset_type_names": asset_type_names,
                "device_type_names": device_type_names,
                "asset_class_names": asset_class_names,
                "owner": owner,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTemplateListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def delete_asset_template(
        cls, asset_template_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset Template

        **Permission Required:** `kelvin.permission.asset.delete`.

        ``deleteAssetTemplate``: ``POST`` ``/api/v4/assets/templates/{asset_template_name}/delete``

        Parameters
        ----------
        asset_template_name : :obj:`str`, optional
            Asset Template Name

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/assets/templates/{asset_template_name}/delete",
            {"asset_template_name": asset_template_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_asset_template(
        cls, asset_template_name: str, _dry_run: bool = False, _client: Any = None
    ) -> AssetTemplate:
        """
        Get Asset Template

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetTemplate``: ``GET`` ``/api/v4/assets/templates/{asset_template_name}/get``

        Parameters
        ----------
        asset_template_name : :obj:`str`, optional
            Asset Template Name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/templates/{asset_template_name}/get",
            {"asset_template_name": asset_template_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetTemplate,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_asset_template(
        self_or_cls,
        asset_template_name: str,
        data: Optional[Union[requests.AssetTemplateUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTemplate:
        """
        Update Asset Template

        **Permission Required:** `kelvin.permission.asset.update`.

        ``updateAssetTemplate``: ``POST`` ``/api/v4/assets/templates/{asset_template_name}/update``

        Parameters
        ----------
        asset_template_name : :obj:`str`, optional
            Asset Template Name
        data: requests.AssetTemplateUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTemplateUpdate
              - asset_type_name: str
              - description: str
              - device_type_name: str
              - failure_detection: str
              - hierarchy: list
              - hints: dict
              - image_url: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/templates/{asset_template_name}/update",
            {"asset_template_name": asset_template_name},
            {},
            {},
            {},
            data,
            requests.AssetTemplateUpdate,
            False,
            {
                "200": AssetTemplate,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_type(
        self_or_cls,
        data: Optional[Union[requests.AssetTypeCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetType:
        """
        Create Asset Type

        **Permission Required:** `kelvin.permission.asset_type.create`.

        ``createAssetType``: ``POST`` ``/api/v4/assets/types/create``

        Parameters
        ----------
        data: requests.AssetTypeCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeCreate
              - asset_class_name: str
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/create",
            {},
            {},
            {},
            {},
            data,
            requests.AssetTypeCreate,
            False,
            {"201": AssetType, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset_type(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        asset_class_name: Optional[Sequence[str]] = None,
        metadata: Optional[Sequence[str]] = None,
        metadata_filter: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        owner: Optional[str] = None,
        device_type_name: Optional[Sequence[str]] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetType], AssetTypeListPaginated]:
        """
        List Asset Type

        **Pagination Sortable Columns:** `asset_type.name`, `asset_type.title`, `asset_type.created`, `asset_type.updated`, `asset_class.name`, `asset_class.title`, `asset_type.owner`

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``listAssetType``: ``GET`` ``/api/v4/assets/types/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search asset type by name or title
        asset_class_name : :obj:`Sequence[str]`
            Filter asset type by asset class
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response
        metadata_filter : :obj:`Sequence[str]`
            Dot separated string to filter by metadata values Same <metadata_name>
            will result in OR queries. Different <metadata_name> will result in
            AND queries. Example:
            <metadata_name>.<payload_key_to_search>=<payload_value_to_search>
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        owner : :obj:`str`
            Filter Asset Class by owner. Possible Values: kelvin/custom
        device_type_name : :obj:`Sequence[str]`
            Filter asset type by asset class device type name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/list",
            {},
            {
                "search": search,
                "asset_class_name": asset_class_name,
                "metadata": metadata,
                "metadata_filter": metadata_filter,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "owner": owner,
                "device_type_name": device_type_name,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTypeListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_asset_type(
        self_or_cls, asset_type_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset Type

        **Permission Required:** `kelvin.permission.asset_type.delete`.

        ``deleteAssetType``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/delete``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/delete",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_asset_type(
        self_or_cls,
        asset_type_name: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> AssetType:
        """
        Get Asset Type

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``getAssetType``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/get``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/get",
            {"asset_type_name": asset_type_name},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {"200": AssetType, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        data: Optional[Union[requests.AssetTypeMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTypeMetadata:
        """
        Create Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.create`.

        ``createAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/create``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        data: requests.AssetTypeMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/create",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            data,
            requests.AssetTypeMetadataCreate,
            False,
            {
                "201": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetTypeMetadata], AssetTypeMetadataListPaginated]:
        """
        List Asset Type Metadata

        **Pagination Sortable Columns:** `asset_type_metadata.name`

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``listAssetTypeMetadata``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/metadata/list``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        search : :obj:`Sequence[str]`
            Search Asset Type Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/metadata/list",
            {"asset_type_name": asset_type_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTypeMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.delete`.

        ``deleteAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/delete``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/delete",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> AssetTypeMetadata:
        """
        Get Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``getAssetTypeMetadata``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/get``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/get",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        data: Optional[Union[requests.AssetTypeMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTypeMetadata:
        """
        Update Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.update`.

        ``updateAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/update``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name
        data: requests.AssetTypeMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/update",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            data,
            requests.AssetTypeMetadataUpdate,
            False,
            {
                "200": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_asset_type(
        self_or_cls,
        asset_type_name: str,
        data: Optional[Union[requests.AssetTypeUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetType:
        """
        Update Asset Type

        **Permission Required:** `kelvin.permission.asset_type.update`.

        ``updateAssetType``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/update``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        data: requests.AssetTypeUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeUpdate
              - asset_class_name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/update",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            data,
            requests.AssetTypeUpdate,
            False,
            {
                "200": AssetType,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_name": "name"})
    def delete_asset(
        self_or_cls, asset_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset

        **Permission Required:** `kelvin.permission.asset.delete`.

        ``deleteAsset``: ``POST`` ``/api/v4/assets/{asset_name}/delete``

        Parameters
        ----------
        asset_name : :obj:`str`, optional
            Asset name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/{asset_name}/delete",
            {"asset_name": asset_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
                "412": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_name": "name"})
    def get_asset(
        self_or_cls,
        asset_name: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Asset:
        """
        Get Asset

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAsset``: ``GET`` ``/api/v4/assets/{asset_name}/get``

        Parameters
        ----------
        asset_name : :obj:`str`, optional
            Asset name
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/{asset_name}/get",
            {"asset_name": asset_name},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {
                "200": Asset,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_name": "name"})
    def get_asset_hierarchy(
        self_or_cls, asset_name: str, _dry_run: bool = False, _client: Any = None
    ) -> Asset:
        """
        Get Asset Hierarchy

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetHierarchy``: ``GET`` ``/api/v4/assets/{asset_name}/hierarchy/get``

        Parameters
        ----------
        asset_name : :obj:`str`, optional
            Asset name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/{asset_name}/hierarchy/get",
            {"asset_name": asset_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": Asset,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_name": "name"})
    def update_asset(
        self_or_cls,
        asset_name: str,
        dry_run: Optional[bool] = None,
        data: Optional[Union[requests.AssetUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Asset:
        """
        Update Asset

        **Permission Required:** `kelvin.permission.asset.update`.

        ``updateAsset``: ``POST`` ``/api/v4/assets/{asset_name}/update``

        Parameters
        ----------
        asset_name : :obj:`str`, optional
            Asset name
        dry_run : :obj:`bool`
            Dry Run
        data: requests.AssetUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetUpdate
              - asset_type_name: str
              - failure_detection: str
              - hierarchy: list
              - location: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/{asset_name}/update",
            {"asset_name": asset_name},
            {"dry_run": dry_run},
            {},
            {},
            data,
            requests.AssetUpdate,
            False,
            {
                "200": Asset,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
                "412": errors.ErrorMessage,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_bridge_assets(
        cls,
        bridge_name: str,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetItem], AssetListPaginated]:
        """
        List Bridge Assets

        **Pagination Sortable Columns:** `asset.name`, `asset.title`, `asset.created`, `asset.updated`, `asset_type.title`

        **Permission Required:** `kelvin.permission.bridge.read`.

        ``listBridgeAssets``: ``GET`` ``/api/v4/bridges/{bridge_name}/assets/list``

        Parameters
        ----------
        bridge_name : :obj:`str`, optional
            Bridge name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/bridges/{bridge_name}/assets/list",
            {"bridge_name": bridge_name},
            {
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result


class AssetListPaginated(PaginatorDataModel[AssetItem]):
    """
    AssetListPaginated object.

    Parameters
    ----------
    data: Optional[List[AssetItem]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AssetItem]] = None
    pagination: Optional[pagination.Pagination] = None


class AssetStatus(DataModelBase):
    """
    AssetStatus object.

    Parameters
    ----------
    data: Optional[List[SimpleAsset]]

    """

    data: Optional[List[SimpleAsset]] = None

    @classmethod
    def get_asset_statuses(cls, _dry_run: bool = False, _client: Any = None) -> AssetStatus:
        """
        Get Asset Statuses

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetStatuses``: ``GET`` ``/api/v4/assets/status/current/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/status/current/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetStatus,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result


class CSVError(DataModelBase):
    """
    CSVError object.

    Parameters
    ----------
    description: Optional[str]
    name: Optional[str]
    positions: Optional[List[CSVPosition]]
    solution: Optional[str]
    title: Optional[str]
    type: Optional[str]

    """

    description: Optional[str] = None
    name: Optional[str] = None
    positions: Optional[List[CSVPosition]] = None
    solution: Optional[str] = None
    title: Optional[str] = None
    type: Optional[str] = None


class CSVErrorMessage(DataModelBase):
    """
    CSVErrorMessage object.

    Parameters
    ----------
    errorDefinitions: Optional[Dict[str, Dict[str, Any]]]
    errorPositions: Optional[Dict[str, int]]
    errors: Optional[List[CSVError]]

    """

    errorDefinitions: Optional[Dict[str, Dict[str, Any]]] = None
    errorPositions: Optional[Dict[str, int]] = None
    errors: Optional[List[CSVError]] = None


class Cluster(DataModelBase):
    """
    Cluster object.

    Parameters
    ----------
    cidr: Optional[ClusterCIDR]
    created: Optional[datetime]
    info: Optional[Dict[str, Any]]
    kelvin_info: Optional[Dict[str, Any]]
    kubeconfig: Optional[str]
    name: Optional[str]
    network_info: Optional[Dict[str, Any]]
    ready: Optional[bool]
    system_info: Optional[Dict[str, Any]]
    title: Optional[str]
    updated: Optional[datetime]

    """

    cidr: Optional[ClusterCIDR] = None
    created: Optional[datetime] = None
    info: Optional[Dict[str, Any]] = None
    kelvin_info: Optional[Dict[str, Any]] = None
    kubeconfig: Optional[str] = None
    name: Optional[str] = None
    network_info: Optional[Dict[str, Any]] = None
    ready: Optional[bool] = None
    system_info: Optional[Dict[str, Any]] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_cluster(
        self_or_cls,
        data: Optional[Union[requests.ClusterCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Cluster:
        """
        Create Cluster

        **Permission Required:** `kelvin.permission.acp.create`.

        ``createCluster``: ``POST`` ``/api/v4/orchestration/clusters/create``

        Parameters
        ----------
        data: requests.ClusterCreate, optional
        **kwargs:
            Extra parameters for requests.ClusterCreate
              - info: dict
              - kelvin_info: dict
              - kubeconfig: str
              - location: str
              - name: str
              - network_info: dict
              - system_info: dict
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/orchestration/clusters/create",
            {},
            {},
            {},
            {},
            data,
            requests.ClusterCreate,
            False,
            {
                "201": Cluster,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_cluster(
        cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[Cluster], ClusterListPaginated]:
        """
        List Cluster

        **Pagination Sortable Columns:** `cluster.name`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listCluster``: ``GET`` ``/api/v4/orchestration/clusters/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search Cluster by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/orchestration/clusters/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": ClusterListPaginated, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def download_cluster_provision_binary(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> Iterator[bytes]:
        """
        Download Cluster Provision Binary

        **Permission Required:** `kelvin.permission.orchestration.read`.

        ``downloadClusterProvisionBinary``: ``GET`` ``/api/v4/orchestration/clusters/provision/bin/download``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/orchestration/clusters/provision/bin/download",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": bytes, "400": ErrorMessages, "401": ErrorMessages},
            True,
            _dry_run,
        )
        return result

    @classmethod
    def download_cluster_provision_script(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> OrchestrationProvision:
        """
        Download Cluster Provision Script

        **Permission Required:** `kelvin.permission.orchestration.read`.

        ``downloadClusterProvisionScript``: ``GET`` ``/api/v4/orchestration/clusters/provision/script/download``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/orchestration/clusters/provision/script/download",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": OrchestrationProvision, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"cluster_name": "name"})
    def get_cluster(
        self_or_cls, cluster_name: str, _dry_run: bool = False, _client: Any = None
    ) -> Cluster:
        """
        Get Cluster

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getCluster``: ``GET`` ``/api/v4/orchestration/clusters/{cluster_name}/get``

        Parameters
        ----------
        cluster_name : :obj:`str`, optional
            Cluster name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/orchestration/clusters/{cluster_name}/get",
            {"cluster_name": cluster_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": Cluster, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessage},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"cluster_name": "name"})
    def get_cluster_manifests(
        self_or_cls,
        cluster_name: str,
        version: Optional[str] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> ClusterManifestList:
        """
        Get Cluster Manifests

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getClusterManifests``: ``GET`` ``/api/v4/orchestration/clusters/{cluster_name}/manifests/get``

        Parameters
        ----------
        cluster_name : :obj:`str`, optional
            Cluster name
        version : :obj:`str`
            Current Version

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/orchestration/clusters/{cluster_name}/manifests/get",
            {"cluster_name": cluster_name},
            {"version": version},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ClusterManifestList,
                "204": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"cluster_name": "name"})
    def download_cluster_private_registry_credentials(
        self_or_cls, cluster_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Download Cluster Private Registry Credentials

        **Permission Required:** `kelvin.permission.acp.read`.

        ``downloadClusterPrivateRegistryCredentials``: ``GET`` ``/api/v4/orchestration/clusters/{cluster_name}/private-registry/download``

        Parameters
        ----------
        cluster_name : :obj:`str`, optional
            Cluster name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/orchestration/clusters/{cluster_name}/private-registry/download",
            {"cluster_name": cluster_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "500": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"cluster_name": "name"})
    def update_cluster(
        self_or_cls,
        cluster_name: str,
        data: Optional[Union[requests.ClusterUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Cluster:
        """
        Update Cluster

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateCluster``: ``POST`` ``/api/v4/orchestration/clusters/{cluster_name}/update``

        Parameters
        ----------
        cluster_name : :obj:`str`, optional
            Cluster name
        data: requests.ClusterUpdate, optional
        **kwargs:
            Extra parameters for requests.ClusterUpdate
              - info: dict
              - kelvin_info: dict
              - kubeconfig: str
              - network_info: dict
              - ready: bool
              - system_info: dict
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/orchestration/clusters/{cluster_name}/update",
            {"cluster_name": cluster_name},
            {},
            {},
            {},
            data,
            requests.ClusterUpdate,
            False,
            {
                "200": Cluster,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessage,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class ClusterListPaginated(PaginatorDataModel[Cluster]):
    """
    ClusterListPaginated object.

    Parameters
    ----------
    data: Optional[List[Cluster]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[Cluster]] = None
    pagination: Optional[pagination.Pagination] = None


class DataLabel(DataModelBase):
    """
    DataLabel object.

    Parameters
    ----------
    confidence: Optional[int]
    created: Optional[datetime]
    description: Optional[str]
    end_date: Optional[datetime]
    id: Optional[int]
    info: Optional[Dict[str, Any]]
    label_name: Optional[str]
    metrics: Optional[List[DataLabelMetricItem]]
    source: Optional[DataLabelSource]
    start_date: Optional[datetime]
    updated: Optional[datetime]
    validation: Optional[bool]

    """

    confidence: Optional[int] = None
    created: Optional[datetime] = None
    description: Optional[str] = None
    end_date: Optional[datetime] = None
    id: Optional[int] = None
    info: Optional[Dict[str, Any]] = None
    label_name: Optional[str] = None
    metrics: Optional[List[DataLabelMetricItem]] = None
    source: Optional[DataLabelSource] = None
    start_date: Optional[datetime] = None
    updated: Optional[datetime] = None
    validation: Optional[bool] = None

    @instance_classmethod
    @DataModelBase.translate()
    def get_data_labels_clustering(
        self_or_cls,
        data: Optional[Union[requests.DataLabelCluster, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> DataLabelCluster:
        """
        Get Data Labels Clustering

        **Note**:
        * `time_bucket`: needed to aggregate a cluster. Use Go parsing for duration:
        https://golang.org/pkg/time/#ParseDuration
        * `label_names`: lists data labels using **OR** operator;
        * `metrics`: lists data labels using **OR** operator;
        * `start_date`: lists data labels with start_date >= query date (RFC3339 format with microseconds);
        * `end_date`: lists data labels with end_date <= query date (RFC3339 format with microseconds)

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``getDataLabelsClustering``: ``POST`` ``/api/v4/datalabels/clustering/get``

        Parameters
        ----------
        data: requests.DataLabelCluster, optional
        **kwargs:
            Extra parameters for requests.DataLabelCluster
              - end_date: datetime
              - label_names: list
              - metrics: list
              - start_date: datetime
              - time_bucket: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/clustering/get",
            {},
            {},
            {},
            {},
            data,
            requests.DataLabelCluster,
            False,
            {"200": DataLabelCluster, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_data_label(
        self_or_cls,
        data: Optional[Union[requests.DataLabelCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> DataLabel:
        """
        Create Data Label

        **Note:**
        * If `label_name` is provided and does not exist, it will create a new label
        * The `source.type` can be `workload` or `user`

        **Permission Required:** `kelvin.permission.data_label.create`.

        ``createDataLabel``: ``POST`` ``/api/v4/datalabels/create``

        Parameters
        ----------
        data: requests.DataLabelCreate, optional
        **kwargs:
            Extra parameters for requests.DataLabelCreate
              - confidence: int
              - description: str
              - end_date: datetime
              - info: dict
              - label_name: str
              - metrics: list
              - source: str
              - start_date: datetime
              - validation: bool

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/create",
            {},
            {},
            {},
            {},
            data,
            requests.DataLabelCreate,
            False,
            {"201": DataLabel, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def filter_data_label(
        self_or_cls,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        data: Optional[Union[requests.DataLabelFilter, Mapping[str, Any]]] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Union[Sequence[DataLabel], DataLabelListPaginated]:
        """
        Filter Data Label

        **Pagination Sortable Columns:** `data_label.id`

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``filterDataLabel``: ``POST`` ``/api/v4/datalabels/filter``

        Parameters
        ----------
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            false
        data: requests.DataLabelFilter, optional
        **kwargs:
            Extra parameters for requests.DataLabelFilter
              - label_names: list
              - metrics: list
              - search: list

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/filter",
            {},
            {
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            data,
            requests.DataLabelFilter,
            False,
            {"200": DataLabelListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def create_label(
        cls,
        data: Optional[Union[requests.LabelCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Label:
        """
        Create Label

        **Permission Required:** `kelvin.permission.data_label.create`.

        ``createLabel``: ``POST`` ``/api/v4/datalabels/labels/create``

        Parameters
        ----------
        data: requests.LabelCreate, optional
        **kwargs:
            Extra parameters for requests.LabelCreate
              - name: str
              - title: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/create",
            {},
            {},
            {},
            {},
            data,
            requests.LabelCreate,
            False,
            {"201": Label, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_label(
        cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        metadata: Optional[Sequence[str]] = None,
        metadata_filter: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[Label], LabelListPaginated]:
        """
        List Label

        **Pagination Sortable Columns:** `label.name`

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``listLabel``: ``GET`` ``/api/v4/datalabels/labels/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter labels by name
        search : :obj:`Sequence[str]`
            Search Labels by name or title
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response
        metadata_filter : :obj:`Sequence[str]`
            Dot separated string to filter by metadata values Same <metadata_name>
            will result in OR queries. Different <metadata_name> will result in
            AND queries. Example:
            <metadata_name>.<payload_key_to_search>=<payload_value_to_search>
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/list",
            {},
            {
                "names": names,
                "search": search,
                "metadata": metadata,
                "metadata_filter": metadata_filter,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": LabelListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def create_label_metadata_bulk(
        cls,
        data: Optional[Union[requests.LabelMetadataBulkCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Create Label Metadata Bulk

        **Permission Required:** `kelvin.permission.data_label.create`.

        ``createLabelMetadataBulk``: ``POST`` ``/api/v4/datalabels/labels/metadata/create``

        Parameters
        ----------
        data: requests.LabelMetadataBulkCreate, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataBulkCreate

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/metadata/create",
            {},
            {},
            {},
            {},
            data,
            requests.LabelMetadataBulkCreate,
            False,
            {
                "201": None,
                "207": ErrorMessages,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def delete_label_metadata_bulk(
        cls,
        data: Optional[Union[requests.LabelMetadataBulkDelete, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> None:
        """
        Delete Label Metadata Bulk

        **Permission Required:** `kelvin.permission.data_label.delete`.

        ``deleteLabelMetadataBulk``: ``POST`` ``/api/v4/datalabels/labels/metadata/delete``

        Parameters
        ----------
        data: requests.LabelMetadataBulkDelete, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataBulkDelete

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/metadata/delete",
            {},
            {},
            {},
            {},
            data,
            requests.LabelMetadataBulkDelete,
            False,
            {
                "200": None,
                "207": ErrorMessages,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_label(
        self_or_cls, label_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Label

        **Permission Required:** `kelvin.permission.data_label.delete`.

        ``deleteLabel``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/delete``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/delete",
            {"label_name": label_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_label(
        self_or_cls,
        label_name: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Label:
        """
        Get Label

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``getLabel``: ``GET`` ``/api/v4/datalabels/labels/{label_name}/get``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/{label_name}/get",
            {"label_name": label_name},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {"200": Label, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_label_metadata(
        self_or_cls,
        label_name: str,
        data: Optional[Union[requests.LabelMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> LabelMetadata:
        """
        Create Label Metadata

        **Permission Required:** `kelvin.permission.data_label.create`.

        ``createLabelMetadata``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/metadata/create``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        data: requests.LabelMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/metadata/create",
            {"label_name": label_name},
            {},
            {},
            {},
            data,
            requests.LabelMetadataCreate,
            False,
            {
                "201": LabelMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_label_metadata(
        self_or_cls,
        label_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[LabelMetadata], LabelMetadataListPaginated]:
        """
        List Label Metadata

        **Pagination Sortable Columns:** `label_metadata.name`, `label_metadata.label_name`

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``listLabelMetadata``: ``GET`` ``/api/v4/datalabels/labels/{label_name}/metadata/list``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        search : :obj:`Sequence[str]`
            Search Label Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/{label_name}/metadata/list",
            {"label_name": label_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": LabelMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_label_metadata(
        self_or_cls,
        label_name: str,
        label_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete Label Metadata

        **Permission Required:** `kelvin.permission.data_label.delete`.

        ``deleteLabelMetadata``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/delete``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        label_metadata_name : :obj:`str`, optional
            Label Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/delete",
            {"label_name": label_name, "label_metadata_name": label_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_label_metadata(
        self_or_cls,
        label_name: str,
        label_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> LabelMetadata:
        """
        Get Label Metadata

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``getLabelMetadata``: ``GET`` ``/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/get``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        label_metadata_name : :obj:`str`, optional
            Label Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/get",
            {"label_name": label_name, "label_metadata_name": label_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": LabelMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_label_metadata(
        self_or_cls,
        label_name: str,
        label_metadata_name: str,
        data: Optional[Union[requests.LabelMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> LabelMetadata:
        """
        Update Label Metadata

        **Permission Required:** `kelvin.permission.data_label.update`.

        ``updateLabelMetadata``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/update``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        label_metadata_name : :obj:`str`, optional
            Label Metadata name
        data: requests.LabelMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.LabelMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/metadata/{label_metadata_name}/update",
            {"label_name": label_name, "label_metadata_name": label_metadata_name},
            {},
            {},
            {},
            data,
            requests.LabelMetadataUpdate,
            False,
            {
                "200": LabelMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_label(
        self_or_cls,
        label_name: str,
        data: Optional[Union[requests.LabelUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Label:
        """
        Update Label

        **Permission Required:** `kelvin.permission.data_label.update`.

        ``updateLabel``: ``POST`` ``/api/v4/datalabels/labels/{label_name}/update``

        Parameters
        ----------
        label_name : :obj:`str`, optional
            Label name
        data: requests.LabelUpdate, optional
        **kwargs:
            Extra parameters for requests.LabelUpdate
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/labels/{label_name}/update",
            {"label_name": label_name},
            {},
            {},
            {},
            data,
            requests.LabelUpdate,
            False,
            {
                "200": Label,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_data_label(
        cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[DataLabel], DataLabelListPaginated]:
        """
        List Data Label

        **Pagination Sortable Columns:** `data_label.id`

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``listDataLabel``: ``GET`` ``/api/v4/datalabels/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search datalabels by description and label name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            false

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": DataLabelListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def get_data_label_range(
        self_or_cls,
        data: Optional[Union[requests.DataLabelRange, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Iterator[DataLabel]:
        """
        Get Data Label Range

        **Note**:
        * Between filters a **AND*+ operator is implicit
        * `label_names`: lists data labels using **OR** operator;
        * `metrics`: lists data labels using **OR** operator;
        * `start_date`: lists data labels with start_date >= query date (RFC3339 format with microseconds);
        * `end_date`: lists data labels with end_date <= query date (RFC3339 format with microseconds)

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``getDataLabelRange``: ``POST`` ``/api/v4/datalabels/range/get``

        Parameters
        ----------
        data: requests.DataLabelRange, optional
        **kwargs:
            Extra parameters for requests.DataLabelRange
              - end_date: datetime
              - label_names: list
              - metrics: list
              - start_date: datetime

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/range/get",
            {},
            {},
            {},
            {},
            data,
            requests.DataLabelRange,
            False,
            {"200": DataLabel, "400": ErrorMessages, "401": ErrorMessages},
            True,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def delete_data_label(
        cls, datalabel_id: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Data Label

        **Permission Required:** `kelvin.permission.data_label.delete`.

        ``deleteDataLabel``: ``POST`` ``/api/v4/datalabels/{datalabel_id}/delete``

        Parameters
        ----------
        datalabel_id : :obj:`str`, optional
            Data Label id

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/{datalabel_id}/delete",
            {"datalabel_id": datalabel_id},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_data_label(
        cls, datalabel_id: str, _dry_run: bool = False, _client: Any = None
    ) -> DataLabel:
        """
        Get Data Label

        **Permission Required:** `kelvin.permission.data_label.read`.

        ``getDataLabel``: ``GET`` ``/api/v4/datalabels/{datalabel_id}/get``

        Parameters
        ----------
        datalabel_id : :obj:`str`, optional
            Data Label id

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/datalabels/{datalabel_id}/get",
            {"datalabel_id": datalabel_id},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": DataLabel, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_data_label(
        self_or_cls,
        datalabel_id: str,
        data: Optional[Union[requests.DataLabelCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> DataLabel:
        """
        Update Data Label

        **Note:**
        * If `metrics` are provided, it will replace the full list
        * If `label_name` is provided and does not exist, it will create a new label

        **Permission Required:** `kelvin.permission.data_label.update`.

        ``updateDataLabel``: ``POST`` ``/api/v4/datalabels/{datalabel_id}/update``

        Parameters
        ----------
        datalabel_id : :obj:`str`, optional
            DataLabel id
        data: requests.DataLabelCreate, optional
        **kwargs:
            Extra parameters for requests.DataLabelCreate
              - confidence: int
              - description: str
              - end_date: datetime
              - info: dict
              - label_name: str
              - metrics: list
              - source: str
              - start_date: datetime
              - validation: bool

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datalabels/{datalabel_id}/update",
            {"datalabel_id": datalabel_id},
            {},
            {},
            {},
            data,
            requests.DataLabelCreate,
            False,
            {
                "200": DataLabel,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class DataLabelListPaginated(PaginatorDataModel[DataLabel]):
    """
    DataLabelListPaginated object.

    Parameters
    ----------
    data: Optional[List[DataLabel]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[DataLabel]] = None
    pagination: Optional[pagination.Pagination] = None


class DataType(DataModelBase):
    """
    DataType object.

    Parameters
    ----------
    connect: Optional[bool]
    created: Optional[datetime]
    icd: Optional[ICDPayload]
    name: Optional[str]
    schema_: Optional[Dict[str, Any]]
    source: Optional[str]
    type: Optional[str]
    updated: Optional[datetime]
    version: Optional[str]

    """

    connect: Optional[bool] = None
    created: Optional[datetime] = None
    icd: Optional[ICDPayload] = None
    name: Optional[str] = Field(None, max_length=64)
    schema_: Optional[Dict[str, Any]] = Field(None, alias="schema")
    source: Optional[str] = None
    type: Optional[str] = Field(None, max_length=45)
    updated: Optional[datetime] = None
    version: Optional[str] = Field(None, max_length=45)

    @instance_classmethod
    @DataModelBase.translate()
    def create_data_type(
        self_or_cls,
        data: Optional[Union[requests.DataTypeCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> DataType:
        """
        Create Data Type
        Regex validators:
        - data type name: `^([a-z][a-z0-9_]+\\.)+[a-z][a-z0-9_]+$`
        - data type class_name: `^[a-zA-Z][a-zA-Z0-9_]+$`

        **Permission Required:** `kelvin.permission.data_type.create`.

        ``createDataType``: ``POST`` ``/api/v4/datatypes/create``

        Parameters
        ----------
        data: requests.DataTypeCreate, optional
        **kwargs:
            Extra parameters for requests.DataTypeCreate
              - icd: str
              - schema: dict
              - source: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/datatypes/create",
            {},
            {},
            {},
            {},
            data,
            requests.DataTypeCreate,
            False,
            {"201": DataType, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_data_type(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        all: Optional[bool] = None,
        connect: Optional[bool] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[DataType], DataTypeListPaginated]:
        """
        List Data Type

        **Pagination Sortable Columns:** `data_type.name`, `data_type.version`
        By default returns only the latest versions for each data type

        **Permission Required:** `kelvin.permission.data_type.read`.

        ``listDataType``: ``GET`` ``/api/v4/datatypes/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search data types by name
        all : :obj:`bool`
            List all versions of all data types. If false, it will only list the
            latest version. Default: False.
        connect : :obj:`bool`
            Only fetch connect data types. Default: False.
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datatypes/list",
            {},
            {
                "search": search,
                "all": all,
                "connect": connect,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": DataTypeListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"data_type_name": "name"})
    def get_data_type_latest_version(
        self_or_cls, data_type_name: str, _dry_run: bool = False, _client: Any = None
    ) -> DataType:
        """
        Get Data Type Latest Version

        **Permission Required:** `kelvin.permission.data_type.read`.

        ``getDataTypeLatestVersion``: ``GET`` ``/api/v4/datatypes/{data_type_name}/versions/latest/get``

        Parameters
        ----------
        data_type_name : :obj:`str`, optional
            Data Type name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datatypes/{data_type_name}/versions/latest/get",
            {"data_type_name": data_type_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": DataType, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"data_type_name": "name", "data_type_version": "version"})
    def get_data_type_dependencies(
        self_or_cls,
        data_type_name: str,
        data_type_version: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> DataType:
        """
        Get Data Type Dependencies. This endpoint ill return all dependencies levels in a flattened array.

        **Permission Required:** `kelvin.permission.data_type.read`.

        ``getDataTypeDependencies``: ``GET`` ``/api/v4/datatypes/{data_type_name}/versions/{data_type_version}/dependencies/get``

        Parameters
        ----------
        data_type_name : :obj:`str`, optional
            Data Type name
        data_type_version : :obj:`str`, optional
            Data Type version

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datatypes/{data_type_name}/versions/{data_type_version}/dependencies/get",
            {"data_type_name": data_type_name, "data_type_version": data_type_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": DataType, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"data_type_name": "name", "data_type_version": "version"})
    def get_data_type(
        self_or_cls,
        data_type_name: str,
        data_type_version: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> DataType:
        """
        Get Data Type

        **Permission Required:** `kelvin.permission.data_type.read`.

        ``getDataType``: ``GET`` ``/api/v4/datatypes/{data_type_name}/versions/{data_type_version}/get``

        Parameters
        ----------
        data_type_name : :obj:`str`, optional
            Data Type name
        data_type_version : :obj:`str`, optional
            Data Type version

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datatypes/{data_type_name}/versions/{data_type_version}/get",
            {"data_type_name": data_type_name, "data_type_version": data_type_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": DataType, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"data_type_name": "name", "data_type_version": "version"})
    def get_data_type_schema(
        self_or_cls,
        data_type_name: str,
        data_type_version: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> DataTypeSchema:
        """
        Get Data Type Schema

        **Permission Required:** `kelvin.permission.data_type.read`.

        ``getDataTypeSchema``: ``GET`` ``/api/v4/datatypes/{data_type_name}/versions/{data_type_version}/schema/get``

        Parameters
        ----------
        data_type_name : :obj:`str`, optional
            Data Type name
        data_type_version : :obj:`str`, optional
            Data Type version

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/datatypes/{data_type_name}/versions/{data_type_version}/schema/get",
            {"data_type_name": data_type_name, "data_type_version": data_type_version},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": DataTypeSchema,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result


class DataTypeListPaginated(PaginatorDataModel[DataType]):
    """
    DataTypeListPaginated object.

    Parameters
    ----------
    data: Optional[List[DataType]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[DataType]] = None
    pagination: Optional[pagination.Pagination] = None


class Dependencies(DataModelBase):
    """
    Dependencies object.

    Parameters
    ----------
    data: Optional[List[Dependency]]

    """

    data: Optional[List[Dependency]] = None

    @classmethod
    def get_dependencies(
        cls,
        data: Optional[Union[requests.GraphDependencies, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Dependencies:
        """
        Get Dependencies

        **Permission Required:** `kelvin.permission.instance.read`.

        ``getDependencies``: ``POST`` ``/api/v4/instance/graph/dependencies/get``

        Parameters
        ----------
        data: requests.GraphDependencies, optional
        **kwargs:
            Extra parameters for requests.GraphDependencies
              - collection: str
              - name: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/instance/graph/dependencies/get",
            {},
            {},
            {},
            {},
            data,
            requests.GraphDependencies,
            False,
            {"200": Dependencies, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result


class FailureClassProperty(DataModelBase):
    """
    FailureClassProperty object.

    Parameters
    ----------
    data_type: Optional[str]
    name: Optional[str]
    title: Optional[str]
    unit_options: Optional[List[PropertyOptions]]
    value_options: Optional[List[PropertyOptions]]

    """

    data_type: Optional[str] = None
    name: Optional[str] = None
    title: Optional[str] = None
    unit_options: Optional[List[PropertyOptions]] = None
    value_options: Optional[List[PropertyOptions]] = None


class FailureDetection(DataModelBase):
    """
    FailureDetection object.

    Parameters
    ----------
    enabled: Optional[bool]
    failure_class_name: Optional[str]
    properties: Optional[List[FailureDetectionProperty]]

    """

    enabled: Optional[bool] = None
    failure_class_name: Optional[str] = None
    properties: Optional[List[FailureDetectionProperty]] = None


class InstanceSettings(DataModelBase):
    """
    InstanceSettings object.

    Parameters
    ----------
    acp: Optional[ACPSettings]

    """

    acp: Optional[ACPSettings] = None

    @classmethod
    def get_instance_settings(cls, _dry_run: bool = False, _client: Any = None) -> InstanceSettings:
        """
        Get Instance Settings

        **Permission Required:** `kelvin.permission.instance.read`.

        ``getInstanceSettings``: ``GET`` ``/api/v4/instance/settings/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/instance/settings/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": InstanceSettings,
                "400": ErrorMessage,
                "401": ErrorMessage,
                "404": ErrorMessage,
                "500": ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_instance_settings(
        self_or_cls,
        data: Optional[Union[requests.InstanceSettings, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> InstanceSettings:
        """
        Update Instance Settings

        **Permission Required:** `kelvin.permission.instance.update`.

        ``updateInstanceSettings``: ``POST`` ``/api/v4/instance/settings/update``

        Parameters
        ----------
        data: requests.InstanceSettings, optional
        **kwargs:
            Extra parameters for requests.InstanceSettings
              - acp: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/instance/settings/update",
            {},
            {},
            {},
            {},
            data,
            requests.InstanceSettings,
            False,
            {"200": InstanceSettings, "400": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result


class Integration(DataModelBase):
    """
    Integration object.

    Parameters
    ----------
    configuration: Optional[IntegrationConfiguration]
    created: Optional[datetime]
    enable: Optional[bool]
    name: Optional[str]
    status: Optional[IntegrationStatus]
    title: Optional[str]
    type: Optional[str]
    updated: Optional[datetime]

    """

    configuration: Optional[IntegrationConfiguration] = None
    created: Optional[datetime] = None
    enable: Optional[bool] = None
    name: Optional[str] = None
    status: Optional[IntegrationStatus] = None
    title: Optional[str] = None
    type: Optional[str] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_aws_region(
        self_or_cls,
        data: Optional[Union[requests.IntegrationAwsRegionCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> IntegrationAwsRegion:
        """
        Create AWS Region

        **Permission Required:** `kelvin.permission.integration.create`.

        ``createAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/create``

        Parameters
        ----------
        data: requests.IntegrationAwsRegionCreate, optional
        **kwargs:
            Extra parameters for requests.IntegrationAwsRegionCreate
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/create",
            {},
            {},
            {},
            {},
            data,
            requests.IntegrationAwsRegionCreate,
            False,
            {"201": IntegrationAwsRegion, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_aws_regions(
        cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> IntegrationAwsRegionList:
        """
        List AWS Regions

        **Pagination Sortable Columns:** `aws_regions.name`, `aws_regions.title`, `aws_regions.created`, `aws_regions.updated`

        **Permission Required:** `kelvin.permission.integration.read`.

        ``listAWSRegions``: ``GET`` ``/api/v4/integrations/aws/regions/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter Regions by name
        search : :obj:`Sequence[str]`
            Search Regions by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/integrations/aws/regions/list",
            {},
            {
                "names": names,
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": IntegrationAwsRegionList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_aws_region(
        self_or_cls, name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete AWS Region

        **Permission Required:** `kelvin.permission.integration.delete`.

        ``deleteAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/{name}/delete``

        Parameters
        ----------
        name : :obj:`str`, optional
            Region name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/{name}/delete",
            {"name": name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_aws_region(
        self_or_cls,
        name: str,
        data: Optional[Union[requests.IntegrationAwsRegionUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> IntegrationAwsRegion:
        """
        Update AWS Region

        **Permission Required:** `kelvin.permission.integration.update`.

        ``updateAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/{name}/update``

        Parameters
        ----------
        name : :obj:`str`, optional
            Region name
        data: requests.IntegrationAwsRegionUpdate, optional
        **kwargs:
            Extra parameters for requests.IntegrationAwsRegionUpdate
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/{name}/update",
            {"name": name},
            {},
            {},
            {},
            data,
            requests.IntegrationAwsRegionUpdate,
            False,
            {
                "200": IntegrationAwsRegion,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_integration(
        self_or_cls,
        data: Optional[Union[requests.IntegrationCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Integration:
        """
        Create Integration

        **Permission Required:** `kelvin.permission.integration.create`.

        ``createIntegration``: ``POST`` ``/api/v4/integrations/create``

        Parameters
        ----------
        data: requests.IntegrationCreate, optional
        **kwargs:
            Extra parameters for requests.IntegrationCreate
              - configuration: str
              - enable: bool
              - name: str
              - title: str
              - type: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/create",
            {},
            {},
            {},
            {},
            data,
            requests.IntegrationCreate,
            False,
            {"201": Integration, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_integrations(
        self_or_cls,
        names: Optional[Sequence[str]] = None,
        types: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        enable: Optional[bool] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> IntegrationList:
        """
        List Integrations

        **Pagination Sortable Columns:** `integration.name`, `integration.title`, `integration.type`, `integration.enable`, `integration.state`, `integration.last_seen`, `integration.created`, `integration.updated`

        **Permission Required:** `kelvin.permission.integration.read`.

        ``listIntegrations``: ``GET`` ``/api/v4/integrations/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter Integrations by name
        types : :obj:`Sequence[str]`
            Filter Integrations by type
        search : :obj:`Sequence[str]`
            Search Integrations by name or title
        enable : :obj:`bool`
            Filter by enable Integrations. Possible Values: true/false. Default:
            none
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/integrations/list",
            {},
            {
                "names": names,
                "types": types,
                "search": search,
                "enable": enable,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": IntegrationList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_integration(
        self_or_cls, name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Integration

        **Permission Required:** `kelvin.permission.integration.delete`.

        ``deleteIntegration``: ``POST`` ``/api/v4/integrations/{name}/delete``

        Parameters
        ----------
        name : :obj:`str`, optional
            Integration name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/{name}/delete",
            {"name": name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_integration(
        self_or_cls, name: str, _dry_run: bool = False, _client: Any = None
    ) -> Integration:
        """
        Get Integration

        **Permission Required:** `kelvin.permission.integration.read`.

        ``getIntegration``: ``GET`` ``/api/v4/integrations/{name}/get``

        Parameters
        ----------
        name : :obj:`str`, optional
            Integration name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/integrations/{name}/get",
            {"name": name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": Integration, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_integration(
        self_or_cls,
        name: str,
        data: Optional[Union[requests.IntegrationUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Integration:
        """
        Update Integration

        **Permission Required:** `kelvin.permission.integration.update`.

        ``updateIntegration``: ``POST`` ``/api/v4/integrations/{name}/update``

        Parameters
        ----------
        name : :obj:`str`, optional
            Integration name
        data: requests.IntegrationUpdate, optional
        **kwargs:
            Extra parameters for requests.IntegrationUpdate
              - configuration: str
              - enable: bool
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/{name}/update",
            {"name": name},
            {},
            {},
            {},
            data,
            requests.IntegrationUpdate,
            False,
            {"200": Integration, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result


class IntegrationListItem(DataModelBase):
    """
    IntegrationListItem object.

    Parameters
    ----------
    created: Optional[datetime]
    enable: Optional[bool]
    name: Optional[str]
    status: Optional[IntegrationStatus]
    title: Optional[str]
    type: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    enable: Optional[bool] = None
    name: Optional[str] = None
    status: Optional[IntegrationStatus] = None
    title: Optional[str] = None
    type: Optional[str] = None
    updated: Optional[datetime] = None


class Workload(DataModelBase):
    """
    Workload object.

    Parameters
    ----------
    acp_name: Optional[str]
    app_name: Optional[str]
    app_version: Optional[str]
    created: Optional[datetime]
    enabled: Optional[bool]
    name: Optional[str]
    networking: Optional[List[NetworkingItem]]
    payload: Optional[Dict[str, Any]]
    status: Optional[WorkloadStatus]
    title: Optional[str]
    updated: Optional[datetime]

    """

    acp_name: Optional[str] = None
    app_name: Optional[str] = None
    app_version: Optional[str] = None
    created: Optional[datetime] = None
    enabled: Optional[bool] = None
    name: Optional[str] = None
    networking: Optional[List[NetworkingItem]] = None
    payload: Optional[Dict[str, Any]] = None
    status: Optional[WorkloadStatus] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def deploy_workload(
        self_or_cls,
        data: Optional[Union[requests.WorkloadDeploy, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Workload:
        """
        Deploy Workload

        **Permission Required:** `kelvin.permission.workload.update`.

        ``deployWorkload``: ``POST`` ``/api/v4/workloads/deploy``

        Parameters
        ----------
        data: requests.WorkloadDeploy, optional
        **kwargs:
            Extra parameters for requests.WorkloadDeploy
              - acp_name: str
              - app_name: str
              - app_version: str
              - name: str
              - payload: dict
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/workloads/deploy",
            {},
            {},
            {},
            {},
            data,
            requests.WorkloadDeploy,
            False,
            {
                "201": Workload,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"workload_name": "name"})
    def list_workload(
        self_or_cls,
        app_name: Optional[Sequence[str]] = None,
        app_version: Optional[Sequence[str]] = None,
        acp_name: Optional[Sequence[str]] = None,
        workload_name: Optional[Sequence[str]] = None,
        enabled: Optional[bool] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        search: Optional[Sequence[str]] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[Workload], WorkloadListPaginated]:
        """
        List Workload

        **Pagination Sortable Columns:** `workload.name`, `workload.enabled

        **Permission Required:** `kelvin.permission.workload.read`.

        ``listWorkload``: ``GET`` ``/api/v4/workloads/list``

        Parameters
        ----------
        app_name : :obj:`Sequence[str]`
            App Name
        app_version : :obj:`Sequence[str]`
            App Version
        acp_name : :obj:`Sequence[str]`
            ACP Name
        workload_name : :obj:`Sequence[str]`
            Workload Name
        enabled : :obj:`bool`
            Filter workloads by enabled status
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        search : :obj:`Sequence[str]`
            Search workloads by name, title, App name and ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/workloads/list",
            {},
            {
                "app_name": app_name,
                "app_version": app_version,
                "acp_name": acp_name,
                "workload_name": workload_name,
                "enabled": enabled,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "search": search,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": WorkloadListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"workload_name": "name"})
    def get_workload(
        self_or_cls, workload_name: str, _dry_run: bool = False, _client: Any = None
    ) -> Workload:
        """
        Get Workload

        **Permission Required:** `kelvin.permission.workload.read`.

        ``getWorkload``: ``GET`` ``/api/v4/workloads/{workload_name}/get``

        Parameters
        ----------
        workload_name : :obj:`str`, optional
            Workload name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/workloads/{workload_name}/get",
            {"workload_name": workload_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": Workload, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"workload_name": "name"})
    def get_workload_logs(
        self_or_cls,
        workload_name: str,
        tail_lines: Optional[str] = None,
        since_time: Optional[str] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> WorkloadLogs:
        """
        Get Workload Logs

        **Permission Required:** `kelvin.permission.workload.read`.

        ``getWorkloadLogs``: ``GET`` ``/api/v4/workloads/{workload_name}/logs/get``

        Parameters
        ----------
        workload_name : :obj:`str`, optional
            Workload name
        tail_lines : :obj:`str`
            The number of lines from the end of the logs to show
        since_time : :obj:`str`
            A timestamp from which to show logs

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/workloads/{workload_name}/logs/get",
            {"workload_name": workload_name},
            {"tail_lines": tail_lines, "since_time": since_time},
            {},
            {},
            None,
            None,
            False,
            {
                "200": WorkloadLogs,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "500": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"workload_name": "name"})
    def start_workload(
        self_or_cls, workload_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Start Workload

        **Permission Required:** `kelvin.permission.workload.update`.

        ``startWorkload``: ``GET`` ``/api/v4/workloads/{workload_name}/start``

        Parameters
        ----------
        workload_name : :obj:`str`, optional
            Workload name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/workloads/{workload_name}/start",
            {"workload_name": workload_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"workload_name": "name"})
    def stop_workload(
        self_or_cls, workload_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Stop Workload

        **Permission Required:** `kelvin.permission.workload.update`.

        ``stopWorkload``: ``GET`` ``/api/v4/workloads/{workload_name}/stop``

        Parameters
        ----------
        workload_name : :obj:`str`, optional
            Workload name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/workloads/{workload_name}/stop",
            {"workload_name": workload_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"workload_name": "name"})
    def get_workload_telemetry(
        self_or_cls, workload_name: str, _dry_run: bool = False, _client: Any = None
    ) -> WorkloadTelemetry:
        """
        Get Workload Telemetry

        **Permission Required:** `kelvin.permission.workload.read`.

        ``getWorkloadTelemetry``: ``GET`` ``/api/v4/workloads/{workload_name}/telemetry/get``

        Parameters
        ----------
        workload_name : :obj:`str`, optional
            Workload name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/workloads/{workload_name}/telemetry/get",
            {"workload_name": workload_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": WorkloadTelemetry, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"workload_name": "name"})
    def undeploy_workload(
        self_or_cls, workload_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Undeploy Workload

        **Permission Required:** `kelvin.permission.workload.delete`.

        ``undeployWorkload``: ``POST`` ``/api/v4/workloads/{workload_name}/undeploy``

        Parameters
        ----------
        workload_name : :obj:`str`, optional
            Workload name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/workloads/{workload_name}/undeploy",
            {"workload_name": workload_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class WorkloadListPaginated(PaginatorDataModel[Workload]):
    """
    WorkloadListPaginated object.

    Parameters
    ----------
    data: Optional[List[Workload]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[Workload]] = None
    pagination: Optional[pagination.Pagination] = None


class ACP(DataModelBase):
    """
    ACP object.

    Parameters
    ----------
    cluster_name: Optional[str]
    created: Optional[datetime]
    kelvin_info: Optional[Dict[str, Any]]
    location: Optional[ACPLocationItem]
    manifests_scrape_interval: Optional[int]
    metadata: Optional[Dict[str, Any]]
    name: Optional[str]
    network_info: Optional[List[Dict[str, Any]]]
    node_name: Optional[str]
    status: Optional[ACPAndWorkloadStatus]
    status_scrape_interval: Optional[int]
    sync_scrape_interval: Optional[int]
    system_info: Optional[Dict[str, Any]]
    telemetry_scrape_interval: Optional[int]
    title: Optional[str]
    updated: Optional[datetime]

    """

    cluster_name: Optional[str] = None
    created: Optional[datetime] = None
    kelvin_info: Optional[Dict[str, Any]] = None
    location: Optional[ACPLocationItem] = None
    manifests_scrape_interval: Optional[int] = None
    metadata: Optional[Dict[str, Any]] = None
    name: Optional[str] = Field(None, max_length=64)
    network_info: Optional[List[Dict[str, Any]]] = None
    node_name: Optional[str] = None
    status: Optional[ACPAndWorkloadStatus] = None
    status_scrape_interval: Optional[int] = None
    sync_scrape_interval: Optional[int] = None
    system_info: Optional[Dict[str, Any]] = None
    telemetry_scrape_interval: Optional[int] = None
    title: Optional[str] = Field(None, max_length=64)
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def delete_acp(self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None) -> None:
        """
        Delete ACP

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteACP``: ``POST`` ``/api/v4/acps/{acp_name}/delete``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/delete",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def force_update_acp_edge_app_version(
        self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Force Update ACP Edge App Version

        **Permission Required:** `kelvin.permission.acp.read`.

        ``forceUpdateACPEdgeAppVersion``: ``POST`` ``/api/v4/acps/{acp_name}/edge-apps/version/force-update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/edge-apps/version/force-update",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_acp_edge_app_version(
        self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> ACPEdgeAppVersion:
        """
        Get ACP Edge App Version

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPEdgeAppVersion``: ``GET`` ``/api/v4/acps/{acp_name}/edge-apps/version/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/edge-apps/version/get",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ACPEdgeAppVersion,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_acp(
        self_or_cls,
        acp_name: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> ACP:
        """
        Get ACP

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACP``: ``GET`` ``/api/v4/acps/{acp_name}/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/get",
            {"acp_name": acp_name},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {"200": ACP, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def create_acp_metadata(
        self_or_cls,
        acp_name: str,
        data: Optional[Union[requests.ACPMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPMetadata:
        """
        Create ACP Metadata

        **Permission Required:** `kelvin.permission.acp.create`.

        ``createACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/create``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        data: requests.ACPMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/create",
            {"acp_name": acp_name},
            {},
            {},
            {},
            data,
            requests.ACPMetadataCreate,
            False,
            {"201": ACPMetadata, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def list_acp_metadata(
        self_or_cls,
        acp_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[ACPMetaDataItem], ACPMetadataListPaginated]:
        """
        List ACP Metadata

        **Pagination Sortable Columns:** `acp_metadata.name`, `acp_metadata.acp_name`

        **Permission Required:** `kelvin.permission.acp.read`.

        ``listACPMetadata``: ``GET`` ``/api/v4/acps/{acp_name}/metadata/list``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        search : :obj:`Sequence[str]`
            Search ACP Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/metadata/list",
            {"acp_name": acp_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": ACPMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def delete_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete ACP Metadata

        **Permission Required:** `kelvin.permission.acp.delete`.

        ``deleteACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/delete``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/delete",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> ACPMetadata:
        """
        Get ACP Metadata

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPMetadata``: ``GET`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/get",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": ACPMetadata, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def update_acp_metadata(
        self_or_cls,
        acp_name: str,
        acp_metadata_name: str,
        data: Optional[Union[requests.ACPMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPMetadata:
        """
        Update ACP Metadata

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateACPMetadata``: ``POST`` ``/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        acp_metadata_name : :obj:`str`, optional
            ACP Metadata name
        data: requests.ACPMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.ACPMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/metadata/{acp_metadata_name}/update",
            {"acp_name": acp_name, "acp_metadata_name": acp_metadata_name},
            {},
            {},
            {},
            data,
            requests.ACPMetadataUpdate,
            False,
            {
                "200": ACPMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def update_acp_status(
        self_or_cls,
        acp_name: str,
        data: Optional[Union[requests.ACPStatusUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACPStatus:
        """
        Update ACP Status

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateACPStatus``: ``POST`` ``/api/v4/acps/{acp_name}/status/update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        data: requests.ACPStatusUpdate, optional
        **kwargs:
            Extra parameters for requests.ACPStatusUpdate
              - node: str
              - workloads: list

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/status/update",
            {"acp_name": acp_name},
            {},
            {},
            {},
            data,
            requests.ACPStatusUpdate,
            False,
            {"200": ACPStatus, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_acp_telemetry(
        self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> ACPTelemetry:
        """
        Get ACP Telemetry

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getACPTelemetry``: ``GET`` ``/api/v4/acps/{acp_name}/telemetry/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/telemetry/get",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": ACPTelemetry, "400": ErrorMessages, "401": ErrorMessages, "500": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def update_acp(
        self_or_cls,
        acp_name: str,
        data: Optional[Union[requests.ACPUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> ACP:
        """
        Update ACP

        **Permission Required:** `kelvin.permission.acp.update`.

        ``updateACP``: ``POST`` ``/api/v4/acps/{acp_name}/update``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name
        data: requests.ACPUpdate, optional
        **kwargs:
            Extra parameters for requests.ACPUpdate
              - location: str
              - manifests_scrape_enabled: bool
              - manifests_scrape_interval: int
              - metrics_enabled: bool
              - metrics_scrape_interval: int
              - status_scrape_enabled: bool
              - status_scrape_interval: int
              - sync_scrape_interval: int
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/acps/{acp_name}/update",
            {"acp_name": acp_name},
            {},
            {},
            {},
            data,
            requests.ACPUpdate,
            False,
            {
                "200": ACP,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"acp_name": "name"})
    def get_workloads_manifests(
        self_or_cls, acp_name: str, _dry_run: bool = False, _client: Any = None
    ) -> ACPWorkloadManifests:
        """
        Get Workloads Manifests assigned to an ACP

        **Permission Required:** `kelvin.permission.acp.read`.

        ``getWorkloadsManifests``: ``GET`` ``/api/v4/acps/{acp_name}/workloads/manifests/get``

        Parameters
        ----------
        acp_name : :obj:`str`, optional
            ACP name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/acps/{acp_name}/workloads/manifests/get",
            {"acp_name": acp_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": ACPWorkloadManifests,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result


class Asset(DataModelBase):
    """
    Asset object.

    Parameters
    ----------
    asset_class_name: Optional[str]
    asset_class_title: Optional[str]
    asset_type_name: Optional[str]
    asset_type_title: Optional[str]
    created: Optional[datetime]
    device_type_name: Optional[str]
    device_type_title: Optional[str]
    failure_detection: Optional[FailureDetection]
    hierarchy: Optional[List[Asset]]
    location: Optional[AssetLocationItem]
    name: Optional[str]
    properties: Optional[List[AssetProperty]]
    status: Optional[AssetStatusItem]
    title: Optional[str]
    updated: Optional[datetime]

    """

    asset_class_name: Optional[str] = Field(None, max_length=64)
    asset_class_title: Optional[str] = Field(None, max_length=64)
    asset_type_name: Optional[str] = Field(None, max_length=64)
    asset_type_title: Optional[str] = None
    created: Optional[datetime] = None
    device_type_name: Optional[str] = None
    device_type_title: Optional[str] = None
    failure_detection: Optional[FailureDetection] = None
    hierarchy: Optional[List[Asset]] = None
    location: Optional[AssetLocationItem] = None
    name: Optional[str] = Field(None, max_length=64)
    properties: Optional[List[AssetProperty]] = None
    status: Optional[AssetStatusItem] = None
    title: Optional[str] = Field(None, max_length=64)
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset(
        self_or_cls,
        dry_run: Optional[bool] = None,
        data: Optional[Union[requests.AssetCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Asset:
        """
        Create Asset

        **Permission Required:** `kelvin.permission.asset.create`.

        ``createAsset``: ``POST`` ``/api/v4/assets/create``

        Parameters
        ----------
        dry_run : :obj:`bool`
            Dry Run
        data: requests.AssetCreate, optional
        **kwargs:
            Extra parameters for requests.AssetCreate
              - asset_type_name: str
              - device_type_name: str
              - failure_detection: str
              - hierarchy: list
              - location: str
              - name: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/create",
            {},
            {"dry_run": dry_run},
            {},
            {},
            data,
            requests.AssetCreate,
            False,
            {
                "201": Asset,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "412": errors.ErrorMessage,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_asset_hierarchies(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> AssetHierarchyListFull:
        """
        List Asset Hierarchies

        **Permission Required:** `kelvin.permission.asset.read`.

        ``listAssetHierarchies``: ``GET`` ``/api/v4/assets/hierarchy/list``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/hierarchy/list",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AssetHierarchyListFull, "400": errors.ErrorMessage, "401": errors.ErrorMessage},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        asset_type_name: Optional[Sequence[str]] = None,
        status_state: Optional[Sequence[str]] = None,
        device_type_name: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetItem], AssetListPaginated]:
        """
        List Asset

        **Pagination Sortable Columns:** `asset.name`, `asset.title`, `asset.created`, `asset.updated`, `asset_type.name`, `asset_type.title`, `status_state`, `status_last_seen`

        **Permission Required:** `kelvin.permission.asset.read`.

        ``listAsset``: ``GET`` ``/api/v4/assets/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search assets by name or title
        asset_type_name : :obj:`Sequence[str]`
            Filter assets by asset type
        status_state : :obj:`Sequence[str]`
            Filter assets by status state [`online`, `offline`,
            `partially_online`, `unknown`]
        device_type_name : :obj:`Sequence[str]`
            Filter assets by device type
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/list",
            {},
            {
                "search": search,
                "asset_type_name": asset_type_name,
                "status_state": status_state,
                "device_type_name": device_type_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetListPaginated,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def get_asset_status_count(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> AssetStatusCount:
        """
        Get Asset Status Count

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetStatusCount``: ``GET`` ``/api/v4/assets/status/count/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/status/count/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetStatusCount,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_asset_statuses(cls, _dry_run: bool = False, _client: Any = None) -> AssetStatus:
        """
        Get Asset Statuses

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetStatuses``: ``GET`` ``/api/v4/assets/status/current/get``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/status/current/get",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetStatus,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_template(
        self_or_cls,
        data: Optional[Union[requests.AssetTemplateCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTemplate:
        """
        Create Asset Template

        **Permission Required:** `kelvin.permission.asset.create`.

        ``createAssetTemplate``: ``POST`` ``/api/v4/assets/templates/create``

        Parameters
        ----------
        data: requests.AssetTemplateCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTemplateCreate
              - asset_type_name: str
              - description: str
              - device_type_name: str
              - failure_detection: str
              - hierarchy: list
              - hints: dict
              - image_url: str
              - name: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/templates/create",
            {},
            {},
            {},
            {},
            data,
            requests.AssetTemplateCreate,
            False,
            {"201": AssetTemplate, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_asset_template(
        cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        asset_type_names: Optional[Sequence[str]] = None,
        device_type_names: Optional[Sequence[str]] = None,
        asset_class_names: Optional[Sequence[str]] = None,
        owner: Optional[str] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetTemplate], AssetTemplateListPaginated]:
        """
        List Asset Template

        **Pagination Sortable Columns:** `asset_template.name`, `asset_template.title`, `asset_template.created`, `asset_template.updated`, `asset_template.asset_type_name`, `asset_template.device_type_name`

        **Permission Required:** `kelvin.permission.asset.read`.

        ``listAssetTemplate``: ``GET`` ``/api/v4/assets/templates/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search Asset Template by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        asset_type_names : :obj:`Sequence[str]`
            Asset Type Names
        device_type_names : :obj:`Sequence[str]`
            Device Type Names
        asset_class_names : :obj:`Sequence[str]`
            Asset Class Names
        owner : :obj:`str`
            Owner

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/templates/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "asset_type_names": asset_type_names,
                "device_type_names": device_type_names,
                "asset_class_names": asset_class_names,
                "owner": owner,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTemplateListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @classmethod
    def delete_asset_template(
        cls, asset_template_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset Template

        **Permission Required:** `kelvin.permission.asset.delete`.

        ``deleteAssetTemplate``: ``POST`` ``/api/v4/assets/templates/{asset_template_name}/delete``

        Parameters
        ----------
        asset_template_name : :obj:`str`, optional
            Asset Template Name

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/assets/templates/{asset_template_name}/delete",
            {"asset_template_name": asset_template_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_asset_template(
        cls, asset_template_name: str, _dry_run: bool = False, _client: Any = None
    ) -> AssetTemplate:
        """
        Get Asset Template

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetTemplate``: ``GET`` ``/api/v4/assets/templates/{asset_template_name}/get``

        Parameters
        ----------
        asset_template_name : :obj:`str`, optional
            Asset Template Name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/templates/{asset_template_name}/get",
            {"asset_template_name": asset_template_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetTemplate,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_asset_template(
        self_or_cls,
        asset_template_name: str,
        data: Optional[Union[requests.AssetTemplateUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTemplate:
        """
        Update Asset Template

        **Permission Required:** `kelvin.permission.asset.update`.

        ``updateAssetTemplate``: ``POST`` ``/api/v4/assets/templates/{asset_template_name}/update``

        Parameters
        ----------
        asset_template_name : :obj:`str`, optional
            Asset Template Name
        data: requests.AssetTemplateUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTemplateUpdate
              - asset_type_name: str
              - description: str
              - device_type_name: str
              - failure_detection: str
              - hierarchy: list
              - hints: dict
              - image_url: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/templates/{asset_template_name}/update",
            {"asset_template_name": asset_template_name},
            {},
            {},
            {},
            data,
            requests.AssetTemplateUpdate,
            False,
            {
                "200": AssetTemplate,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_type(
        self_or_cls,
        data: Optional[Union[requests.AssetTypeCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetType:
        """
        Create Asset Type

        **Permission Required:** `kelvin.permission.asset_type.create`.

        ``createAssetType``: ``POST`` ``/api/v4/assets/types/create``

        Parameters
        ----------
        data: requests.AssetTypeCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeCreate
              - asset_class_name: str
              - name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/create",
            {},
            {},
            {},
            {},
            data,
            requests.AssetTypeCreate,
            False,
            {"201": AssetType, "400": ErrorMessages, "401": ErrorMessages, "409": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset_type(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        asset_class_name: Optional[Sequence[str]] = None,
        metadata: Optional[Sequence[str]] = None,
        metadata_filter: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        owner: Optional[str] = None,
        device_type_name: Optional[Sequence[str]] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetType], AssetTypeListPaginated]:
        """
        List Asset Type

        **Pagination Sortable Columns:** `asset_type.name`, `asset_type.title`, `asset_type.created`, `asset_type.updated`, `asset_class.name`, `asset_class.title`, `asset_type.owner`

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``listAssetType``: ``GET`` ``/api/v4/assets/types/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search asset type by name or title
        asset_class_name : :obj:`Sequence[str]`
            Filter asset type by asset class
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response
        metadata_filter : :obj:`Sequence[str]`
            Dot separated string to filter by metadata values Same <metadata_name>
            will result in OR queries. Different <metadata_name> will result in
            AND queries. Example:
            <metadata_name>.<payload_key_to_search>=<payload_value_to_search>
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        owner : :obj:`str`
            Filter Asset Class by owner. Possible Values: kelvin/custom
        device_type_name : :obj:`Sequence[str]`
            Filter asset type by asset class device type name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/list",
            {},
            {
                "search": search,
                "asset_class_name": asset_class_name,
                "metadata": metadata,
                "metadata_filter": metadata_filter,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "owner": owner,
                "device_type_name": device_type_name,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTypeListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_asset_type(
        self_or_cls, asset_type_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset Type

        **Permission Required:** `kelvin.permission.asset_type.delete`.

        ``deleteAssetType``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/delete``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/delete",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_asset_type(
        self_or_cls,
        asset_type_name: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> AssetType:
        """
        Get Asset Type

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``getAssetType``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/get``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/get",
            {"asset_type_name": asset_type_name},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {"200": AssetType, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        data: Optional[Union[requests.AssetTypeMetadataCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTypeMetadata:
        """
        Create Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.create`.

        ``createAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/create``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        data: requests.AssetTypeMetadataCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeMetadataCreate
              - name: str
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/create",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            data,
            requests.AssetTypeMetadataCreate,
            False,
            {
                "201": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetTypeMetadata], AssetTypeMetadataListPaginated]:
        """
        List Asset Type Metadata

        **Pagination Sortable Columns:** `asset_type_metadata.name`

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``listAssetTypeMetadata``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/metadata/list``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        search : :obj:`Sequence[str]`
            Search Asset Type Metadata by name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/metadata/list",
            {"asset_type_name": asset_type_name},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTypeMetadataListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate()
    def delete_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> None:
        """
        Delete Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.delete`.

        ``deleteAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/delete``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/delete",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def get_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> AssetTypeMetadata:
        """
        Get Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.read`.

        ``getAssetTypeMetadata``: ``GET`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/get``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/get",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_asset_type_metadata(
        self_or_cls,
        asset_type_name: str,
        asset_type_metadata_name: str,
        data: Optional[Union[requests.AssetTypeMetadataUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTypeMetadata:
        """
        Update Asset Type Metadata

        **Permission Required:** `kelvin.permission.asset_type.update`.

        ``updateAssetTypeMetadata``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/update``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        asset_type_metadata_name : :obj:`str`, optional
            Asset Type Metadata name
        data: requests.AssetTypeMetadataUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeMetadataUpdate
              - payload: dict

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/metadata/{asset_type_metadata_name}/update",
            {
                "asset_type_name": asset_type_name,
                "asset_type_metadata_name": asset_type_metadata_name,
            },
            {},
            {},
            {},
            data,
            requests.AssetTypeMetadataUpdate,
            False,
            {
                "200": AssetTypeMetadata,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def update_asset_type(
        self_or_cls,
        asset_type_name: str,
        data: Optional[Union[requests.AssetTypeUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetType:
        """
        Update Asset Type

        **Permission Required:** `kelvin.permission.asset_type.update`.

        ``updateAssetType``: ``POST`` ``/api/v4/assets/types/{asset_type_name}/update``

        Parameters
        ----------
        asset_type_name : :obj:`str`, optional
            Asset Type name
        data: requests.AssetTypeUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTypeUpdate
              - asset_class_name: str
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/types/{asset_type_name}/update",
            {"asset_type_name": asset_type_name},
            {},
            {},
            {},
            data,
            requests.AssetTypeUpdate,
            False,
            {
                "200": AssetType,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
                "409": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_name": "name"})
    def delete_asset(
        self_or_cls, asset_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset

        **Permission Required:** `kelvin.permission.asset.delete`.

        ``deleteAsset``: ``POST`` ``/api/v4/assets/{asset_name}/delete``

        Parameters
        ----------
        asset_name : :obj:`str`, optional
            Asset name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/{asset_name}/delete",
            {"asset_name": asset_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
                "412": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_name": "name"})
    def get_asset(
        self_or_cls,
        asset_name: str,
        metadata: Optional[Sequence[str]] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Asset:
        """
        Get Asset

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAsset``: ``GET`` ``/api/v4/assets/{asset_name}/get``

        Parameters
        ----------
        asset_name : :obj:`str`, optional
            Asset name
        metadata : :obj:`Sequence[str]`
            Inject metadata into the response

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/{asset_name}/get",
            {"asset_name": asset_name},
            {"metadata": metadata},
            {},
            {},
            None,
            None,
            False,
            {
                "200": Asset,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_name": "name"})
    def get_asset_hierarchy(
        self_or_cls, asset_name: str, _dry_run: bool = False, _client: Any = None
    ) -> Asset:
        """
        Get Asset Hierarchy

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetHierarchy``: ``GET`` ``/api/v4/assets/{asset_name}/hierarchy/get``

        Parameters
        ----------
        asset_name : :obj:`str`, optional
            Asset name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/{asset_name}/hierarchy/get",
            {"asset_name": asset_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": Asset,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_name": "name"})
    def update_asset(
        self_or_cls,
        asset_name: str,
        dry_run: Optional[bool] = None,
        data: Optional[Union[requests.AssetUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Asset:
        """
        Update Asset

        **Permission Required:** `kelvin.permission.asset.update`.

        ``updateAsset``: ``POST`` ``/api/v4/assets/{asset_name}/update``

        Parameters
        ----------
        asset_name : :obj:`str`, optional
            Asset name
        dry_run : :obj:`bool`
            Dry Run
        data: requests.AssetUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetUpdate
              - asset_type_name: str
              - failure_detection: str
              - hierarchy: list
              - location: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/{asset_name}/update",
            {"asset_name": asset_name},
            {"dry_run": dry_run},
            {},
            {},
            data,
            requests.AssetUpdate,
            False,
            {
                "200": Asset,
                "400": errors.ErrorMessage,
                "401": errors.ErrorMessage,
                "404": errors.ErrorMessage,
                "412": errors.ErrorMessage,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class AssetHierarchyListFull(DataModelBase):
    """
    AssetHierarchyListFull object.

    Parameters
    ----------
    data: Optional[List[Asset]]

    """

    data: Optional[List[Asset]] = None

    @classmethod
    def list_asset_hierarchies(
        cls, _dry_run: bool = False, _client: Any = None
    ) -> AssetHierarchyListFull:
        """
        List Asset Hierarchies

        **Permission Required:** `kelvin.permission.asset.read`.

        ``listAssetHierarchies``: ``GET`` ``/api/v4/assets/hierarchy/list``

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/assets/hierarchy/list",
            {},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": AssetHierarchyListFull, "400": errors.ErrorMessage, "401": errors.ErrorMessage},
            False,
            _dry_run,
        )
        return result


class AssetTemplateHierarchy(DataModelBase):
    """
    AssetTemplateHierarchy object.

    Parameters
    ----------
    asset_type_name: Optional[str]
    device_type_name: Optional[str]
    device_type_title: Optional[str]
    failure_detection: Optional[FailureDetection]
    from_template_name: Optional[str]
    hierarchy: Optional[List[AssetTemplateHierarchy]]
    hints: Optional[Dict[str, Any]]
    properties: Optional[List[AssetTemplateProperties]]
    title: Optional[str]

    """

    asset_type_name: Optional[str] = None
    device_type_name: Optional[str] = None
    device_type_title: Optional[str] = None
    failure_detection: Optional[FailureDetection] = None
    from_template_name: Optional[str] = None
    hierarchy: Optional[List[AssetTemplateHierarchy]] = None
    hints: Optional[Dict[str, Any]] = None
    properties: Optional[List[AssetTemplateProperties]] = None
    title: Optional[str] = None


class FailureClass(DataModelBase):
    """
    FailureClass object.

    Parameters
    ----------
    created: Optional[datetime]
    device_type_name: Optional[str]
    name: Optional[str]
    properties: Optional[List[FailureClassProperty]]
    title: Optional[str]
    updated: Optional[datetime]

    """

    created: Optional[datetime] = None
    device_type_name: Optional[str] = None
    name: Optional[str] = None
    properties: Optional[List[FailureClassProperty]] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def list_failure_classes(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        device_type_name: Optional[str] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[FailureClass], FailureClassPaginated]:
        """
        List Failure Classes

        **Pagination Sortable Columns:** `failure_class.name`, `failure_class.title`, `failure_class.device_type_name`, `failure_class.created`, `failure_class.updated`

        **Permission Required:** `kelvin.permission.failure_detection.read`.

        ``listFailureClasses``: ``GET`` ``/api/v4/failure-detection/classes/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search failure classes by name or title
        device_type_name : :obj:`str`
            Filter failure classes by device type name
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/failure-detection/classes/list",
            {},
            {
                "search": search,
                "device_type_name": device_type_name,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": FailureClassPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"failure_class_name": "name"})
    def get_failure_class(
        self_or_cls, failure_class_name: str, _dry_run: bool = False, _client: Any = None
    ) -> FailureClass:
        """
        Get Failure Class

        **Permission Required:** `kelvin.permission.failure_detection.read`.

        ``getFailureClass``: ``GET`` ``/api/v4/failure-detection/classes/{failure_class_name}/get``

        Parameters
        ----------
        failure_class_name : :obj:`str`, optional
            Failure Class Name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/failure-detection/classes/{failure_class_name}/get",
            {"failure_class_name": failure_class_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": FailureClass, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result


class FailureClassPaginated(PaginatorDataModel[FailureClass]):
    """
    FailureClassPaginated object.

    Parameters
    ----------
    data: Optional[List[FailureClass]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[FailureClass]] = None
    pagination: Optional[pagination.Pagination] = None


class IntegrationList(PaginatorDataModel[IntegrationListItem]):
    """
    IntegrationList object.

    Parameters
    ----------
    data: Optional[List[IntegrationListItem]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[IntegrationListItem]] = None
    pagination: Optional[pagination.Pagination] = None

    @classmethod
    def create_aws_region(
        cls,
        data: Optional[Union[requests.IntegrationAwsRegionCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> IntegrationAwsRegion:
        """
        Create AWS Region

        **Permission Required:** `kelvin.permission.integration.create`.

        ``createAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/create``

        Parameters
        ----------
        data: requests.IntegrationAwsRegionCreate, optional
        **kwargs:
            Extra parameters for requests.IntegrationAwsRegionCreate
              - name: str
              - title: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/create",
            {},
            {},
            {},
            {},
            data,
            requests.IntegrationAwsRegionCreate,
            False,
            {"201": IntegrationAwsRegion, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_aws_regions(
        cls,
        names: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> IntegrationAwsRegionList:
        """
        List AWS Regions

        **Pagination Sortable Columns:** `aws_regions.name`, `aws_regions.title`, `aws_regions.created`, `aws_regions.updated`

        **Permission Required:** `kelvin.permission.integration.read`.

        ``listAWSRegions``: ``GET`` ``/api/v4/integrations/aws/regions/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter Regions by name
        search : :obj:`Sequence[str]`
            Search Regions by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/integrations/aws/regions/list",
            {},
            {
                "names": names,
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": IntegrationAwsRegionList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def delete_aws_region(cls, name: str, _dry_run: bool = False, _client: Any = None) -> None:
        """
        Delete AWS Region

        **Permission Required:** `kelvin.permission.integration.delete`.

        ``deleteAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/{name}/delete``

        Parameters
        ----------
        name : :obj:`str`, optional
            Region name

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/{name}/delete",
            {"name": name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def update_aws_region(
        cls,
        name: str,
        data: Optional[Union[requests.IntegrationAwsRegionUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> IntegrationAwsRegion:
        """
        Update AWS Region

        **Permission Required:** `kelvin.permission.integration.update`.

        ``updateAWSRegion``: ``POST`` ``/api/v4/integrations/aws/regions/{name}/update``

        Parameters
        ----------
        name : :obj:`str`, optional
            Region name
        data: requests.IntegrationAwsRegionUpdate, optional
        **kwargs:
            Extra parameters for requests.IntegrationAwsRegionUpdate
              - name: str
              - title: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/aws/regions/{name}/update",
            {"name": name},
            {},
            {},
            {},
            data,
            requests.IntegrationAwsRegionUpdate,
            False,
            {
                "200": IntegrationAwsRegion,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def create_integration(
        cls,
        data: Optional[Union[requests.IntegrationCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Integration:
        """
        Create Integration

        **Permission Required:** `kelvin.permission.integration.create`.

        ``createIntegration``: ``POST`` ``/api/v4/integrations/create``

        Parameters
        ----------
        data: requests.IntegrationCreate, optional
        **kwargs:
            Extra parameters for requests.IntegrationCreate
              - configuration: str
              - enable: bool
              - name: str
              - title: str
              - type: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/create",
            {},
            {},
            {},
            {},
            data,
            requests.IntegrationCreate,
            False,
            {"201": Integration, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @classmethod
    def list_integrations(
        cls,
        names: Optional[Sequence[str]] = None,
        types: Optional[Sequence[str]] = None,
        search: Optional[Sequence[str]] = None,
        enable: Optional[bool] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> IntegrationList:
        """
        List Integrations

        **Pagination Sortable Columns:** `integration.name`, `integration.title`, `integration.type`, `integration.enable`, `integration.state`, `integration.last_seen`, `integration.created`, `integration.updated`

        **Permission Required:** `kelvin.permission.integration.read`.

        ``listIntegrations``: ``GET`` ``/api/v4/integrations/list``

        Parameters
        ----------
        names : :obj:`Sequence[str]`
            Filter Integrations by name
        types : :obj:`Sequence[str]`
            Filter Integrations by type
        search : :obj:`Sequence[str]`
            Search Integrations by name or title
        enable : :obj:`bool`
            Filter by enable Integrations. Possible Values: true/false. Default:
            none
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/integrations/list",
            {},
            {
                "names": names,
                "types": types,
                "search": search,
                "enable": enable,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": IntegrationList, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def delete_integration(cls, name: str, _dry_run: bool = False, _client: Any = None) -> None:
        """
        Delete Integration

        **Permission Required:** `kelvin.permission.integration.delete`.

        ``deleteIntegration``: ``POST`` ``/api/v4/integrations/{name}/delete``

        Parameters
        ----------
        name : :obj:`str`, optional
            Integration name

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/{name}/delete",
            {"name": name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": None, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def get_integration(cls, name: str, _dry_run: bool = False, _client: Any = None) -> Integration:
        """
        Get Integration

        **Permission Required:** `kelvin.permission.integration.read`.

        ``getIntegration``: ``GET`` ``/api/v4/integrations/{name}/get``

        Parameters
        ----------
        name : :obj:`str`, optional
            Integration name

        """

        result = cls._make_request(
            _client,
            "get",
            "/api/v4/integrations/{name}/get",
            {"name": name},
            {},
            {},
            {},
            None,
            None,
            False,
            {"200": Integration, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
        )
        return result

    @classmethod
    def update_integration(
        cls,
        name: str,
        data: Optional[Union[requests.IntegrationUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> Integration:
        """
        Update Integration

        **Permission Required:** `kelvin.permission.integration.update`.

        ``updateIntegration``: ``POST`` ``/api/v4/integrations/{name}/update``

        Parameters
        ----------
        name : :obj:`str`, optional
            Integration name
        data: requests.IntegrationUpdate, optional
        **kwargs:
            Extra parameters for requests.IntegrationUpdate
              - configuration: str
              - enable: bool
              - name: str
              - title: str

        """

        result = cls._make_request(
            _client,
            "post",
            "/api/v4/integrations/{name}/update",
            {"name": name},
            {},
            {},
            {},
            data,
            requests.IntegrationUpdate,
            False,
            {"200": Integration, "400": ErrorMessages, "401": ErrorMessages, "404": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result


class AssetTemplate(DataModelBase):
    """
    AssetTemplate object.

    Parameters
    ----------
    asset_class_name: Optional[str]
    asset_class_title: Optional[str]
    asset_type_name: Optional[str]
    asset_type_title: Optional[str]
    created: Optional[datetime]
    description: Optional[str]
    device_type_name: Optional[str]
    device_type_title: Optional[str]
    failure_detection: Optional[FailureDetection]
    hierarchy: Optional[List[AssetTemplateHierarchy]]
    hints: Optional[Dict[str, Any]]
    image_url: Optional[str]
    name: Optional[str]
    owner: Optional[str]
    properties: Optional[List[AssetTemplateProperties]]
    title: Optional[str]
    updated: Optional[datetime]

    """

    asset_class_name: Optional[str] = None
    asset_class_title: Optional[str] = None
    asset_type_name: Optional[str] = None
    asset_type_title: Optional[str] = None
    created: Optional[datetime] = None
    description: Optional[str] = None
    device_type_name: Optional[str] = None
    device_type_title: Optional[str] = None
    failure_detection: Optional[FailureDetection] = None
    hierarchy: Optional[List[AssetTemplateHierarchy]] = None
    hints: Optional[Dict[str, Any]] = None
    image_url: Optional[str] = None
    name: Optional[str] = None
    owner: Optional[str] = None
    properties: Optional[List[AssetTemplateProperties]] = None
    title: Optional[str] = None
    updated: Optional[datetime] = None

    @instance_classmethod
    @DataModelBase.translate()
    def create_asset_template(
        self_or_cls,
        data: Optional[Union[requests.AssetTemplateCreate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTemplate:
        """
        Create Asset Template

        **Permission Required:** `kelvin.permission.asset.create`.

        ``createAssetTemplate``: ``POST`` ``/api/v4/assets/templates/create``

        Parameters
        ----------
        data: requests.AssetTemplateCreate, optional
        **kwargs:
            Extra parameters for requests.AssetTemplateCreate
              - asset_type_name: str
              - description: str
              - device_type_name: str
              - failure_detection: str
              - hierarchy: list
              - hints: dict
              - image_url: str
              - name: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/templates/create",
            {},
            {},
            {},
            {},
            data,
            requests.AssetTemplateCreate,
            False,
            {"201": AssetTemplate, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
            kwargs,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate()
    def list_asset_template(
        self_or_cls,
        search: Optional[Sequence[str]] = None,
        page_size: Optional[int] = None,
        starting_after: Optional[str] = None,
        ending_before: Optional[str] = None,
        sort_by: Optional[str] = None,
        sort_asc: Optional[bool] = None,
        asset_type_names: Optional[Sequence[str]] = None,
        device_type_names: Optional[Sequence[str]] = None,
        asset_class_names: Optional[Sequence[str]] = None,
        owner: Optional[str] = None,
        fetch: bool = True,
        _dry_run: bool = False,
        _client: Any = None,
    ) -> Union[Sequence[AssetTemplate], AssetTemplateListPaginated]:
        """
        List Asset Template

        **Pagination Sortable Columns:** `asset_template.name`, `asset_template.title`, `asset_template.created`, `asset_template.updated`, `asset_template.asset_type_name`, `asset_template.device_type_name`

        **Permission Required:** `kelvin.permission.asset.read`.

        ``listAssetTemplate``: ``GET`` ``/api/v4/assets/templates/list``

        Parameters
        ----------
        search : :obj:`Sequence[str]`
            Search Asset Template by name or title
        page_size : :obj:`int`
            Number of objects to be returned, between 1 and 1000. Default: 20
        starting_after : :obj:`str`
            Lower bound cursor for use in pagination
        ending_before : :obj:`str`
            Upper bound cursor for use in pagination
        sort_by : :obj:`str`
            Parameter to sort by. Must be one of the sortable columns
        sort_asc : :obj:`bool`
            Order of sorting is Ascending. Possible Values: true/false. Default:
            true
        asset_type_names : :obj:`Sequence[str]`
            Asset Type Names
        device_type_names : :obj:`Sequence[str]`
            Device Type Names
        asset_class_names : :obj:`Sequence[str]`
            Asset Class Names
        owner : :obj:`str`
            Owner

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/templates/list",
            {},
            {
                "search": search,
                "page_size": page_size,
                "starting_after": starting_after,
                "ending_before": ending_before,
                "sort_by": sort_by,
                "sort_asc": sort_asc,
                "asset_type_names": asset_type_names,
                "device_type_names": device_type_names,
                "asset_class_names": asset_class_names,
                "owner": owner,
            },
            {},
            {},
            None,
            None,
            False,
            {"200": AssetTemplateListPaginated, "400": ErrorMessages, "401": ErrorMessages},
            False,
            _dry_run,
        )
        return result.fetch() if fetch and not _dry_run else result

    @instance_classmethod
    @DataModelBase.translate({"asset_template_name": "name"})
    def delete_asset_template(
        self_or_cls, asset_template_name: str, _dry_run: bool = False, _client: Any = None
    ) -> None:
        """
        Delete Asset Template

        **Permission Required:** `kelvin.permission.asset.delete`.

        ``deleteAssetTemplate``: ``POST`` ``/api/v4/assets/templates/{asset_template_name}/delete``

        Parameters
        ----------
        asset_template_name : :obj:`str`, optional
            Asset Template Name

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/templates/{asset_template_name}/delete",
            {"asset_template_name": asset_template_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": None,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "403": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_template_name": "name"})
    def get_asset_template(
        self_or_cls, asset_template_name: str, _dry_run: bool = False, _client: Any = None
    ) -> AssetTemplate:
        """
        Get Asset Template

        **Permission Required:** `kelvin.permission.asset.read`.

        ``getAssetTemplate``: ``GET`` ``/api/v4/assets/templates/{asset_template_name}/get``

        Parameters
        ----------
        asset_template_name : :obj:`str`, optional
            Asset Template Name

        """

        result = self_or_cls._make_request(
            _client,
            "get",
            "/api/v4/assets/templates/{asset_template_name}/get",
            {"asset_template_name": asset_template_name},
            {},
            {},
            {},
            None,
            None,
            False,
            {
                "200": AssetTemplate,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
        )
        return result

    @instance_classmethod
    @DataModelBase.translate({"asset_template_name": "name"})
    def update_asset_template(
        self_or_cls,
        asset_template_name: str,
        data: Optional[Union[requests.AssetTemplateUpdate, Mapping[str, Any]]] = None,
        _dry_run: bool = False,
        _client: Any = None,
        **kwargs: Any,
    ) -> AssetTemplate:
        """
        Update Asset Template

        **Permission Required:** `kelvin.permission.asset.update`.

        ``updateAssetTemplate``: ``POST`` ``/api/v4/assets/templates/{asset_template_name}/update``

        Parameters
        ----------
        asset_template_name : :obj:`str`, optional
            Asset Template Name
        data: requests.AssetTemplateUpdate, optional
        **kwargs:
            Extra parameters for requests.AssetTemplateUpdate
              - asset_type_name: str
              - description: str
              - device_type_name: str
              - failure_detection: str
              - hierarchy: list
              - hints: dict
              - image_url: str
              - properties: list
              - title: str

        """

        result = self_or_cls._make_request(
            _client,
            "post",
            "/api/v4/assets/templates/{asset_template_name}/update",
            {"asset_template_name": asset_template_name},
            {},
            {},
            {},
            data,
            requests.AssetTemplateUpdate,
            False,
            {
                "200": AssetTemplate,
                "400": ErrorMessages,
                "401": ErrorMessages,
                "404": ErrorMessages,
            },
            False,
            _dry_run,
            kwargs,
        )
        return result


class AssetTemplateListPaginated(PaginatorDataModel[AssetTemplate]):
    """
    AssetTemplateListPaginated object.

    Parameters
    ----------
    data: Optional[List[AssetTemplate]]
    pagination: Optional[pagination.Pagination]

    """

    data: Optional[List[AssetTemplate]] = None
    pagination: Optional[pagination.Pagination] = None


AssetHierarchyItem.update_forward_refs()
Asset.update_forward_refs()
AssetTemplateHierarchy.update_forward_refs()
